from mozilla_django_oidc.auth import OIDCAuthenticationBackend

from apps.MySU.models import AssociationMembership
from apps.MySU.models import GroupMembership
from apps.MySU.models import Profile
from apps.MySU.models import User


class MySUOidcBackend(OIDCAuthenticationBackend):
    def verify_claims(self, claims):
        """Verify the provided claims to decide if authentication should be allowed."""
        username = claims.get('eduperson_principal_name', None)
        student_number = claims.get('uids', None)
        return username is not None and student_number is not None and len(student_number) > 0

    def filter_users_by_claims(self, claims):
        username = claims.get('eduperson_principal_name')
        if not username:
            return self.UserModel.objects.none()

        try:
            return User.objects.filter(username=username)
        except User.DoesNotExist:
            return self.UserModel.objects.none()

    def create_user(self, claims):
        username = claims.get('eduperson_principal_name')
        email = claims.get('email', None)
        student_number = claims.get('uids')[0]
        user = self.UserModel.objects.create_user(
            username=username,
            email=email,
            student_number=student_number,
            surfconext_signin=True
        )
        found_profiles = Profile.objects.filter(student_number=student_number)
        # A user is linked to various object through its profiles
        # When creating a user we need to properly set those
        # In practice only groupmemberships and associationmemberships exist for userless profiles
        # so only those need linking
        for profile in found_profiles:
            # find all linked group memberships
            found_groupmemberships = GroupMembership.objects.filter(profile=profile)
            # update each one of them
            for groupmembership in found_groupmemberships:
                groupmembership.user = user
            # save to the database
            GroupMembership.objects.bulk_update(found_groupmemberships, ["user"])

            # find all linked association memberships
            found_association_memberships = AssociationMembership.objects.filter(profile=profile)
            # update each one of them
            for association_membership in found_association_memberships:
                association_membership.user = user
            # save to the database
            AssociationMembership.objects.bulk_update(found_association_memberships, ["user"])

            # finish with linking the profile
            profile.user = user
        Profile.objects.bulk_update(found_profiles, ["user"])

        return user
