from django.contrib.auth.models import Permission
from rest_framework import serializers


class PermissionField(serializers.SlugRelatedField):
    """
    The permissions shown in the Clickable API provided by DRF are shown as slugs because this is a SlugRelatedField.
    This does not help if you want to select the board permission. To be able to find the relevant permissions this
    field overwrites the to_representation to shown its codename instead of slug.
    """
    def to_representation(self, obj):
        if isinstance(obj, Permission):
            return getattr(obj, 'codename')
        else:
            return getattr(obj, 'permission').codename
