from django import forms
from url_filter import filters


class ChoiceFilter(filters.CallableFilter):
    """
    All the endpoints can be filtered. This however does not work for the MultiSelectFields. This ChoiceFilter
    overwrites the default CallableFilter, such that queries like ?status="Accepted" and ?status__in=["Accepted","Rejected"]
    work.
    """
    def __init__(self, choices, *args, **kwargs):
        form_field = forms.ChoiceField(choices=choices)
        super().__init__(form_field, *args, **kwargs)

    def filter_exact_for_django(self, queryset, spec):
        value = [spec.value]
        f = queryset.filter if not spec.is_negated else queryset.exclude
        return f(status=value)

    def filter_in_for_django(self, queryset, spec):
        values = [spec.value.split(",")]
        f = queryset.filter if not spec.is_negated else queryset.exclude
        return f(status__in=values)
