import calendar

from django import forms
from django.core.validators import ValidationError
from django.forms.widgets import MultiWidget
from django.forms.widgets import Select
from django.forms.widgets import TextInput

from MySU.objects import YearlessDate

DAY_CHOICES = tuple([('', '---------')] + [(i, i) for i in range(1, 32)])
MONTH_CHOICES = tuple([('', '---------')] + [(i, calendar.month_name[i]) for i in range(1, 13)])


class YearlessDateSelect(MultiWidget):
    def __init__(self, *args, **kwargs):
        widgets = (
            Select(attrs={'class': 'select-dateinyear-month'}, choices=MONTH_CHOICES),
            Select(attrs={'class': 'select-dateinyear-day'}, choices=DAY_CHOICES),
        )
        super(YearlessDateSelect, self).__init__(widgets=widgets, *args, **kwargs)

    def decompress(self, value):
        if value is None:
            return [None, None]
        return [value.month, value.day]


class YearlessDateForm(forms.Field):
    widget = YearlessDateSelect

    def clean(self, value):
        if value == ['', '']:
            # If the values are both None, trigger the default validation for null
            super(YearlessDateForm, self).clean(None)
        else:
            try:
                return YearlessDate(*value)
            except Exception:
                raise ValidationError('Invalid date.')
