from guardian.shortcuts import get_objects_for_user
from rest_framework import status
from rest_framework.response import Response
from rest_framework_extensions.bulk_operations.mixins import ListDestroyModelMixin


class BulkDestroyModelMixin(ListDestroyModelMixin):
    def destroy_bulk(self, request, *args, **kwargs):
        is_valid, errors = self.is_valid_bulk_operation()
        if is_valid:
            queryset = self.filter_queryset(self.get_queryset())
            # expanded ListDestroyModelMixin to allow for a validation check
            is_valid, errors = self.validate_delete_bulk(queryset)
            if not is_valid:
                return Response(errors, status=status.HTTP_400_BAD_REQUEST)
            self.pre_delete_bulk(queryset)
            queryset.delete()
            self.post_delete_bulk(queryset)
            return Response(status=status.HTTP_204_NO_CONTENT)
        else:
            return Response(errors, status=status.HTTP_400_BAD_REQUEST)

    def pre_delete_bulk(self, queryset):
        user = self.request.user
        associations_for_which_the_user_is_board = get_objects_for_user(user, "MySU.board")
        return queryset.filter(association__in=associations_for_which_the_user_is_board)

    def validate_delete_bulk(self, queryset):
        return True, {}
