from __future__ import unicode_literals

from collections import OrderedDict

from django.urls.exceptions import NoReverseMatch
from rest_framework import views
from rest_framework.response import Response
from rest_framework.reverse import reverse
from rest_framework.routers import DefaultRouter

from apps.MySU.viewsets import DjangoGroupViewSet
from apps.MySU.viewsets import GroupObjectPermissionViewSet
from apps.MySU.viewsets import PermissionViewSet
from MySU import urls


class MySURouter(DefaultRouter):
    ADMIN_ONLY_VIEWSETS = [PermissionViewSet, DjangoGroupViewSet, GroupObjectPermissionViewSet]

    def __init__(self, *args, **kwargs):
        super(DefaultRouter, self).__init__(*args, **kwargs)  # pylint: disable=bad-super-call
        self.trailing_slash = '/?'

    def get_api_root_view(self, api_urls=None):
        """
        Return a basic root view.
        """
        api_root_dict = OrderedDict()
        list_name = self.routes[0].name
        for prefix, _, basename in self.registry:
            api_root_dict[prefix] = list_name.format(basename=basename)

        class APIRootView(views.APIView):
            _ignore_model_permissions = True
            exclude_from_schema = True

            @staticmethod
            def get_api_root_dict(request):
                api_root_dict = OrderedDict()
                list_name = urls.router.routes[0].name
                for prefix, viewset, basename in urls.router.registry:
                    if viewset in MySURouter.ADMIN_ONLY_VIEWSETS:
                        continue
                    api_root_dict[prefix] = list_name.format(basename=basename)

                return api_root_dict

            def get(self, request, *args, **kwargs):
                # Return a plain {"name": "hyperlink"} response.
                ret = OrderedDict()
                namespace = request.resolver_match.namespace
                for key, url_name in self.get_api_root_dict(request).items():
                    if namespace:
                        url_name = namespace + ':' + url_name
                    try:
                        ret[key] = reverse(
                            url_name,
                            args=args,
                            kwargs=kwargs,
                            request=request,
                            format=kwargs.get('format', None)
                        )
                    except NoReverseMatch:
                        # Don't bail out if eg. no list routes exist, only detail routes.
                        continue

                return Response(ret)

        return APIRootView.as_view()
