from corsheaders.defaults import default_headers

from MySU.settings.base import *

DEBUG = True

# This is a nuclear option: do NOT use this unless you really really need the debug toolbar and it refuses to show up.
# DEBUG_TOOLBAR_CONFIG = {
#    'SHOW_TOOLBAR_CALLBACK': lambda request: True
# }

LOGGING = {
    'version': 1,
    'disable_existing_loggers': False,
    'incremental': True,
    'root': {
        'level': 'DEBUG',
    },
    'loggers': {
        'mozilla_django_oidc': {
            'handlers': ['console'],
            'level': 'DEBUG'
        }
    }
}


ENVIRONMENT_NAME = "DEVELOP"
ENVIRONMENT_COLOR = "#808080"

INSTALLED_APPS += [
    'debug_toolbar',
    'django_extensions',
]

MIDDLEWARE = MIDDLEWARE[:4] + ['debug_toolbar.middleware.DebugToolbarMiddleware'] + MIDDLEWARE[4:]


SITE_URL = 'https://dev.sun.snt.utwente.nl'

LOGIN_REDIRECT_URL_FAILURE = SITE_URL
LOGIN_REDIRECT_URL = SITE_URL
LOGOUT_REDIRECT_URL = SITE_URL

CORS_ORIGIN_ALLOW_ALL = True
CORS_ALLOW_CREDENTIALS = True
CORS_ALLOW_HEADERS = list(default_headers) + [
    'X-BULK-OPERATION',
]

CSRF_TRUSTED_ORIGINS = ['https://sun.snt.utwente.nl', 'https://dev.sun.snt.utwente.nl', 'dev.sun.snt.utwente.nl', 'sun.snt.utwente.nl']
CSRF_COOKIE_DOMAIN = 'sun.snt.utwente.nl'
SESSION_COOKIE_DOMAIN = 'sun.snt.utwente.nl'

ALLOWED_HOSTS = ['dev.sun.snt.utwente.nl', 'dev.sun.utwente.nl', 'sun.snt.utwente.nl',
                 'module7.sun.snt.utwente.nl', 'localhost', '127.0.0.1', '[::1]']

INTERNAL_IPS = ['127.0.0.1']

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.postgresql_psycopg2',
        'NAME': os.environ.get('DATABASE_NAME', ''),
        'USER': os.environ.get('DATABASE_USER', ''),
        'PASSWORD': os.environ.get('DATABASE_PASSWORD', ''),
        'HOST': os.environ.get('DATABASE_HOST', ''),
        'PORT': os.environ.get('DATABASE_PORT', ''),
    }
}
SECRET_KEY = os.environ.get('SECRET_KEY', '%)vgz6ff)rdl8qq1@7tf#p#nld-hx=fr)y88%g(v82!r%6oc)*')

OIDC_OP_AUTHORIZATION_ENDPOINT = "https://connect.test.surfconext.nl/oidc/authorize"
OIDC_OP_TOKEN_ENDPOINT = "https://connect.test.surfconext.nl/oidc/token"
OIDC_OP_USER_ENDPOINT = "https://connect.test.surfconext.nl/oidc/userinfo"
OIDC_OP_JWKS_ENDPOINT = "https://connect.test.surfconext.nl/oidc/certs"

# Default settings for the graph_models command
GRAPH_MODELS = {
    'all_applications': True,
    'group_models': True,
}
