from corsheaders.defaults import default_headers

from MySU.settings.base import *

ENVIRONMENT_NAME = "PRODUCTION"
ENVIRONMENT_COLOR = "#FF0000"

CSRF_COOKIE_SECURE = True
SESSION_COOKIE_SECURE = True
CSRF_COOKIE_HTTPONLY = False
SECURE_SSL_REDIRECT = True
SECURE_HSTS_SECONDS = 31536000
SECURE_HSTS_PRELOAD = True

ALLOWED_HOSTS = ['api.sun.utwente.nl', 'sun.utwente.nl', 'my.su.utwente.nl']

SITE_URL = 'https://my.su.utwente.nl/api'

CORS_ORIGIN_ALLOW_ALL = True
CORS_ALLOW_CREDENTIALS = True
CORS_ALLOW_HEADERS = list(default_headers) + [
    'X-BULK-OPERATION',
]

CSRF_TRUSTED_ORIGINS = ['api.sun.utwente.nl', 'sun.utwente.nl', 'my.su.utwente.nl']
CSRF_COOKIE_DOMAIN = 'utwente.nl'
SESSION_COOKIE_DOMAIN = 'utwente.nl'

LOGIN_REDIRECT_URL_FAILURE = SITE_URL
LOGIN_REDIRECT_URL = SITE_URL
LOGOUT_REDIRECT_URL = SITE_URL

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.postgresql_psycopg2',
        'NAME': os.environ.get('PROD_DATABASE_NAME', ''),
        'USER': os.environ.get('PROD_DATABASE_USER', ''),
        'PASSWORD': os.environ.get('PROD_DATABASE_PASSWORD', ''),
        'HOST': os.environ.get('PROD_DATABASE_HOST', ''),
        'PORT': os.environ.get('PROD_DATABASE_PORT', ''),
    }
}
SECRET_KEY = os.environ.get('PROD_SECRET_KEY', '')

OIDC_OP_AUTHORIZATION_ENDPOINT = "https://connect.surfconext.nl/oidc/authorize"
OIDC_OP_TOKEN_ENDPOINT = "https://connect.surfconext.nl/oidc/token"
OIDC_OP_USER_ENDPOINT = "https://connect.surfconext.nl/oidc/userinfo"
OIDC_OP_JWKS_ENDPOINT = "https://connect.surfconext.nl/oidc/certs"

OIDC_RP_CLIENT_ID = os.environ.get('PROD_OIDC_RP_CLIENT_ID', None)
OIDC_RP_CLIENT_SECRET = os.environ.get('PROD_OIDC_RP_CLIENT_SECRET', None)

EMAIL_BACKEND = 'MySU.backends.gmail_backend.GmailBackend'

GOOGLE_API_ADMIN_SERVICE_ACCOUNT_FILE = os.environ.get('GOOGLE_API_ADMIN_SERVICE_ACCOUNT_FILE', None)
GOOGLE_API_ADMIN_SERVICE_ACCOUNT_EMAIL = os.environ.get('GOOGLE_API_ADMIN_SERVICE_ACCOUNT_EMAIL', None)
GOOGLE_API_GMAIL_SERVICE_ACCOUNT_FILE = os.environ.get('GOOGLE_API_GMAIL_SERVICE_ACCOUNT_FILE', None)
GOOGLE_API_GMAIL_SERVICE_ACCOUNT_EMAIL = os.environ.get('GOOGLE_API_GMAIL_SERVICE_ACCOUNT_EMAIL', None)

FORCE_SCRIPT_NAME = "/api"
STATIC_URL = "%s/static/" % FORCE_SCRIPT_NAME
MEDIA_URL = "%s/media/" % FORCE_SCRIPT_NAME
