from __future__ import absolute_import
from __future__ import unicode_literals

from django import template
from django.urls import reverse
from django.urls.exceptions import NoReverseMatch
from django.utils.html import escape
from django.utils.html import format_html
from django.utils.safestring import mark_safe
from django.utils.safestring import SafeData

register = template.Library()


@register.simple_tag
def login(request, user):
    """
    Include a logout snippet if REST framework's logout view is in the URLconf.
    """
    if user.surfconext_signin:
        try:
            logout_url = reverse('oidc_logout')
        except NoReverseMatch:
            snippet = format_html('<li class="navbar-text">{user}</li>', user=escape(user))
            return mark_safe(snippet)
    else:
        try:
            logout_url = reverse('logout')
        except NoReverseMatch:
            snippet = format_html('<li class="navbar-text">{user}</li>', user=escape(user))
            return mark_safe(snippet)

    snippet = """<li class="dropdown">
        <a href="#" class="dropdown-toggle" data-toggle="dropdown">
            {user}
            <b class="caret"></b>
        </a>
        <ul class="dropdown-menu">
            <li><a href='{href}?next={next}'>Log out</a></li>
        </ul>
    </li>"""
    snippet = format_html(snippet, user=escape(user), href=logout_url, next=escape(request.path))

    return mark_safe(snippet)


@register.simple_tag
def logout(request):
    """
    Include a login snippet if REST framework's login view is in the URLconf.
    """
    try:
        login_url = reverse('login')
    except NoReverseMatch:
        return ''

    snippet = "<li><a href='{href}?next={next}'>Log in</a></li>"
    snippet = format_html(snippet, href=login_url, next=escape(request.path))

    return mark_safe(snippet)
