import os

from django.conf.urls import include
from django.conf.urls import url
from django.contrib import admin
from django.contrib.staticfiles.urls import static
from drf_yasg import openapi
from drf_yasg.views import get_schema_view
from rest_framework import permissions
from rest_framework_jwt.views import obtain_jwt_token
from rest_framework_jwt.views import refresh_jwt_token
from rest_framework_jwt.views import verify_jwt_token

from apps.buddy.urls import router as buddy_router
from apps.calendar.urls import router as calendar_router
from apps.financial.urls import router as financial_router
from apps.MySU.urls import router as core_router
from MySU.router import MySURouter
from MySU.settings.base import MEDIA_ROOT
from MySU.settings.base import MEDIA_URL
from MySU.settings.base import STATIC_ROOT
from MySU.settings.base import STATIC_URL

if os.environ.get('DJANGO_SETTINGS_MODULE') in ("MySU.settings.develop", "MySU.settings.local", "MySU.settings.test"):
    import debug_toolbar

router = MySURouter()  # pylint: disable=invalid-name

router.registry.extend(buddy_router.registry)
router.registry.extend(calendar_router.registry)
router.registry.extend(core_router.registry)
router.registry.extend(financial_router.registry)


admin.site.site_header = "MySU Admin Panel"
admin.site.site_title = "MySU Admin Panel"

schema_view = get_schema_view(
    openapi.Info(
        title="My SU API",
        default_version='v1',
        description="",
        # terms_of_service="",
        contact=openapi.Contact(email="sun@union.utwente.nl"),
        license=openapi.License(name="BSD License"),
    ),
    public=True,
    permission_classes=(permissions.IsAuthenticated,),
)


urlpatterns = [  # pylint: disable=invalid-name
    url(r'^', include("apps.buddy.urls")),
    url(r'^', include("apps.calendar.urls")),
    url(r'^', include("apps.MySU.urls")),
    url(r'^', include("apps.financial.urls")),
    url(r'^swagger(?P<format>\.json|\.yaml)$', schema_view.without_ui(cache_timeout=0), name='schema-json'),
    url(r'^swagger/$', schema_view.with_ui('swagger', cache_timeout=0), name='schema-swagger-ui'),
    url(r'^redoc/$', schema_view.with_ui('redoc', cache_timeout=0), name='schema-redoc'),
    url(r'^', include(router.urls)),
    url(r'^oidc/', include('mozilla_django_oidc.urls')),
    url(r'^account/', include('django.contrib.auth.urls')),
    url(r'^account/', include('rest_registration.api.urls')),
    url(r'^account/', include('django_registration.backends.activation.urls')),
    url(r'^admin/', admin.site.urls),
    url(r'^auth/token/verify/', verify_jwt_token),
    url(r'^auth/token/refresh/', refresh_jwt_token),
    url(r'^auth/token/obtain/', obtain_jwt_token),
    url(r'^i18n/', include('django.conf.urls.i18n')),
]

urlpatterns += static(MEDIA_URL, document_root=MEDIA_ROOT)
urlpatterns += static(STATIC_URL, document_root=STATIC_ROOT)

if os.environ.get('DJANGO_SETTINGS_MODULE') in ("MySU.settings.develop", "MySU.settings.local", "MySU.settings.test"):
    urlpatterns += [
        url(r'^__debug__/', include(debug_toolbar.urls)),
    ]
