from django.contrib import admin
from django_admin_listfilter_dropdown.filters import RelatedDropdownFilter
from guardian.models import GroupObjectPermission

from apps.MySU.models import Group


class GroupObjectPermissionInlineAdmin(admin.TabularInline):
    model = GroupObjectPermission
    extra = 0
    fields = (
        'permission',
    )
    list_display = (
        'group',
        'permission',
    )
    search_fields = (
        'group',
    )

    def formfield_for_foreignkey(self, db_field, request=None, **kwargs):
        field = super(GroupObjectPermissionInlineAdmin, self).formfield_for_foreignkey(db_field, request, **kwargs)
        # prefetch the permission, so that the str() method on permission doesn't cause hundreds of queries
        if db_field.name == "permission":
            field.queryset = field.queryset.select_related()
        return field


@admin.register(Group)
class MemberGroupAdmin(admin.ModelAdmin):
    inlines = [GroupObjectPermissionInlineAdmin]
    search_fields = (
        'group',
    )
    list_display = (
        'group',
        'association',
        'parent_group',
        'founding_date',
        'dissolution_date',
    )

    list_filter = (
        ('parent_group', RelatedDropdownFilter),
        ('association', RelatedDropdownFilter),
    )
    readonly_fields = (
        'name',
    )
    exclude = (
        'permissions',
    )
    list_select_related = (
        'association',
        'parent_group',
        'django_group'
    )

    def get_readonly_fields(self, request, obj=None):
        if obj:
            return self.readonly_fields + ('association',)
        else:
            return self.readonly_fields

    def save_formset(self, request, form, formset, change):
        """
        Set the content object, so the generic foreignkey of the groupobjectpermission instances matches the association
        linked to the group.
        """
        for form in formset.forms:
            form.instance.content_object = form.instance.group.association
        super().save_formset(request, form, formset, change)
