from django.contrib import admin
from django.contrib.auth.admin import UserAdmin as DjangoUserAdmin
from django.utils.translation import gettext_lazy as _
from import_export import resources
from import_export.admin import ImportExportModelAdmin
from simple_history.admin import SimpleHistoryAdmin

from apps.MySU.models import User


class UserResource(resources.ModelResource):
    class Meta:
        model = User
        fields = (
            'id',
            'username',
            'email',
        )


@admin.register(User)
class UserAdmin(ImportExportModelAdmin, SimpleHistoryAdmin, DjangoUserAdmin):
    resource_class = UserResource
    base_fieldsets = (
        (_('Permissions'), {
            'fields': ('is_active', 'is_staff', 'is_superuser'),
        }),
        (_('Important dates'), {'fields': ('last_login', 'date_joined', 'date_left')}),
    )
    api_fieldsets = (
        (None, {'fields': ('username', 'password', 'surfconext_signin', 'is_api')}),
        (_('Personal info'), {'fields': ('email', )}),
        *base_fieldsets
    )
    fieldsets = (
        (None, {'fields': ('username', 'surfconext_signin', 'is_api')}),
        (_('Personal info'), {'fields': ('email', 'preferred_profile')}),
        *base_fieldsets
    )
    list_display = ('username', 'email', 'date_joined', 'is_staff', 'is_api')
    list_filter = ('is_staff', 'is_superuser', 'is_active', 'surfconext_signin', 'is_api', "date_joined")
    search_fields = ('username', 'email')
    ordering = ('date_joined',)

    def get_fieldsets(self, request, obj=None):
        if not obj:
            return self.add_fieldsets
        if obj.is_api:
            return self.api_fieldsets
        return self.fieldsets

    def get_readonly_fields(self, request, obj=None):
        if not obj:
            return []
        if obj.is_api:
            return ['preferred_profile', 'is_staff', 'is_superuser', 'surfconext_signin', 'is_api', 'last_login', 'date_joined']
        if obj.surfconext_signin:
            return ['username', 'password', 'surfconext_signin', 'is_api', 'last_login', 'date_joined']
        else:
            return ['username', 'password', 'last_login', 'date_joined']
