# Generated by Django 2.2.13 on 2020-12-06 11:16
import datetime
import uuid

import autoslug.fields
import computedfields.resolver
import django.contrib.auth.models
import django.core.validators
import django.db.models.deletion
import guardian.mixins
import multiselectfield.db.fields
from django.conf import settings
from django.db import migrations
from django.db import models

import apps.MySU.models.permission
import MySU.fields.yearlessdate
import MySU.objects.yearlessdate


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('auth', '0011_update_proxy_permissions'),
    ]

    operations = [
        migrations.CreateModel(
            name='Association',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('phone_number', models.CharField(blank=True, max_length=20)),
                ('address', models.CharField(blank=True, max_length=100)),
                ('zip_code', models.CharField(blank=True, max_length=10)),
                ('city', models.CharField(blank=True, max_length=50)),
                ('country', models.CharField(blank=True, max_length=50)),
                ('slug', autoslug.fields.AutoSlugField(default=uuid.uuid4, editable=False, max_length=200, populate_from='name')),
                ('name', models.CharField(max_length=200)),
                ('short_name', models.CharField(max_length=200)),
                ('founding_date', models.DateField(default=datetime.date.today)),
                ('description', models.TextField(blank=True)),
                ('new_member_text', models.TextField(blank=True)),
                ('full_year_membership_start_date', MySU.fields.yearlessdate.YearlessDateField(max_length=5)),
                ('photo', models.ImageField(blank=True, upload_to='association/photo/')),
                ('logo', models.ImageField(blank=True, upload_to='association/logo/')),
                ('bylaws', models.FileField(blank=True, upload_to='association/bylaws/')),
                ('articles_of_association', models.FileField(blank=True, upload_to='association/articles_of_association/')),
                ('privacy_statement', models.FileField(blank=True, upload_to='association/privacy_statement/')),
                ('terms_of_service', models.FileField(blank=True, upload_to='association/terms_of_service/')),
                ('dissolution_date', models.DateField(blank=True, null=True)),
                ('postal_address', models.CharField(blank=True, max_length=200)),
                ('email', models.EmailField(max_length=254)),
                ('color', models.CharField(blank=True, max_length=10)),
            ],
            options={
                'verbose_name_plural': 'Associations',
                'ordering': ['name'],
                'permissions': (('board', 'Board'),),
            },
        ),
        migrations.CreateModel(
            name='AssociationMembership',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('slug', models.UUIDField(default=uuid.uuid4, editable=False)),
                ('status', multiselectfield.db.fields.MultiSelectField(choices=[('Accepted', 'Accepted'), ('Claimed', 'Claimed'), ('Archived', 'Archived'), ('Disputed', 'Disputed'), ('Ended', 'Ended'), ('Pending', 'Pending'), ('Rejected', 'Rejected')], default='Pending', max_length=57)),
                ('date_joined', models.DateField(default=datetime.date.today)),
                ('date_left', models.DateField(blank=True, null=True)),
                ('visible_after_date_left', models.BooleanField(default=False)),
                ('change_type_on_period_end', models.BooleanField(default=False)),
                ('current', models.BooleanField(default=False, editable=False)),
                ('association', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to='MySU.Association')),
            ],
            options={
                'verbose_name': 'AssociationMembership',
                'verbose_name_plural': 'AssociationMemberships',
            },
            bases=(computedfields.resolver._ComputedFieldsModelBase, models.Model),
        ),
        migrations.CreateModel(
            name='DebtCollection',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('slug', models.SlugField(default=uuid.uuid4, editable=False)),
                ('creation_date', models.DateTimeField(auto_now_add=True)),
                ('debt_collection_date', models.DateTimeField()),
                ('locked', models.BooleanField(default=False)),
                ('amount', models.FloatField(blank=True, null=True, validators=[django.core.validators.MinValueValidator(0.0)])),
                ('association', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, related_name='debt_collections', to='MySU.Association')),
            ],
            options={
                'verbose_name': 'Debt collection',
                'verbose_name_plural': 'Debt collections',
                'ordering': ['debt_collection_date'],
            },
        ),
        migrations.CreateModel(
            name='DebtEntry',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('slug', models.SlugField(default=uuid.uuid4, editable=False)),
                ('total_price', models.FloatField(validators=[django.core.validators.MinValueValidator(0.0)])),
                ('association', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, related_name='debt_entries', to='MySU.Association')),
                ('debt_collection', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, related_name='debt_entries', to='MySU.DebtCollection')),
            ],
            options={
                'verbose_name': 'Debt Entry',
                'verbose_name_plural': 'Debt Entries',
            },
        ),
        migrations.CreateModel(
            name='Enrollment',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('slug', models.SlugField(default=uuid.uuid4, editable=False)),
                ('association', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to='MySU.Association')),
                ('debt_entry', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='enrollments', to='MySU.DebtEntry')),
            ],
            options={
                'verbose_name_plural': 'Enrollments',
            },
        ),
        migrations.CreateModel(
            name='GroupMembership',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('slug', models.UUIDField(default=uuid.uuid4, editable=False)),
                ('duty', models.CharField(default='General member', max_length=100)),
                ('date_joined', models.DateField(default=datetime.date.today)),
                ('date_left', models.DateField(blank=True, null=True)),
                ('archived', models.BooleanField(default=False)),
                ('email', models.EmailField(blank=True, max_length=254, null=True)),
                ('order', models.PositiveSmallIntegerField(default=1)),
                ('association', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, related_name='groupmemberships', to='MySU.Association')),
                ('django_group', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, related_name='groupmemberships', to='auth.Group')),
            ],
            options={
                'verbose_name': 'Group Membership',
                'verbose_name_plural': 'Group Memberships',
            },
        ),
        migrations.CreateModel(
            name='Institution',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('phone_number', models.CharField(blank=True, max_length=20)),
                ('address', models.CharField(blank=True, max_length=100)),
                ('zip_code', models.CharField(blank=True, max_length=10)),
                ('city', models.CharField(blank=True, max_length=50)),
                ('country', models.CharField(blank=True, max_length=50)),
                ('slug', autoslug.fields.AutoSlugField(always_update=True, editable=False, populate_from='short_name', unique=True)),
                ('name', models.CharField(max_length=100)),
                ('short_name', models.CharField(max_length=100)),
            ],
            options={
                'verbose_name': 'Institution',
                'verbose_name_plural': 'Institutions',
            },
        ),
        migrations.CreateModel(
            name='Profile',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('phone_number', models.CharField(blank=True, max_length=20)),
                ('address', models.CharField(blank=True, max_length=100)),
                ('zip_code', models.CharField(blank=True, max_length=10)),
                ('city', models.CharField(blank=True, max_length=50)),
                ('country', models.CharField(blank=True, max_length=50)),
                ('slug', models.SlugField(default=uuid.uuid4, editable=False)),
                ('profilename', models.CharField(blank=True, default='Created by you', max_length=100)),
                ('photo', models.ImageField(blank=True, null=True, upload_to='profile_photo/')),
                ('student_number', models.CharField(blank=True, max_length=100, null=True)),
                ('given_name', models.CharField(blank=True, max_length=100)),
                ('first_names', models.CharField(blank=True, max_length=100)),
                ('surname', models.CharField(blank=True, max_length=100)),
                ('email', models.EmailField(blank=True, max_length=254, null=True)),
                ('gender', models.CharField(blank=True, max_length=100, null=True)),
                ('date_of_birth', models.DateField(blank=True, null=True)),
                ('is_osiris', models.BooleanField(default=False)),
                ('iban', models.CharField(max_length=100)),
                ('bic_code', models.CharField(blank=True, max_length=100, null=True)),
                ('is_master', models.NullBooleanField(default=False)),
            ],
            options={
                'verbose_name': 'Profile',
                'verbose_name_plural': 'Profiles',
            },
        ),
        migrations.CreateModel(
            name='Study',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('slug', models.SlugField(default=uuid.uuid4, editable=False)),
                ('studycode', models.CharField(max_length=100)),
                ('name_en', models.CharField(max_length=100)),
                ('name_nl', models.CharField(max_length=100)),
                ('institution', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to='MySU.Institution')),
            ],
            options={
                'verbose_name': 'Study',
                'verbose_name_plural': 'Studies',
            },
        ),
        migrations.CreateModel(
            name='StudyRecord',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('slug', models.UUIDField(default=uuid.uuid4, editable=False)),
                ('phase', multiselectfield.db.fields.MultiSelectField(choices=[('P', 'Premaster'), ('B', 'Bachelor'), ('M', 'Master'), ('PB', 'Prebachelor')], default='B', max_length=8)),
                ('year', models.IntegerField()),
                ('date_start', models.DateField()),
                ('date_end', models.DateField()),
                ('is_primary', models.BooleanField()),
                ('institution', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, related_name='study_records', to='MySU.Institution')),
            ],
            options={
                'verbose_name': 'Study record',
                'verbose_name_plural': 'Study records',
            },
        ),
        migrations.CreateModel(
            name='SUNPermission',
            fields=[
            ],
            options={
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('auth.permission',),
            managers=[
                ('objects', apps.MySU.models.permission.SUNPermissionManager()),
            ],
        ),
        migrations.CreateModel(
            name='Society',
            fields=[
                ('association_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='MySU.Association')),
            ],
            options={
                'verbose_name': 'Society',
                'verbose_name_plural': 'Societies',
                'ordering': ['name'],
                'permissions': (('society', 'Society'),),
            },
            bases=('MySU.association',),
        ),
        migrations.CreateModel(
            name='User',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('password', models.CharField(max_length=128, verbose_name='password')),
                ('last_login', models.DateTimeField(blank=True, null=True, verbose_name='last login')),
                ('is_superuser', models.BooleanField(default=False, help_text='Designates that this user has all permissions without explicitly assigning them.', verbose_name='superuser status')),
                ('slug', models.UUIDField(default=uuid.uuid4, editable=False)),
                ('username', models.CharField(max_length=255, unique=True)),
                ('student_number', models.CharField(blank=True, max_length=100, null=True)),
                ('surfconext_signin', models.BooleanField(default=False)),
                ('date_joined', models.DateField(default=datetime.date.today)),
                ('date_activated', models.DateField(blank=True, null=True)),
                ('date_left', models.DateField(blank=True, null=True)),
                ('is_active', models.BooleanField(default=True)),
                ('is_staff', models.BooleanField(default=False)),
                ('email', models.EmailField(max_length=254, null=True, validators=[django.core.validators.RegexValidator(inverse_match=True, message='Recovery email cannot be your student email', regex='student.utwente')])),
                ('groups', models.ManyToManyField(blank=True, help_text='The groups this user belongs to. A user will get all permissions granted to each of their groups.', related_name='user_set', related_query_name='user', through='MySU.GroupMembership', to='auth.Group', verbose_name='groups')),
                ('preferred_profile', models.OneToOneField(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to='MySU.Profile')),
                ('studies', models.ManyToManyField(related_name='users', through='MySU.StudyRecord', to='MySU.Study')),
                ('user_permissions', models.ManyToManyField(blank=True, help_text='Specific permissions for this user.', related_name='user_set', related_query_name='user', to='auth.Permission', verbose_name='user permissions')),
            ],
            options={
                'verbose_name': 'User',
                'verbose_name_plural': 'Users',
            },
            bases=(models.Model, guardian.mixins.GuardianUserMixin),
            managers=[
                ('objects', django.contrib.auth.models.UserManager()),
            ],
        ),
        migrations.AddField(
            model_name='studyrecord',
            name='student',
            field=models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, related_name='study_records', to=settings.AUTH_USER_MODEL),
        ),
        migrations.AddField(
            model_name='studyrecord',
            name='study',
            field=models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to='MySU.Study'),
        ),
        migrations.AddField(
            model_name='profile',
            name='study',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='profile', to='MySU.Study'),
        ),
        migrations.AddField(
            model_name='profile',
            name='user',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='profiles', to=settings.AUTH_USER_MODEL),
        ),
        migrations.CreateModel(
            name='Notification',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('slug', models.SlugField(default=uuid.uuid4, editable=False)),
                ('date', models.DateField(auto_now=True)),
                ('message', models.TextField()),
                ('read', models.BooleanField(default=False)),
                ('receiver', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to=settings.AUTH_USER_MODEL)),
                ('sender', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to='MySU.Association')),
                ('to', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to='MySU.Profile')),
            ],
            options={
                'verbose_name': 'Notification',
                'verbose_name_plural': 'Notifications',
            },
        ),
        migrations.CreateModel(
            name='MemberType',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('slug', models.SlugField(default=uuid.uuid4, editable=False)),
                ('type', models.CharField(max_length=200)),
                ('only_chooseable_by_board', models.BooleanField(default=False)),
                ('auto_end', models.BooleanField(default=False)),
                ('start_date', MySU.fields.yearlessdate.YearlessDateField(default=MySU.objects.yearlessdate.YearlessDate(8, 1), max_length=5)),
                ('end_date', MySU.fields.yearlessdate.YearlessDateField(default=MySU.objects.yearlessdate.YearlessDate(7, 31), max_length=5)),
                ('membership_fee', models.FloatField(validators=[django.core.validators.MinValueValidator(0.0)])),
                ('payment_per', multiselectfield.db.fields.MultiSelectField(choices=[('Year', 'Year'), ('Semester', 'Semester'), ('Month', 'Month'), ('Week', 'Week')], default='Year', max_length=24)),
                ('grace_period', models.PositiveSmallIntegerField(default=7)),
                ('association', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to='MySU.Association')),
            ],
            options={
                'unique_together': {('type', 'association')},
            },
        ),
        migrations.CreateModel(
            name='MembershipFee',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('slug', models.SlugField(default=uuid.uuid4, editable=False)),
                ('payments', models.PositiveIntegerField(help_text='Use the following format: <YYYY>', validators=[django.core.validators.MinValueValidator(1900), django.core.validators.MaxValueValidator(2020)])),
                ('association', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, related_name='membership_fees', to='MySU.Association')),
                ('association_membership', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, related_name='membership_fees', to='MySU.AssociationMembership')),
                ('debt_entry', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, related_name='membership_fees', to='MySU.DebtEntry')),
                ('membertype', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, related_name='membership_fees', to='MySU.MemberType')),
            ],
            options={
                'verbose_name': 'Membership fee',
                'verbose_name_plural': 'Membership fees',
            },
        ),
        migrations.CreateModel(
            name='Matching',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('slug', models.SlugField(default=uuid.uuid4, editable=False)),
                ('association', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='matchings', to='MySU.Association')),
                ('left_profile', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='left_matchings', to='MySU.Profile')),
                ('left_user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='left_matchings', to=settings.AUTH_USER_MODEL)),
                ('right_profile', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='right_matchings', to='MySU.Profile')),
                ('right_user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='right_matchings', to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'verbose_name': 'Matching',
                'verbose_name_plural': 'Matchings',
            },
        ),
        migrations.AddField(
            model_name='groupmembership',
            name='profile',
            field=models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, related_name='group_member', to='MySU.Profile'),
        ),
        migrations.AddField(
            model_name='groupmembership',
            name='user',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.PROTECT, to=settings.AUTH_USER_MODEL),
        ),
        migrations.CreateModel(
            name='Group',
            fields=[
                ('slug', models.SlugField(default=uuid.uuid4, editable=False)),
                ('full_name', models.CharField(max_length=100)),
                ('short_name', models.CharField(blank=True, max_length=100)),
                ('alternate_name', models.CharField(blank=True, max_length=100, null=True)),
                ('number', models.PositiveSmallIntegerField(blank=True, null=True)),
                ('motto', models.CharField(blank=True, max_length=200, null=True)),
                ('founding_date', models.DateField(default=datetime.date.today)),
                ('dissolution_date', models.DateField(blank=True, null=True)),
                ('archived', models.BooleanField(default=False)),
                ('description', models.TextField(blank=True)),
                ('photo', models.ImageField(blank=True, null=True, upload_to='group/photo/')),
                ('logo', models.ImageField(blank=True, null=True, upload_to='group/logo/')),
                ('board_group', models.BooleanField(default=False)),
                ('current_board', models.BooleanField(default=False)),
                ('email', models.EmailField(blank=True, max_length=254, null=True)),
                ('django_group', models.OneToOneField(on_delete=django.db.models.deletion.PROTECT, parent_link=True, primary_key=True, related_name='group', serialize=False, to='auth.Group')),
                ('association', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, related_name='groups', to='MySU.Association')),
                ('parent_group', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, to='MySU.Group')),
            ],
            options={
                'verbose_name': 'Group',
                'verbose_name_plural': 'Groups',
            },
            bases=('auth.group',),
        ),
        migrations.CreateModel(
            name='EventType',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('slug', models.SlugField(default=uuid.uuid4, editable=False)),
                ('type', models.CharField(max_length=200)),
                ('association', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, related_name='event_types', to='MySU.Association')),
            ],
            options={
                'verbose_name_plural': 'EventTypes',
            },
        ),
        migrations.CreateModel(
            name='Event',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('slug', models.SlugField(default=uuid.uuid4, editable=False)),
                ('name', models.CharField(max_length=100)),
                ('description', models.TextField(blank=True, null=True)),
                ('start_date', models.DateTimeField()),
                ('end_date', models.DateTimeField()),
                ('enrollable', models.NullBooleanField()),
                ('enrollable_from', models.DateTimeField(blank=True, null=True)),
                ('enrollable_until', models.DateTimeField(blank=True, null=True)),
                ('unenrollable', models.NullBooleanField()),
                ('unenrollable_until', models.DateTimeField(blank=True, null=True)),
                ('max_number_of_enrollments', models.PositiveIntegerField(blank=True, null=True)),
                ('public', models.BooleanField(default=False)),
                ('location', models.CharField(blank=True, max_length=100)),
                ('association', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to='MySU.Association')),
                ('attendants', models.ManyToManyField(related_name='enrolled_events', through='MySU.Enrollment', to='MySU.Profile')),
                ('type', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to='MySU.EventType')),
            ],
            options={
                'verbose_name_plural': 'Events',
            },
        ),
        migrations.CreateModel(
            name='EnrollmentOption',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('slug', models.SlugField(default=uuid.uuid4, editable=False)),
                ('name', models.CharField(max_length=100)),
                ('participation_fee', models.FloatField(validators=[django.core.validators.MinValueValidator(0.0)])),
                ('association', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, related_name='enrollment_options', to='MySU.Association')),
                ('event', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, related_name='enrollment_options', to='MySU.Event')),
            ],
            options={
                'verbose_name_plural': 'EnrollmentOptions',
                'unique_together': {('name', 'event')},
            },
        ),
        migrations.AddField(
            model_name='enrollment',
            name='enrollment_option',
            field=models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, related_name='enrollments', to='MySU.EnrollmentOption'),
        ),
        migrations.AddField(
            model_name='enrollment',
            name='event',
            field=models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, related_name='enrollments', to='MySU.Event'),
        ),
        migrations.AddField(
            model_name='enrollment',
            name='profile',
            field=models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, related_name='enrollments', to='MySU.Profile'),
        ),
        migrations.AddField(
            model_name='enrollment',
            name='user',
            field=models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, related_name='enrollments', to=settings.AUTH_USER_MODEL),
        ),
        migrations.AddField(
            model_name='debtentry',
            name='profile',
            field=models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, related_name='debt_entries', to='MySU.Profile'),
        ),
        migrations.AddField(
            model_name='debtentry',
            name='user',
            field=models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, related_name='debt_entries', to=settings.AUTH_USER_MODEL),
        ),
        migrations.CreateModel(
            name='AssociationSpecificDataFields',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('slug', models.SlugField(default=uuid.uuid4, editable=False)),
                ('type', multiselectfield.db.fields.MultiSelectField(choices=[('Boolean', 'Boolean'), ('Number', 'Number'), ('String', 'String'), ('Choice', 'Choice')], default='Boolean', max_length=28)),
                ('name', models.CharField(max_length=50)),
                ('helper_text', models.TextField(blank=True, default='', max_length=200)),
                ('choices', models.CharField(blank=True, max_length=200, null=True)),
                ('default', models.TextField(blank=True, null=True)),
                ('include_with_members_incoming', models.BooleanField(default=False)),
                ('board_only', models.BooleanField(default=False)),
                ('mandatory', models.BooleanField(default=False)),
                ('association', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='MySU.Association')),
            ],
            options={
                'verbose_name_plural': 'AssociationSpecificDataFields',
                'unique_together': {('name', 'association')},
            },
        ),
        migrations.CreateModel(
            name='AssociationSpecificData',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('slug', models.SlugField(default=uuid.uuid4, editable=False)),
                ('value', models.TextField(blank=True, null=True)),
                ('association', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='MySU.Association')),
                ('data_field', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='MySU.AssociationSpecificDataFields')),
                ('membership', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='MySU.AssociationMembership')),
            ],
            options={
                'verbose_name_plural': 'AssociationSpecificData',
                'unique_together': {('membership', 'data_field')},
            },
        ),
        migrations.AddField(
            model_name='associationmembership',
            name='data_fields',
            field=models.ManyToManyField(related_name='association_memberships', through='MySU.AssociationSpecificData', to='MySU.AssociationSpecificDataFields'),
        ),
        migrations.AddField(
            model_name='associationmembership',
            name='new_type',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='new_memberships', to='MySU.MemberType'),
        ),
        migrations.AddField(
            model_name='associationmembership',
            name='profile',
            field=models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, related_name='association_memberships', to='MySU.Profile'),
        ),
        migrations.AddField(
            model_name='associationmembership',
            name='type',
            field=models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, related_name='memberships', to='MySU.MemberType'),
        ),
        migrations.AddField(
            model_name='associationmembership',
            name='user',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, to=settings.AUTH_USER_MODEL),
        ),
        migrations.AddField(
            model_name='association',
            name='members',
            field=models.ManyToManyField(related_name='associations', through='MySU.AssociationMembership', to='MySU.Profile'),
        ),
        migrations.AddField(
            model_name='association',
            name='users',
            field=models.ManyToManyField(related_name='associations', through='MySU.AssociationMembership', to=settings.AUTH_USER_MODEL),
        ),
        migrations.AlterUniqueTogether(
            name='groupmembership',
            unique_together={('profile', 'django_group', 'date_left')},
        ),
        migrations.AlterUniqueTogether(
            name='enrollment',
            unique_together={('profile', 'event')},
        ),
        migrations.AlterUniqueTogether(
            name='associationmembership',
            unique_together={('user', 'association', 'date_left')},
        ),
    ]
