from django.db import migrations


def forwards_func(apps, schema_editor):
    # We get the model from the versioned app registry;
    # if we directly import it, it'll be the wrong version
    Profile = apps.get_model("MySU", "Profile")
    BankAccount = apps.get_model("financial", "BankAccount")
    db_alias = schema_editor.connection.alias
    profiles = Profile.objects.using(db_alias).all()
    bank_accounts = BankAccount.objects.using(db_alias).bulk_create([ BankAccount(iban=profile.iban, bic=profile.bic_code) for profile in profiles ])
    for (profile, bank_account) in zip(profiles, bank_accounts):
        profile.bank_account = bank_account
    Profile.objects.using(db_alias).bulk_update(profiles, ['bank_account'])

def reverse_func(apps, schema_editor):
    # forwards_func() adds bank_accounts and populates them using field from the profile,
    # so reverse_func() should populate the profile fields using the bank accounts, and then remove the bank accounts.
    Profile = apps.get_model("MySU", "Profile")
    BankAccount = apps.get_model("financial", "BankAccount")
    db_alias = schema_editor.connection.alias
    to_be_removed_bank_accounts = []
    profiles = Profile.objects.using(db_alias).select_related('bank_account')
    for profile in profiles:
        profile.iban = profile.bank_account.iban
        profile.bic_code = profile.bank_account.bic
        to_be_removed_bank_accounts.append(profile.bank_account)
        profile.bank_account = None
    Profile.objects.using(db_alias).bulk_update(profiles, ['iban', 'bic_code', 'bank_account'])
    BankAccount.objects.using(db_alias).filter(id__in=[bank_account.id for bank_account in to_be_removed_bank_accounts]).delete()

class Migration(migrations.Migration):

    dependencies = [
        ('MySU', '0033_profile_bank_account'),
    ]

    operations = [
        migrations.RunPython(forwards_func, reverse_func),
    ]
