import uuid

from django.db import migrations


def forwards_func(apps, schema_editor):
    # We get the model from the versioned app registry;
    # if we directly import it, it'll be the wrong version
    AssociationMembership = apps.get_model("MySU", "AssociationMembership")
    SepaMandate = apps.get_model("financial", "SepaMandate")
    db_alias = schema_editor.connection.alias
    memberships = AssociationMembership.objects.using(db_alias).all()
    sepa_mandates = SepaMandate.objects.using(db_alias).bulk_create([
        SepaMandate(
            slug=uuid.uuid4(),
            mandate_id=uuid.uuid4(),
            signature_date=membership.date_joined,
            association=membership.association,
            profile=membership.profile
        ) for membership in memberships
    ])
    for (membership, sepa_mandate) in zip(memberships, sepa_mandates):
        membership.sepa_mandate = sepa_mandate
    AssociationMembership.objects.using(db_alias).bulk_update(memberships, ['sepa_mandate'])

def reverse_func(apps, schema_editor):
    # forwards_func() adds sepa mandates,
    # so reverse_func() should remove them.
    AssociationMembership = apps.get_model("MySU", "AssociationMembership")
    SepaMandate = apps.get_model("financial", "SepaMandate")
    db_alias = schema_editor.connection.alias
    to_be_removed_sepa_mandates = []
    memberships = AssociationMembership.objects.using(db_alias).select_related('sepa_mandate')
    for membership in memberships:
        to_be_removed_sepa_mandates.append(membership.sepa_mandate)
        membership.sepa_mandate = None
    AssociationMembership.objects.using(db_alias).bulk_update(memberships, ['sepa_mandate'])
    SepaMandate.objects.using(db_alias).filter(id__in=[mandate.id for mandate in to_be_removed_sepa_mandates]).delete()

class Migration(migrations.Migration):

    dependencies = [
        ('MySU', '0036_associationmembership_sepa_mandate'),
        ('financial', '0011_auto_20220112_1440')
    ]

    operations = [
        migrations.RunPython(forwards_func, reverse_func),
    ]
