import uuid

from django.db import migrations


def forwards_func(apps, schema_editor):
    # We get the model from the versioned app registry;
    # if we directly import it, it'll be the wrong version
    AssociationMembership = apps.get_model("MySU", "AssociationMembership")
    AssociationSpecificData = apps.get_model("MySU", "AssociationSpecificData")
    AssociationSpecificDataFields = apps.get_model("MySU", "AssociationSpecificDataFields")
    db_alias = schema_editor.connection.alias

    memberships = AssociationMembership.objects.using(db_alias).prefetch_related("specific_data")
    data_fields = AssociationSpecificDataFields.objects.using(db_alias).all()

    to_be_updated_specific_data = []
    to_be_created_specific_data = []

    for membership in memberships:
        for field in data_fields:
            if membership.association == field.association:
                field_found = False
                default = field.default if field.default is not None else ""
                for data in membership.specific_data.all():
                    if data.data_field == field:
                        field_found = True
                        if data.value == "" or data.value is None:
                            # Previous error elsewhere caused mandatory fields to be set to "" instead of "True",
                            # but default for mandatory fields is "False". Hence, we need to make an exception here.
                            if field.mandatory:
                                data.value = "True"
                            else:
                                data.value = default
                            to_be_updated_specific_data.append(data)
                if not field_found:
                    data = AssociationSpecificData(
                        value=default,
                        association=membership.association,
                        membership=membership,
                        data_field=field
                    )
                    to_be_created_specific_data.append(data)

    AssociationSpecificData.objects.using(db_alias).bulk_create(to_be_created_specific_data)
    AssociationSpecificData.objects.using(db_alias).bulk_update(to_be_updated_specific_data, ["value"])


def reverse_func(apps, schema_editor):
    pass

class Migration(migrations.Migration):

    dependencies = [
        ('MySU', '0038_associationmembership_confirmation_of_request_to_end_email_send'),
    ]

    operations = [
        migrations.RunPython(forwards_func, reverse_func),
    ]
