import uuid

from django.core.exceptions import ValidationError
from django.db import models

from apps.MySU.models import Association
from apps.MySU.models import Profile
from apps.MySU.models import User


class NotificationQuerySet(models.QuerySet):
    def unread(self):
        return self.filter(read=True)


class NotificationManager(models.Manager):
    def get_queryset(self):
        return NotificationQuerySet(self.model, using=self._db)

    def unread(self):
        return self.get_queryset().unread()


class Notification(models.Model):
    """
    A user can be notified through email or an in-app notification. This stores the in-app notfications.
    """
    slug = models.SlugField(default=uuid.uuid4, editable=False)
    date = models.DateField(auto_now=True)
    message = models.TextField()
    read = models.BooleanField(default=False)

    sender = models.ForeignKey(Association, on_delete=models.PROTECT)
    to = models.ForeignKey(Profile, on_delete=models.PROTECT, related_name="notifiers")
    receiver = models.ForeignKey(Profile, on_delete=models.PROTECT, related_name="notified")

    objects = NotificationManager()

    class Meta:
        verbose_name = "Notification"
        verbose_name_plural = "Notifications"

    def __str__(self):
        return "{}: {}".format(self.sender, self.message)

    def clean(self):
        if self.receiver:
            if self.to.user is not self.receiver:
                raise ValidationError("Profile (to) not linked to User (receiver)")
