from django.contrib.auth.models import Permission
from django.contrib.auth.models import PermissionManager


class SUNPermissionManager(PermissionManager):
    def get_queryset(self):
        return super(SUNPermissionManager, self).get_queryset().exclude(name__startswith='Can')


class SUNPermission(Permission):
    """
    A proxy model overwrites can be treated as the model its extending, but with a different manager. The manager above
    filters the normal results by excluding those that start with `Can`. All default permission created by django start
    with the word `Can` whilst all additionally created permission do not. Hence this manager only queries from the
    custom created permissions.

    By having this model querying for the custom permissions is thus much simpler.
    """
    objects = SUNPermissionManager()

    class Meta:
        proxy = True

    def __str__(self):
        return self.name
