import uuid

from django.db import models
from guardian.shortcuts import get_objects_for_user

from apps.MySU.models.contact_info import ContactInfo
from apps.MySU.models.group_membership import GroupMembership


class ProfileQuerySet(models.QuerySet):
    def emails(self):
        pass


class ProfileManager(models.Manager):
    def get_queryset(self):
        return ProfileQuerySet(self.model, using=self._db)

    def emails(self, user, association=None):
        from apps.MySU.models.association import Association
        assoc_emails = get_objects_for_user(user, 'MySU.board', Association).only('email')
        duty_emails = GroupMembership.objects.emails(user)
        return assoc_emails.union(duty_emails)


class Profile(ContactInfo):
    """
    Information about the users is stored in a profile.
    """
    class Gender(models.TextChoices):
        MALE = "Male", "Male"
        FEMALE = "Female", "Female"
        OTHER = "Other", "Other"

    class Phase(models.TextChoices):
        PHD = "PhD", "PhD"
        MASTER = "Master", "Master"
        PREMASTER = "Premaster", "Premaster"
        BACHELOR = "Bachelor", "Bachelor"
        OTHER = "Other", "Other"

    slug = models.SlugField(default=uuid.uuid4, editable=False)
    last_modified = models.DateTimeField(auto_now=True)
    profilename = models.CharField(max_length=100, blank=True, default="Created by you")
    photo = models.ImageField(upload_to='profile/photo/', blank=True, null=True)
    student_number = models.CharField(max_length=100, blank=True, null=True)
    given_name = models.CharField(max_length=100, blank=True)
    initials = models.CharField(max_length=20, blank=True)
    surname = models.CharField(max_length=100, blank=True)
    email = models.EmailField(null=True, blank=True)
    dutch_speaking = models.BooleanField(default=True)
    gender = models.CharField(max_length=20, choices=Gender.choices, default=Gender.OTHER)
    date_of_birth = models.DateField(null=True, blank=True)
    phase = models.CharField(max_length=20, choices=Phase.choices, default=Phase.BACHELOR)
    read_and_agreed_to_privacy_policy = models.BooleanField(default=False)

    bank_account = models.ForeignKey('financial.BankAccount', related_name='profiles', on_delete=models.PROTECT, null=True, blank=True)
    user = models.ForeignKey('User', related_name='profiles', on_delete=models.PROTECT, null=True, blank=True)
    study = models.ForeignKey('Study', related_name='profile', on_delete=models.PROTECT, null=True, blank=True)

    objects = ProfileManager()

    class Meta:
        verbose_name = "Profile"
        verbose_name_plural = "Profiles"

    def __str__(self):
        return "{} ({})".format(self.full_name, self.student_number)

    @property
    def full_name(self):
        return self.given_name + ' ' + self.surname

    @property
    def formal_name(self):
        return self.initials + ' ' + self.surname
