import uuid

from django.db import models
from multiselectfield import MultiSelectField

from apps.MySU.models import Institution
from apps.MySU.models import Profile
from apps.MySU.models import Study


class StudyRecordQuerySet(models.QuerySet):
    pass


class StudyRecordManager(models.Manager):
    def get_queryset(self):
        return StudyRecordQuerySet(self.model, using=self._db)


class StudyRecord(models.Model):
    """
    Currently this is not really used.

    Every year the student is enrolled at the university is stored as a study record. The fields given here are equal to
    the fields stored by the university.
    """
    class Phase(models.TextChoices):
        PHD = "PhD", "PhD"
        MASTER = "Master", "Master"
        PREMASTER = "Premaster", "Premaster"
        BACHELOR = "Bachelor", "Bachelor"
        OTHER = "Other", "Other"

    slug = models.UUIDField(default=uuid.uuid4, editable=False)
    phase = models.CharField(max_length=20, choices=Phase.choices, default=Phase.BACHELOR)
    year = models.IntegerField()
    date_start = models.DateField()
    date_end = models.DateField()
    is_primary = models.BooleanField()

    study = models.ForeignKey(Study, on_delete=models.PROTECT)
    institution = models.ForeignKey(Institution, related_name='study_records', on_delete=models.PROTECT)
    student = models.ForeignKey(Profile, related_name='study_records', on_delete=models.PROTECT)

    objects = StudyRecordManager()

    class Meta:
        verbose_name = "Study record"
        verbose_name_plural = "Study records"

    def __str__(self):
        return "{student} - {study}".format(student=self.student, study=self.study)
