from rest_framework import serializers

from apps.MySU.models import Association
from apps.MySU.serializers.assoc_other import AssociationNestedGroupsSerializer
from apps.MySU.serializers.base import BaseSerializer


class AssociationSerializerList(BaseSerializer):
    is_board = serializers.SerializerMethodField()

    def get_is_board(self, obj):
        user = self.context['request'].user
        return user.has_perm('board', obj)

    class Meta:
        model = Association
        fields = ('url', 'slug', 'name', 'short_name', 'is_board', 'logo', 'description', 'photo',
                  'full_year_membership_start_date', 'member_needs_iban')
        nested_fields = ['types']
        extra_kwargs = {
            'url': {'lookup_field': 'slug'}
        }


class AssociationSerializer(BaseSerializer):
    board = AssociationNestedGroupsSerializer(source='current_board', many=True, read_only=True)

    class Meta:
        model = Association
        exclude = ("members", "users")
        extra_kwargs = {
            'dissolution_date': {'read_only': True},
            'url': {'lookup_field': 'slug'},
        }
