from rest_framework import serializers
from rest_framework.exceptions import ValidationError

from MySU.fields.association import AssociationField


class BulkEmailSerializer(serializers.Serializer):
    association = AssociationField(view_name='association-detail', lookup_field='slug')
    sent_to_all = serializers.BooleanField()

    def validate(self, attrs):
        attrs = super().validate(attrs)

        # check whether user is board for the objects
        user = self.context['request'].user
        if not user.has_perm("board", attrs["association"]):
            raise ValidationError({"association": "User must be board in order to do bulk email."})

        return attrs
