from drf_writable_nested import mixins
from rest_framework import serializers

from apps.financial.models import BankAccount
from apps.MySU.models import AssociationMembership
from apps.MySU.models import AssociationSpecificData
from apps.MySU.models import Profile
from apps.MySU.serializers.base import BaseSerializer
from apps.MySU.serializers.profile import ProfileSerializer


class BankAccountImportSerializer(mixins.CreateOnlyNestedSerializerMixin, BaseSerializer):
    class Meta:
        model = BankAccount
        fields = ('name', 'iban', 'bic', )


class ProfileImportSerializer(mixins.CreateOnlyNestedSerializerMixin, BaseSerializer):
    bank_account = BankAccountImportSerializer()

    class Meta:
        model = Profile
        exclude = ('photo', 'user')
        nested_fields = ['user']
        lookup_field = 'slug'
        extra_kwargs = {
            'url': {'lookup_field': 'slug'},
            'study': {'lookup_field': 'slug'},
        }


class AssociationSpecificDataImportSerializer(mixins.CreateOnlyNestedSerializerMixin, BaseSerializer):
    name = serializers.ReadOnlyField(source='data_field.name')

    class Meta:
        model = AssociationSpecificData
        fields = ('url', 'slug', 'name', 'association', 'data_field', 'value', 'membership')
        extra_kwargs = {
            'url': {'lookup_field': 'slug'},
            'association': {'lookup_field': 'slug'},
            'data_field': {'lookup_field': 'slug'},
            'membership': {'lookup_field': 'slug'},
        }
        create_only_fields = ('association', 'data_field', 'membership')


class AssociationMemberImportSerializer(mixins.CreateOnlyNestedSerializerMixin, BaseSerializer):
    profile = ProfileImportSerializer()
    specific_data = AssociationSpecificDataImportSerializer(many=True)

    class Meta:
        model = AssociationMembership
        fields = ('url', 'slug', 'profile', 'association', 'date_joined', 'status', 'type', 'specific_data')
        extra_kwargs = {
            'url': {'lookup_field': 'slug'},
            'profile': {'lookup_field': 'slug'},
            'type': {'lookup_field': 'slug'},
            'association': {'lookup_field': 'slug'},
            'status': {'initial': 'Pending'},
        }
        create_only_fields = ('association', 'date_joined', )
