from rest_flex_fields import FlexFieldsModelSerializer
from rest_framework import serializers

from apps.MySU.models import User
from apps.MySU.serializers.association_membership import AssociationMembershipSerializerBase
from apps.MySU.serializers.base import BaseSerializer
from apps.MySU.serializers.profile import ProfileSerializer
from MySU.mixins.related_field import FilterRelatedMixin


class UserSerializerList(serializers.HyperlinkedModelSerializer):
    class Meta:
        model = User
        fields = ('url', 'username')
        extra_kwargs = {
            'url': {'lookup_field': 'slug'},
        }


class UserSerializer(FilterRelatedMixin, FlexFieldsModelSerializer, BaseSerializer):
    association_memberships = serializers.HyperlinkedRelatedField(many=True,
                                                                  read_only=True,
                                                                  view_name='associationmembership-detail',
                                                                  lookup_field="slug")

    class Meta:
        model = User
        fields = ('email', 'preferred_profile', 'association_memberships', 'profiles', 'profile_count')
        extra_kwargs = {
            'url': {'lookup_field': 'slug'},
            'preferred_profile': {'lookup_field': 'slug'},
            'profiles': {'lookup_field': 'slug',
                         'read_only': True},
            'association_memberships': {'lookup_field': 'slug'},
        }
        expandable_fields = {
            'association_memberships': (AssociationMembershipSerializerBase, {"many": True, "read_only": True}),
            'preferred_profile': (ProfileSerializer, {"read_only": True}),
            "profiles": (ProfileSerializer, {"many": True, "read_only": True})
        }

    def filter_preferred_profile(self, queryset):
        user = self.context['request'].user
        return user.profiles.all()
