from django.conf.urls import url
from django.urls import path
from rest_framework.routers import DefaultRouter

from .views.membership_end import MembershipEndView
from .views.membership_resolve import MembershipResolveView
from .views.my_su_password_reset import MySUPasswordResetView
from .views.profile_merge import ProfileMergeView
from .views.society import SocietyView
from .viewsets.association import AssociationViewSet
from .viewsets.association_membership import AssociationMembershipViewSet
from .viewsets.association_specific_data import AssociationSpecificDataViewSet
from .viewsets.association_specific_data_fields import AssociationSpecificDataFieldsViewSet
from .viewsets.django_group import DjangoGroupViewSet
from .viewsets.email import EmailViewSet
from .viewsets.group import GroupViewSet
from .viewsets.group_membership import GroupMembershipViewSet
from .viewsets.group_object_permission import GroupObjectPermissionViewSet
from .viewsets.institution import InstitutionViewSet
from .viewsets.membertype import MemberTypeViewSet
from .viewsets.modules import ModulesViewSet
from .viewsets.news import NewsViewSet
from .viewsets.notification import NotificationViewSet
from .viewsets.permission import PermissionViewSet
from .viewsets.profile import ProfileViewSet
from .viewsets.study import StudyViewSet
from .viewsets.study_record import StudyRecordViewSet
from .viewsets.user import UserViewSet

router = DefaultRouter()
router.register(r'users', UserViewSet)
router.register(r'profiles', ProfileViewSet)
router.register(r'institutions', InstitutionViewSet)
router.register(r'studyrecords', StudyRecordViewSet)
router.register(r'studies', StudyViewSet)
router.register(r'associations', AssociationViewSet)
router.register(r'association_data', AssociationSpecificDataViewSet)
router.register(r'association_data_fields', AssociationSpecificDataFieldsViewSet)
router.register(r'notifications', NotificationViewSet)
router.register(r'memberships', AssociationMembershipViewSet)
router.register(r'membertypes', MemberTypeViewSet)
router.register(r'modules', ModulesViewSet)
router.register(r'news', NewsViewSet)
router.register(r'groups', GroupViewSet)
router.register(r'group_memberships', GroupMembershipViewSet)
router.register(r'emails', EmailViewSet, basename='email')
router.register(r'permissions', PermissionViewSet)
router.register(r'global_permissions', DjangoGroupViewSet, basename="global_permissions")
router.register(r'object_permissions', GroupObjectPermissionViewSet)

urlpatterns = [
    url(r'^profiles/merge', ProfileMergeView.as_view(), name='profile-merge'),
    url(r'^memberships/(?P<slug>[^/.]+)/end', MembershipEndView.as_view(), name='membership-end'),
    url(r'^memberships/resolve', MembershipResolveView.as_view(), name='memberships-resolve'),
    path('account/password_reset/', MySUPasswordResetView.as_view(), name='password_reset'),
    url(r'^statistics/societies', SocietyView.as_view(), name='society_statistics'),
]
