import datetime

from django.core.exceptions import ObjectDoesNotExist
from django.db.models import Q
from django.http import JsonResponse
from django.utils.decorators import method_decorator
from django.views.decorators.csrf import csrf_exempt
from rest_framework import permissions
from rest_framework.views import APIView

from apps.MySU.models import AssociationMembership
from apps.MySU.models import AssociationSpecificData
from apps.MySU.models import GroupMembership
from apps.MySU.models import Profile


@method_decorator(csrf_exempt, name='dispatch')
class MembershipEndView(APIView):
    http_method_names = ['get']
    permission_classes = (permissions.IsAuthenticated,)

    def get(self, request, **kwargs):
        user = request.user

        membership_slug = kwargs["slug"]
        try:
            membership = AssociationMembership.objects.get(slug=membership_slug, user=user)
        except ObjectDoesNotExist:
            return JsonResponse({
                'status': 'failed',
                'reason': "No association exists with slug {} for this user.".format(membership_slug),
            })

        group_memberships = GroupMembership.objects.current(user, membership.association)
        if group_memberships:
            return JsonResponse({
                'status': "failed",
                'reason': "You cannot leave the association, while still being in a committee.",
                'committtees': "{}".format(group_memberships)
            })

        # if not membership.visible_after_date_left:
        #     anonymous_profile = Profile.objects.get(id=1)
        #     # Anonymize association membership
        #     membership.profile = anonymous_profile
        #
        #     # Anonymize group membership(s)
        #     group_memberships = GroupMembership.objects\
        #         .filter(user=user, group__membergroup__association=membership.association)
        #     if group_memberships:
        #         for group_membership in group_memberships:
        #             group_membership.profile = anonymous_profile
        #         GroupMembership.objects.bulk_update(group_memberships, ['profile'])
        #
        #     # Remove specific data
        #     association_specific_data = AssociationSpecificData.objects.filter(membership=membership)
        #     for data in association_specific_data:
        #         data.delete()

        membership.date_left = membership.type.end_date
        membership.save()

        return JsonResponse({
            'status': 'success',
        })
