from django.db.models import Q
from guardian.shortcuts import get_objects_for_user
from rest_framework import viewsets, status
from rest_framework.decorators import action
from rest_framework.permissions import BasePermission
from rest_framework.permissions import IsAuthenticated
from url_filter import filtersets
from rest_framework.response import Response

from apps.MySU.models import Association
from apps.MySU.models import AssociationSpecificData
from apps.MySU.serializers.association_specific_data import AssociationSpecificDataSerializer

"""
----------------------------------------------------------------------------------------------------------------------
 ASSOCIATIONSPECIFICDATA
----------------------------------------------------------------------------------------------------------------------

 GET:       Everyone
 PUT/PATCH: Board
 POST:      Board
 DELETE:    Staff
"""


class AssociationSpecificDataHttpPermissions(BasePermission):
    def has_object_permission(self, request, view, obj):
        if request.method == "PATCH" or request.method == "PUT":  # HANDLES THE UPDATE
            return request.user.has_perm('board', obj.association) or (
                        obj.membership.user == request.user and not obj.data_field.board_only)
        return True

    def has_permission(self, request, view):
        if request.method == "GET":  # HANDLES THE LIST
            return True
        if request.method == "POST":  # HANDLES THE CREATION/INSERTION
            return True
        if request.method == "DELETE":
            return request.user.is_staff
        return True


class AssociationSpecificDataFilterSet(filtersets.ModelFilterSet):
    class Meta:
        model = AssociationSpecificData


class AssociationSpecificDataViewSet(viewsets.ModelViewSet):
    serializer_class = AssociationSpecificDataSerializer
    queryset = AssociationSpecificData.objects.none()
    lookup_field = 'slug'
    permission_classes = (IsAuthenticated, AssociationSpecificDataHttpPermissions,)
    http_method_names = ['get', 'post', 'put', 'patch', 'head', 'options', 'delete']
    filter_class = AssociationSpecificDataFilterSet

    def get_queryset(self):
        queryset = AssociationSpecificData.objects \
            .filter(
            Q(association__in=get_objects_for_user(self.request.user, 'board', Association)) |
            Q(membership__user=self.request.user)) \
            .prefetch_related("association", "membership", "data_field") \
            .distinct()
        return queryset

    def destroy(self, request, *args, **kwargs):
        instance = self.get_object()
        self.perform_destroy(instance)
        return Response(status=status.HTTP_204_NO_CONTENT)
