from django.db.models import Q
from guardian.shortcuts import get_objects_for_user
from rest_framework import status
from rest_framework import viewsets
from rest_framework.permissions import BasePermission
from rest_framework.permissions import IsAuthenticated
from rest_framework.response import Response
from url_filter import filtersets

from apps.MySU.models import Association
from apps.MySU.models import AssociationSpecificDataFields
from apps.MySU.serializers.association_specific_data_fields import AssociationSpecificDataFieldsSerializer

"""
----------------------------------------------------------------------------------------------------------------------
 ASSOCIATIONSPECIFICDATAFIELDS
----------------------------------------------------------------------------------------------------------------------

 GET:       Everyone
 PUT/PATCH: Board
 POST:      Board
"""


class AssociationSpecificDataFieldsHttpPermissions(BasePermission):
    def has_object_permission(self, request, view, obj):
        if request.user.has_perm('board', obj.association):
            return True
        return False

    def has_permission(self, request, view):
        if request.method == "GET":  # HANDLES THE LIST
            return True
        if request.method == "POST":  # HANDLES THE CREATION/INSERTION
            return get_objects_for_user(request.user, 'MySU.board').count() > 0
        return True


class AssociationSpecificDataFieldsFilterSet(filtersets.ModelFilterSet):
    class Meta:
        model = AssociationSpecificDataFields


class AssociationSpecificDataFieldsViewSet(viewsets.ModelViewSet):
    serializer_class = AssociationSpecificDataFieldsSerializer
    queryset = AssociationSpecificDataFields.objects.none()
    lookup_field = 'slug'
    permission_classes = (IsAuthenticated, AssociationSpecificDataFieldsHttpPermissions,)
    http_method_names = ['get', 'post', 'put', 'patch', 'delete', 'head', 'options']
    filter_class = AssociationSpecificDataFieldsFilterSet

    def get_queryset(self):
        return AssociationSpecificDataFields.objects.all()

    def destroy(self, request, *args, **kwargs):
        data_field = self.get_object()
        self.perform_destroy(data_field)
        return Response(status=status.HTTP_204_NO_CONTENT)


