from django.contrib.auth.models import Group
from rest_framework import viewsets
from rest_framework.permissions import BasePermission
from rest_framework.permissions import IsAuthenticated

from apps.MySU.serializers.django_group import DjangoGroupSerializer
from apps.MySU.serializers.django_group import DjangoGroupSerializerList


"""
----------------------------------------------------------------------------------------------------------------------
 GROUP
----------------------------------------------------------------------------------------------------------------------

 GET:       None
 PUT/PATCH: None
 POST:      None
"""


class DjangoGroupHttpPermissions(BasePermission):
    def has_object_permission(self, request, view, obj):
        if request.method == "PATCH" or request.method == "PUT":  # HANDLES THE UPDATE
            return False
        return True

    def has_permission(self, request, view):
        if request.method == "GET":  # HANDLES THE LIST
            return False
        if request.method == "POST":  # HANDLES THE CREATION/INSERTION
            return False
        return True


class DjangoGroupViewSet(viewsets.ModelViewSet):
    queryset = Group.objects.none()
    serializer_class = DjangoGroupSerializer
    http_method_names = ['get', 'put', 'patch', 'head', 'options']
    permission_classes = (IsAuthenticated, DjangoGroupHttpPermissions,)

    def get_serializer_class(self):
        if self.action == 'retrieve':
            return DjangoGroupSerializer
        elif self.action == 'list':
            return DjangoGroupSerializerList
        return DjangoGroupSerializer
