from django.core.mail import send_mail
from guardian.shortcuts import get_objects_for_user
from guardian.shortcuts import Q
from rest_framework import viewsets
from rest_framework.permissions import BasePermission
from rest_framework.permissions import IsAuthenticated
from rest_framework.response import Response

from apps.MySU.models import Profile
from apps.MySU.serializers.email import EmailSentSerializer


"""
----------------------------------------------------------------------------------------------------------------------
 Email
----------------------------------------------------------------------------------------------------------------------

 GET:       Board
 PUT/PATCH: None
 POST:      Board
"""


class EmailHttpPermissions(BasePermission):
    def has_object_permission(self, request, view, obj):
        if request.method == "PATCH" or request.method == "PUT":  # HANDLES THE UPDATE
            return False
        return False

    def has_permission(self, request, view):
        board = get_objects_for_user(request.user, 'MySU.board').count() > 0
        if request.method == "GET":  # HANDLES THE LIST
            return board
        if request.method == "POST":  # HANDLES THE CREATION/INSERTION
            return board
        return False


class EmailViewSet(viewsets.ViewSet):
    permission_classes = (IsAuthenticated, EmailHttpPermissions,)
    http_method_names = ['get', 'list', 'post', 'head', 'options']
    serializer_class = EmailSentSerializer

    def list(self, request):
        queryset = Profile.objects.emails(user=request.user)
        results = list(queryset.values_list('email', flat=True))
        return Response({
            "count": len(results),
            "results": results
        })

    def create(self, request):
        serializer = EmailSentSerializer(data=request.data)
        if serializer.is_valid(raise_exception=True):
            email = serializer.save()
            email.send()
        else:
            Response(serializer.errors, status=400)
        return Response(serializer.validated_data, status=201)
