from guardian.shortcuts import get_objects_for_user
from guardian.shortcuts import Q
from rest_framework import viewsets
from rest_framework.permissions import BasePermission
from rest_framework.permissions import IsAuthenticated
from url_filter import filtersets

from apps.MySU.models import Association
from apps.MySU.models import GroupMembership
from apps.MySU.serializers.group_membership import GroupMembershipSerializer
from apps.MySU.serializers.group_membership import GroupMembershipSerializerList
from MySU.mixins import BulkCreateModelMixin


"""
----------------------------------------------------------------------------------------------------------------------
 GROUPMEMBERSHIP
----------------------------------------------------------------------------------------------------------------------

 GET:       Board or Self
 PUT/PATCH: Board
 POST:      Board
"""


class GroupMembershipHttpPermissions(BasePermission):
    def has_object_permission(self, request, view, obj):
        if request.method == "PATCH" or request.method == "PUT":    # HANDLES THE UPDATE
            return request.user.has_perm('board', obj.association)
        if request.method == "DELETE":                              # HANDLES THE REMOVAL
            return request.user.has_perm('board', obj.association)
        return True

    def has_permission(self, request, view):
        if request.method == "GET":                                 # HANDLES THE LIST
            return True
        if request.method == "POST":                                # HANDLES THE CREATION/INSERTION
            return get_objects_for_user(request.user, 'MySU.board').count() > 0
        if request.method == "DELETE":                              # HANDLES THE CREATION/INSERTION
            return get_objects_for_user(request.user, 'MySU.board').count() > 0
        return True


class GroupMembershipFilterSet(filtersets.ModelFilterSet):
    class Meta:
        model = GroupMembership


class GroupMembershipViewSet(BulkCreateModelMixin, viewsets.ModelViewSet):
    queryset = GroupMembership.objects.none()
    serializer_class = GroupMembershipSerializer
    permission_classes = (IsAuthenticated, GroupMembershipHttpPermissions,)
    http_method_names = ['get', 'post', 'put', 'patch', 'delete', 'head', 'options']
    lookup_field = 'slug'
    ordering_fields = ['slug', 'duty', 'date_joined', 'date_left', 'group']
    ordering = ['slug']
    filter_class = GroupMembershipFilterSet

    def get_serializer(self, *args, **kwargs):
        """
        Return the serializer instance that should be used for validating and
        deserializing input, and for serializing output.
        """
        serializer_class = self.get_serializer_class()
        kwargs['context'] = self.get_serializer_context()
        return serializer_class(*args, **kwargs)

    def get_serializer_class(self):
        return GroupMembershipSerializer

    def get_queryset(self):
        queryset = GroupMembership.objects.filter(
            Q(user=self.request.user) |
            Q(association__in=get_objects_for_user(self.request.user, 'board', Association))
        ).prefetch_related('django_group__group', 'profile')
        return queryset.distinct()
