from rest_framework import viewsets
from rest_framework.permissions import BasePermission
from rest_framework.permissions import IsAuthenticated
from url_filter import filtersets

from apps.MySU.models import Institution
from apps.MySU.serializers.institution import InstitutionSerializer

"""
----------------------------------------------------------------------------------------------------------------------
 INSTITUTION
----------------------------------------------------------------------------------------------------------------------

 GET:       Everyone
 PUT/PATCH: None
 POST:      None
"""


class InstitutionHttpPermissions(BasePermission):
    def has_object_permission(self, request, view, obj):
        if request.method == "PATCH" or request.method == "PUT":  # HANDLES THE UPDATE
            return False
        return True

    def has_permission(self, request, view):
        if request.method == "GET":  # HANDLES THE LIST
            return True
        if request.method == "POST":  # HANDLES THE CREATION/INSERTION
            return False
        return True


class InstitutionFilterSet(filtersets.ModelFilterSet):
    class Meta:
        model = Institution


class InstitutionViewSet(viewsets.ModelViewSet):
    queryset = Institution.objects.all()
    lookup_field = 'slug'
    permission_classes = (IsAuthenticated, InstitutionHttpPermissions,)
    http_method_names = ['get', 'post', 'put', 'patch', 'head', 'options']
    filter_class = InstitutionFilterSet

    def get_serializer_class(self):
        return InstitutionSerializer
