from guardian.shortcuts import get_objects_for_user
from rest_framework import status
from rest_framework import viewsets
from rest_framework.permissions import BasePermission
from rest_framework.permissions import IsAuthenticated
from rest_framework.response import Response
from url_filter import filtersets

from apps.financial.models import MembershipFee
from apps.MySU.models import AssociationMembership
from apps.MySU.models import MemberType
from apps.MySU.serializers.membertype import MemberTypeSerializer

"""
----------------------------------------------------------------------------------------------------------------------
 MEMBERTYPE
----------------------------------------------------------------------------------------------------------------------

 GET:       Board or Self
 PUT/PATCH: Board
 POST:      Board
"""


class MemberTypeHttpPermissions(BasePermission):
    def has_object_permission(self, request, view, obj):
        if request.method == "PATCH" or request.method == "PUT":  # HANDLES THE UPDATE
            return request.user.has_perm('board', obj.association)
        if request.method == "DELETE":  # HANDLES THE REMOVAL
            return request.user.has_perm('board', obj.association)
        return True

    def has_permission(self, request, view):
        if request.method == "GET":  # HANDLES THE LIST
            return True
        if request.method == "POST":  # HANDLES THE CREATION/INSERTION
            return get_objects_for_user(request.user, 'MySU.board').count() > 0
        return True


class MembershipFeeFilterSet(filtersets.ModelFilterSet):
    class Meta:
        model = MemberType


class MemberTypeViewSet(viewsets.ModelViewSet):
    queryset = MemberType.objects.none()
    serializer_class = MemberTypeSerializer
    permission_classes = (IsAuthenticated, MemberTypeHttpPermissions,)
    http_method_names = ['get', 'post', 'put', 'patch', 'delete', 'head', 'options']
    lookup_field = 'slug'
    ordering_fields = ['slug', 'type', 'association']
    ordering = ['order']
    filter_class = MembershipFeeFilterSet

    def get_queryset(self):
        queryset = MemberType.objects.all()
        return queryset

    def destroy(self, request, *args, **kwargs):
        member_type = self.get_object()

        # update memberships with this member type to the new supplied member type.
        new_member_type_slug = self.request.data.get("change_membertype_to", None)
        if not new_member_type_slug:
            return Response(status=status.HTTP_400_BAD_REQUEST, data="delete needs required field 'change_membertype_to'.")
        new_member_type = MemberType.objects.get(slug=new_member_type_slug)
        if not new_member_type:
            return Response(status=status.HTTP_400_BAD_REQUEST, data="Membertype passed using field 'change_membertype_to' does not exist.")
        AssociationMembership.objects.filter(type=member_type).update(type=new_member_type)
        AssociationMembership.objects.filter(new_type=member_type).update(new_type=new_member_type)

        # set the member type to disabled to keep the database consistent, or remove if that is an option.
        has_fees_associated_to_it = MembershipFee.objects.filter(membertype=member_type).exists()
        if has_fees_associated_to_it:
            member_type.disabled = True
            member_type.save()
        else:
            self.perform_destroy(member_type)
        return Response(status=status.HTTP_204_NO_CONTENT)
