from rest_framework import viewsets
from rest_framework.permissions import BasePermission
from rest_framework.permissions import IsAuthenticated
from url_filter import filtersets

from apps.MySU.models import Modules
from apps.MySU.serializers.modules import ModulesSerializer

"""
----------------------------------------------------------------------------------------------------------------------
 MODULES
----------------------------------------------------------------------------------------------------------------------

 GET:       Everyone
 PUT/PATCH: None
 POST:      None
"""


class ModulesHttpPermissions(BasePermission):
    def has_object_permission(self, request, view, obj):
        if request.method == "PATCH" or request.method == "PUT":  # HANDLES THE UPDATE
            return request.user.has_perm('board', obj.association)
        return True

    def has_permission(self, request, view):
        if request.method == "GET":  # HANDLES THE LIST
            return True
        if request.method == "POST":  # HANDLES THE CREATION/INSERTION
            return False
        return True


class ModulesFilterSet(filtersets.ModelFilterSet):
    class Meta:
        model = Modules


class ModulesViewSet(viewsets.ModelViewSet):
    queryset = Modules.objects.all()
    lookup_field = 'slug'
    permission_classes = (IsAuthenticated, ModulesHttpPermissions,)
    http_method_names = ['get', 'patch', 'put', 'head', 'options']
    filter_class = ModulesFilterSet

    def get_serializer_class(self):
        return ModulesSerializer
