from guardian.shortcuts import get_objects_for_user
from guardian.shortcuts import Q
from rest_framework import viewsets
from rest_framework.permissions import BasePermission
from rest_framework.permissions import IsAuthenticated
from url_filter import filtersets

from apps.MySU.models import News, Association
from apps.MySU.serializers.news import NewsSerializer

"""
----------------------------------------------------------------------------------------------------------------------
 NEWS
----------------------------------------------------------------------------------------------------------------------

 GET:       Board or Self
 PUT/PATCH: Board
 POST:      Board
"""


class NewsHttpPermissions(BasePermission):
    def has_object_permission(self, request, view, obj):
        if request.method == "PATCH" or request.method == "PUT":    # HANDLES THE UPDATE
            if obj.association:
                return request.user.has_perm('board', obj.association)
            return False
        if request.method == "DELETE":                              # HANDLES THE UPDATE
            if obj.association:
                return request.user.has_perm('board', obj.association)
            return False
        return True

    def has_permission(self, request, view):
        if request.method == "GET":                                 # HANDLES THE LIST
            return True
        if request.method == "POST":                                # HANDLES THE CREATION/INSERTION
            return get_objects_for_user(request.user, 'MySU.board').count() > 0
        return True


class NewsFilterSet(filtersets.ModelFilterSet):
    class Meta:
        model = News


class NewsViewSet(viewsets.ModelViewSet):
    queryset = News.objects.none()
    serializer_class = NewsSerializer
    permission_classes = (IsAuthenticated, NewsHttpPermissions,)
    http_method_names = ['get', 'post', 'put', 'patch', 'head', 'delete', 'options']
    lookup_field = 'slug'
    ordering_fields = ['slug', 'date', 'association']
    ordering = ['date']
    filter_class = NewsFilterSet

    def get_queryset(self):
        associations = Association.objects.associations_where_user_is_member(self.request.user)
        queryset = News.objects \
            .filter(Q(association__in=associations))
        return queryset.distinct()

