from rest_framework import viewsets
from rest_framework.permissions import BasePermission
from rest_framework.permissions import IsAuthenticated
from url_filter import filtersets

from apps.MySU.models import Notification
from apps.MySU.serializers.notification import NotificationSerializer

"""
----------------------------------------------------------------------------------------------------------------------
 NOTIFICATION
----------------------------------------------------------------------------------------------------------------------

 GET:       Self
 PUT/PATCH: Everyone
 POST:      Board
"""


class NotificationHttpPermissions(BasePermission):
    def has_object_permission(self, request, view, obj):
        if request.method == "PATCH" or request.method == "PUT":  # HANDLES THE UPDATE
            return False
        return True

    def has_permission(self, request, view):
        if request.method == "GET":  # HANDLES THE LIST
            return True
        if request.method == "POST":  # HANDLES THE CREATION/INSERTION
            return False
        return True


class NotificationFilterSet(filtersets.ModelFilterSet):
    class Meta:
        model = Notification


class NotificationViewSet(viewsets.ModelViewSet):
    queryset = Notification.objects.none()
    serializer_class = NotificationSerializer
    lookup_field = 'slug'
    permission_classes = (IsAuthenticated, NotificationHttpPermissions,)
    http_method_names = ['get', 'delete', 'head', 'options']
    filter_class = NotificationFilterSet

    def get_queryset(self):
        queryset = Notification.objects.filter(receiver=self.request.user.preferred_profile_id)
        return queryset
