from django.contrib.auth.models import Permission
from rest_framework import viewsets
from rest_framework.permissions import BasePermission
from rest_framework.permissions import IsAuthenticated
from url_filter import filtersets

from apps.MySU.serializers.permission import PermissionSerializer
from apps.MySU.serializers.permission import PermissionSerializerList


"""
----------------------------------------------------------------------------------------------------------------------
 PERMISSION
----------------------------------------------------------------------------------------------------------------------

 GET:       None
 PUT/PATCH: None
 POST:      None
"""


class PermissionHttpPermissions(BasePermission):
    def has_object_permission(self, request, view, obj):
        if request.method == "PATCH" or request.method == "PUT":  # HANDLES THE UPDATE
            return False
        return True

    def has_permission(self, request, view):
        if request.method == "GET":  # HANDLES THE LIST
            return False
        if request.method == "POST":  # HANDLES THE CREATION/INSERTION
            return False
        return True


class PermissionFilterSet(filtersets.ModelFilterSet):
    class Meta:
        model = Permission


class PermissionViewSet(viewsets.ModelViewSet):
    queryset = Permission.objects.none()
    serializer_class = PermissionSerializer
    ordering_fields = ('id', 'name', "codename")
    ordering = ('id',)
    permission_classes = (IsAuthenticated, PermissionHttpPermissions,)
    http_method_names = ['get', 'post', 'put', 'patch', 'head', 'options']
    filter_class = PermissionFilterSet

    def get_serializer_class(self):
        if self.action == 'retrieve':
            return PermissionSerializer
        elif self.action == 'list':
            return PermissionSerializerList
        return PermissionSerializer
