from guardian.shortcuts import get_objects_for_user
from guardian.shortcuts import Q
from rest_framework import viewsets
from rest_framework.permissions import BasePermission
from rest_framework.permissions import IsAuthenticated
from url_filter import filtersets

from apps.MySU.models import Association
from apps.MySU.models import AssociationMembership
from apps.MySU.models import Profile
from apps.MySU.serializers.profile import ProfileSerializer
from apps.MySU.serializers.profile import ProfileSerializerList

"""
----------------------------------------------------------------------------------------------------------------------
 PROFILE
----------------------------------------------------------------------------------------------------------------------

 GET:       Board or Self
 PUT/PATCH: Self
 POST:      Self
"""


class ProfileHttpPermissions(BasePermission):
    def has_object_permission(self, request, view, obj):
        if request.method == "PATCH" or request.method == "PUT":  # HANDLES THE UPDATE
            if obj.user == request.user:
                return True
            associations = get_objects_for_user(request.user, 'board', klass=Association)
            return obj.association_memberships.filter(status=AssociationMembership.Status.CLAIMED, association__in=associations).exists()
        return True

    def has_permission(self, request, view):
        if request.method == "GET":  # HANDLES THE LIST
            return True
        if request.method == "POST":  # HANDLES THE CREATION/INSERTION
            return True
        return True


class ProfileFilterSet(filtersets.ModelFilterSet):
    class Meta:
        model = Profile


class ProfileViewSet(viewsets.ModelViewSet):
    queryset = Profile.objects.none()
    permission_classes = (IsAuthenticated, ProfileHttpPermissions,)
    http_method_names = ['get', 'post', 'put', 'patch', 'head', 'options']
    lookup_field = 'slug'
    ordering_fields = ['slug', 'profilename', 'initials', 'surname', 'student_number', 'given_name']
    ordering = ['slug']
    filter_class = ProfileFilterSet

    def get_serializer_class(self):
        return ProfileSerializer

    def get_queryset(self):
        associations = get_objects_for_user(self.request.user, 'board', klass=Association)
        queryset = Profile.objects.filter(
            Q(user=self.request.user) | Q(association_memberships__association__in=associations, associations__memberships__financial_obligations_satisfied=False)
        ).distinct()
        return queryset
