from rest_framework import viewsets
from rest_framework.permissions import BasePermission
from rest_framework.permissions import IsAuthenticated
from url_filter import filtersets

from apps.MySU.models import Study
from apps.MySU.serializers.study import StudySerializer


"""
----------------------------------------------------------------------------------------------------------------------
 STUDY
----------------------------------------------------------------------------------------------------------------------

 GET:       Everyone
 PUT/PATCH: None
 POST:      None
"""


class StudyHttpPermissions(BasePermission):
    def has_object_permission(self, request, view, obj):
        if request.method == "GET":  # HANDLES THE LIST
            return True
        return False

    def has_permission(self, request, view):
        if request.method == "GET":  # HANDLES THE LIST
            return True
        if request.method == "POST":  # HANDLES THE CREATION/INSERTION
            return False
        return True


class StudyFilterSet(filtersets.ModelFilterSet):
    class Meta:
        model = Study


class StudyViewSet(viewsets.ModelViewSet):
    queryset = Study.objects.all()
    lookup_field = 'slug'
    permission_classes = (IsAuthenticated, StudyHttpPermissions,)
    http_method_names = ['get', 'post', 'put', 'patch', 'head', 'options']
    filter_class = StudyFilterSet

    def get_serializer_class(self):
        return StudySerializer
