from rest_framework import viewsets
from rest_framework.permissions import BasePermission
from rest_framework.permissions import IsAuthenticated
from url_filter import filtersets

from apps.MySU.models import StudyRecord
from apps.MySU.serializers.study_record import StudyRecordSerializer
from apps.MySU.serializers.study_record import StudyRecordSerializerList

"""
----------------------------------------------------------------------------------------------------------------------
 STUDYRECORD
----------------------------------------------------------------------------------------------------------------------

 GET:       Board or Self
 PUT/PATCH: None
 POST:      None
"""


class StudyRecordHttpPermissions(BasePermission):
    def has_object_permission(self, request, view, obj):
        if request.method == "PATCH" or request.method == "PUT":  # HANDLES THE UPDATE
            return False
        return True

    def has_permission(self, request, view):
        if request.method == "GET":  # HANDLES THE LIST
            return True
        if request.method == "POST":  # HANDLES THE CREATION/INSERTION
            return False
        return True


class StudyRecordFilterSet(filtersets.ModelFilterSet):
    class Meta:
        model = StudyRecord


class StudyRecordViewSet(viewsets.ModelViewSet):
    queryset = StudyRecord.objects.all()
    serializer_class = StudyRecordSerializer
    permission_classes = (IsAuthenticated, StudyRecordHttpPermissions,)
    http_method_names = ['get', 'post', 'put', 'patch', 'head', 'options']
    lookup_field = 'slug'
    ordering_fields = ['study', 'phase', 'date_start', 'date_end', 'institution']
    ordering = ['study']
    filter_class = StudyRecordFilterSet

    def get_serializer_class(self):
        if self.action == 'list':
            return StudyRecordSerializerList
        return StudyRecordSerializer

    def get_queryset(self):
        queryset = StudyRecord.objects.filter(student=self.request.user.preferred_profile_id)
        return queryset
