from django.db.models import Prefetch
from rest_framework import viewsets, status
from rest_framework.permissions import BasePermission
from rest_framework.permissions import IsAuthenticated
from rest_framework.response import Response
from url_filter import filtersets

from apps.MySU.models import AssociationMembership
from apps.MySU.models import User
from apps.MySU.serializers.user import UserSerializer
from apps.MySU.serializers.user import UserSerializerList

"""
----------------------------------------------------------------------------------------------------------------------
 USER
----------------------------------------------------------------------------------------------------------------------

 GET:       Self
 PUT/PATCH: Self
 POST:      None
"""


class UserHttpPermissions(BasePermission):
    def has_object_permission(self, request, view, obj):
        if request.method == "PATCH" or request.method == "PUT":  # HANDLES THE UPDATE
            return obj == request.user
        return True

    def has_permission(self, request, view):
        if request.method == "GET":  # HANDLES THE LIST
            return True
        if request.method == "POST":  # HANDLES THE CREATION/INSERTION
            return False
        return True


class UserFilterSet(filtersets.ModelFilterSet):
    class Meta:
        model = User


class UserViewSet(viewsets.ModelViewSet):
    queryset = User.objects.none()
    lookup_field = 'slug'
    permission_classes = (IsAuthenticated, UserHttpPermissions,)
    http_method_names = ['get', 'post', 'put', 'patch', 'head', 'options']
    ordering_fields = ['username']
    ordering = ['username']
    filter_class = UserFilterSet

    def get_serializer_class(self):
        if self.action == 'list':
            return UserSerializerList
        return UserSerializer

    def get_object(self):
        if self.kwargs['slug'] == 'me':
            self.kwargs['slug'] = self.request.user.slug
        return super().get_object()

    def get_queryset(self):
        queryset = User.objects.filter(username=self.request.user.username)
        token = super().get_authenticators()
        print(token)
        membership_subquery = AssociationMembership.objects.filter(user=self.request.user)
        membership_subquery = membership_subquery\
            .select_related('association', 'new_type', 'type', 'profile', 'profile__study')\
            .prefetch_related('specific_data')

        queryset = queryset.prefetch_related(
            Prefetch('associationmembership_set', queryset=membership_subquery, to_attr="association_memberships"),
            "profiles"
        )
        queryset = queryset.select_related("preferred_profile")
        return queryset
