import uuid

from django.core.exceptions import ValidationError
from django.db import models

from apps.MySU.models import Association
from apps.MySU.models import Profile
from apps.MySU.models import User


class MatchingQuerySet(models.QuerySet):
    pass


class MatchingManager(models.Manager):
    def get_queryset(self):
        return MatchingQuerySet(self.model, using=self._db)


class Matching(models.Model):
    """
    Sometimes association need or want to pair members. This model represent that. It is a match between two members.
    """
    slug = models.SlugField(default=uuid.uuid4, editable=False)
    email_send = models.BooleanField(default=False)

    association = models.ForeignKey(Association, related_name='matchings', on_delete=models.CASCADE)
    left_profile = models.ForeignKey(Profile, related_name='left_matchings', on_delete=models.CASCADE)
    right_profile = models.ForeignKey(Profile, related_name='right_matchings', on_delete=models.CASCADE)

    objects = MatchingManager()

    class Meta:
        verbose_name = "Matching"
        verbose_name_plural = "Matchings"
        unique_together = ["association", "left_profile", "right_profile"]

    def __str__(self):
        return str(self.slug)

    def clean(self):
        if self.left_profile is self.right_profile:
            raise ValidationError("Right Profile must be different from the Left Profile")
        if self.association not in self.left_profile.associations.all():
            raise ValidationError("Left Profile is not a member of association")
        if self.association not in self.right_profile.associations.all():
            raise ValidationError("Right Profile is not a member of association")
