from django.core.exceptions import ValidationError
from rest_framework import serializers

from apps.buddy.models import Matching
from apps.MySU.serializers.base import BaseSerializer
from MySU.fields.association import AssociationField


class MatchingSerializer(BaseSerializer):
    association = AssociationField(view_name='association-detail', lookup_field='slug')
    left_slug = serializers.ReadOnlyField(source='left_profile.slug')
    right_slug = serializers.ReadOnlyField(source='right_profile.slug')
    left_name = serializers.ReadOnlyField(source='left_profile.full_name')
    right_name = serializers.ReadOnlyField(source='right_profile.full_name')
    left_email = serializers.ReadOnlyField(source='left_profile.email')
    right_email = serializers.ReadOnlyField(source='right_profile.email')

    class Meta:
        model = Matching
        fields = ('url', 'slug', 'association', 'left_slug', 'left_name', 'right_slug', 'right_name', 'left_profile',
                  'right_profile', 'left_email', 'right_email', 'email_send')
        create_only_fields = ('association', 'left_profile', 'right_profile')
        nested_fields = ['association']
        extra_kwargs = {
            'url': {'lookup_field': 'slug'},
            'association': {'lookup_field': 'slug'},
            'left_profile': {'lookup_field': 'slug'},
            'right_profile': {'lookup_field': 'slug'}
        }

    def validate(self, attrs):
        errors = []
        if not self.instance:
            association = attrs['association']
            left_profile = attrs['left_profile']
            right_profile = attrs['right_profile']
            if left_profile is right_profile:
                errors.append({"profiles": "Right Profile must be different from the Left Profile"})
            if association not in left_profile.associations.all():
                raise ValidationError({"left_profile": "Left Profile is not a member of association"})
            if association not in right_profile.associations.all():
                raise ValidationError({"right_profile": "Right Profile is not a member of association"})

        if errors:
            raise ValidationError(errors)
        return super(MatchingSerializer, self).validate(attrs)
