from django.core.mail import send_mail
from rest_framework import permissions
from rest_framework import status
from rest_framework.response import Response
from rest_framework.views import APIView

from apps.buddy.models import Matching
from apps.MySU.serializers.bulk_email import BulkEmailSerializer


class EmailMatchingsView(APIView):
    http_method_names = ['post']
    permission_classes = (permissions.IsAuthenticated,)

    def post(self, request, format=None):
        serializer = BulkEmailSerializer(data=request.data, context={"request": request})
        if serializer.is_valid():
            data = serializer.validated_data
            matchings_for_association = Matching.objects.filter(association=data["association"])
            if data["sent_to_all"]:
                matchings_to_be_emailed = matchings_for_association
            else:
                matchings_to_be_emailed = matchings_for_association.filter(email_send=False)
            for matching in matchings_to_be_emailed:
                send_mail(
                    "You have been matched",
                    "Hi {}\n\nYou have been matched with {} by {}. You can contact your match by email using {}.\n\nKind regards,\n\nThe MySU team"
                    .format(matching.left_profile.given_name, matching.association.short_name,
                            matching.right_profile.full_name, matching.right_profile.email),
                    "no-reply@su.utwente.nl",
                    [matching.left_profile.email]
                )
                send_mail(
                    "You have been matched",
                    "Hi {}\n\nYou have been matched with {} by {}. You can contact your match by email using {}.\n\nKind regards,\n\nThe MySU team"
                    .format(matching.right_profile.given_name, matching.association.short_name,
                            matching.left_profile.full_name, matching.left_profile.email),
                    "no-reply@su.utwente.nl",
                    [matching.right_profile.email]
                )
                matching.email_send = True
            Matching.objects.bulk_update(matchings_to_be_emailed, ["email_send"], 50)
            return Response({"emails send": len(matchings_to_be_emailed)}, status=status.HTTP_200_OK)
        return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)
