from django.db.models import Q
from guardian.shortcuts import get_objects_for_user
from rest_framework import viewsets
from rest_framework.permissions import BasePermission
from rest_framework.permissions import IsAuthenticated
from url_filter import filtersets

from apps.buddy.models import Matching
from apps.buddy.serializers.matching import MatchingSerializer
from apps.MySU.models import Association
from MySU.mixins import BulkCreateModelMixin
from MySU.mixins import BulkDestroyModelMixin

"""
----------------------------------------------------------------------------------------------------------------------
 MATCHING
----------------------------------------------------------------------------------------------------------------------

 GET:       Board or self
 PUT/PATCH: Board
 POST:      Board
"""


class MatchingHttpPermissions(BasePermission):
    def has_object_permission(self, request, view, obj):
        if request.method == "PATCH" or request.method == "PUT":
            return request.user.has_perm('board', obj.association)
        # This will not be called for the bulk delete operation.
        if request.method == "DELETE":  # HANDLES THE UPDATE
            return request.user.has_perm('board', obj.association)
        return True

    def has_permission(self, request, view):
        if request.method == "GET":  # HANDLES THE LIST
            return True
        if request.method == "POST":  # HANDLES THE CREATION/INSERTION
            return get_objects_for_user(request.user, 'MySU.board').count() > 0
        if request.method == "DELETE":  # HANDLES THE DELETION
            return get_objects_for_user(request.user, 'MySU.board').count() > 0
        return True


class MatchingFilterSet(filtersets.ModelFilterSet):
    class Meta:
        model = Matching


class MatchingViewSet(BulkCreateModelMixin, BulkDestroyModelMixin, viewsets.ModelViewSet):
    queryset = Matching.objects.none()
    serializer_class = MatchingSerializer
    permission_classes = (IsAuthenticated, MatchingHttpPermissions,)
    http_method_names = ['get', 'post', 'head', 'options', 'delete']
    lookup_field = 'slug'
    ordering_fields = ['slug', 'association']
    ordering = ['slug']
    filter_class = MatchingFilterSet

    def get_queryset(self):
        queryset = Matching.objects.filter(
            Q(association__in=get_objects_for_user(self.request.user, 'board', Association)) |
            Q(left_profile=self.request.user.preferred_profile) |
            Q(right_profile=self.request.user.preferred_profile)
        )
        return queryset
