import uuid

from django.core.exceptions import ValidationError
from django.core.validators import MinValueValidator
from django.db import models

from apps.calendar.models import Event
from apps.MySU.models import Association


class EnrollmentOptionQuerySet(models.QuerySet):
    pass


class EnrollmentOptionManager(models.Manager):
    def get_queryset(self):
        return EnrollmentOptionQuerySet(self.model, using=self._db)


class EnrollmentOption(models.Model):
    """
    In order to enroll for an event the member needs options to enroll. This model provides the options for an event a
    member can enroll for.
    """
    slug = models.SlugField(default=uuid.uuid4, editable=False)
    name = models.CharField(max_length=100, blank=False)
    participation_fee = models.DecimalField(validators=[MinValueValidator(0.0)], decimal_places=2, max_digits=5)
    description = models.TextField(blank=True)

    association = models.ForeignKey(Association, on_delete=models.PROTECT, related_name='enrollment_options')
    event = models.ForeignKey(Event, on_delete=models.CASCADE, related_name='enrollment_options')

    objects = EnrollmentOptionManager()

    class Meta:
        unique_together = ('name', 'event')
        verbose_name = "Enrollment Option"
        verbose_name_plural = "Enrollment Options"

    def __str__(self):
        return super().__str__()

    def clean(self):
        if self.association != self.event.association:
            raise ValidationError("Event not linked to association")
