import uuid

from django.db import models

from apps.MySU.models import Association


class EventTypeQuerySet(models.QuerySet):
    pass


class EventTypeManager(models.Manager):
    def get_queryset(self):
        return EventTypeQuerySet(self.model, using=self._db)


class EventType(models.Model):
    """
    Event can be categorized. Its categories can be dependent on the association. This model encodes what type an event
    can be, e.g. a sports event can have event type "sports".
    """
    slug = models.SlugField(default=uuid.uuid4, editable=False)
    type = models.CharField(max_length=200)

    association = models.ForeignKey(Association, on_delete=models.PROTECT, related_name='event_types')

    objects = EventTypeManager()

    class Meta:
        verbose_name = "Event Type"
        verbose_name_plural = "Event Types"

    def __str__(self):
        return "{type}".format(type=self.type)
