from guardian.shortcuts import get_objects_for_user
from rest_framework import status
from rest_framework import viewsets
from rest_framework.permissions import BasePermission
from rest_framework.permissions import IsAuthenticated
from rest_framework.response import Response
from url_filter import filtersets

from apps.calendar.models import EnrollmentOption
from apps.calendar.serializers.enrollment_option import EnrollmentOptionSerializer


"""
----------------------------------------------------------------------------------------------------------------------
 EnrollmentOption
----------------------------------------------------------------------------------------------------------------------

 GET:       Everyone
 PUT/PATCH: Board
 POST:      Board
"""


class EnrollmentOptionHttpPermissions(BasePermission):
    def has_object_permission(self, request, view, obj):
        if request.method == "PATCH" or request.method == "PUT":  # HANDLES THE UPDATE
            return request.user.has_perm('board', obj.association)
        if request.method == "DELETE":
            return request.user.has_perm('board', obj.association)
        return True

    def has_permission(self, request, view):
        if request.method == "GET":  # HANDLES THE LIST
            return True
        if request.method == "POST":  # HANDLES THE CREATION/INSERTION
            return get_objects_for_user(request.user, 'MySU.board').count() > 0
        return True


class EnrollmentOptionFilterSet(filtersets.ModelFilterSet):
    class Meta:
        model = EnrollmentOption


class EnrollmentOptionViewSet(viewsets.ModelViewSet):
    queryset = EnrollmentOption.objects.none()
    serializer_class = EnrollmentOptionSerializer
    permission_classes = (IsAuthenticated, EnrollmentOptionHttpPermissions,)
    http_method_names = ['get', 'post', 'put', 'patch', 'delete', 'head', 'options']
    lookup_field = 'slug'
    ordering_fields = ['slug']
    ordering = ['slug']
    filter_class = EnrollmentOptionFilterSet

    def get_queryset(self):
        queryset = EnrollmentOption.objects.filter(
            association__users=self.request.user
        ).distinct()
        return queryset

    def destroy(self, request: 'Request', *args: 'Any', **kwargs: 'Any') -> 'Response':
        enrollment_option = self.get_object()
        if enrollment_option.enrollments.exists():
            return Response(
                status=status.HTTP_400_BAD_REQUEST,
                data={"status": "Only enrollment options without enrollments can be deleted"}
            )
        self.perform_destroy(enrollment_option)
        return Response(status=status.HTTP_204_NO_CONTENT)
