from guardian.shortcuts import get_objects_for_user
from guardian.shortcuts import Q
from rest_framework import viewsets
from rest_framework.permissions import BasePermission
from rest_framework.permissions import IsAuthenticated
from url_filter import filtersets

from apps.MySU.models import Association
from apps.calendar.models import EventType
from apps.calendar.serializers.eventtype import EventTypeSerializer


"""
----------------------------------------------------------------------------------------------------------------------
 EventType
----------------------------------------------------------------------------------------------------------------------

 GET:       Everyone
 PUT/PATCH: Board
 POST:      Board
"""


class EventTypeHttpPermissions(BasePermission):
    def has_object_permission(self, request, view, obj):
        if request.method == "PATCH" or request.method == "PUT":  # HANDLES THE UPDATE
            return request.user.has_perm('board', obj.association)
        return True

    def has_permission(self, request, view):
        if request.method == "GET":  # HANDLES THE LIST
            return True
        if request.method == "POST":  # HANDLES THE CREATION/INSERTION
            return get_objects_for_user(request.user, 'MySU.board').count() > 0
        return True


class EventTypeFilterSet(filtersets.ModelFilterSet):
    class Meta:
        model = EventType


class EventTypeViewSet(viewsets.ModelViewSet):
    queryset = EventType.objects.none()
    serializer_class = EventTypeSerializer
    permission_classes = (IsAuthenticated, EventTypeHttpPermissions,)
    http_method_names = ['get', 'post', 'put', 'patch', 'head', 'options']
    lookup_field = 'slug'
    ordering_fields = ['slug']
    ordering = ['slug']
    filter_class = EventTypeFilterSet

    def get_queryset(self):
        associations = Association.objects.associations_where_user_is_member(user=self.request.user)
        queryset = EventType.objects.filter(Q(association__in=associations)).distinct()
        return queryset

