from django.db import migrations


def forwards_func(apps, schema_editor):
    # We get the model from the versioned app registry;
    # if we directly import it, it'll be the wrong version
    DebtEntry = apps.get_model("financial", "DebtEntry")
    db_alias = schema_editor.connection.alias
    debt_entries = DebtEntry.objects.using(db_alias).all()
    for debt_entry in debt_entries:
        debt_entry.bank_account = debt_entry.profile.bank_account
    DebtEntry.objects.using(db_alias).bulk_update(debt_entries, ['bank_account'])

def reverse_func(apps, schema_editor):
    # forwards_func() sets the bank accounts on the debt entries based on the profile,
    # so reverse_func() should null the bank accounts on the debt entries.
    DebtEntry = apps.get_model("financial", "DebtEntry")
    db_alias = schema_editor.connection.alias
    debt_entries = DebtEntry.objects.using(db_alias).all()
    for debt_entry in debt_entries:
        debt_entry.bank_account = None
    DebtEntry.objects.using(db_alias).bulk_update(debt_entries, ['bank_account'])

class Migration(migrations.Migration):

    dependencies = [
        ('financial', '0005_auto_20211212_0846'),
        ('MySU', '0034_populate_bank_account'),
    ]

    operations = [
        migrations.RunPython(forwards_func, reverse_func),
    ]
