from django.db import migrations


def forwards_func(apps, schema_editor):
    # We get the model from the versioned app registry;
    # if we directly import it, it'll be the wrong version
    BankAccount = apps.get_model("financial", "BankAccount")
    db_alias = schema_editor.connection.alias
    bank_accounts = BankAccount.objects.using(db_alias).prefetch_related("profiles").all()
    for bank_account in bank_accounts:
        bank_account.name = bank_account.profiles.first().initials + " " + bank_account.profiles.first().surname
    BankAccount.objects.using(db_alias).bulk_update(bank_accounts, ['name'])

def reverse_func(apps, schema_editor):
    # forwards_func() set the name based on the initials and surname of the profile.
    # so reverse_func() should do nothing.
    pass

class Migration(migrations.Migration):

    dependencies = [
        ('financial', '0007_bankaccount_name'),
    ]

    operations = [
        migrations.RunPython(forwards_func, reverse_func),
    ]
