import uuid

from computedfields.models import computed
from computedfields.models import ComputedFieldsModel
from django.core.exceptions import ValidationError
from django.core.validators import MinValueValidator
from django.db import models

from apps.MySU.models import Association


class DebtCollectionQuerySet(models.QuerySet):
    def fees(self):
        pass

    def events(self):
        pass


class DebtCollectionManager(models.Manager):
    def get_queryset(self):
        return DebtCollectionQuerySet(self.model, using=self._db)

    def fees(self):
        return self.get_queryset().fees()

    def events(self):
        return self.get_queryset().events()


class DebtCollection(ComputedFieldsModel, models.Model):
    """
    Many action by members give rise to a financial obligation. To collect this a debt collection (incasso) is
    used. Every member that owns the association money will get a single debt_entry. This debt collection will group
    the debt entries together.
    """
    slug = models.SlugField(default=uuid.uuid4, editable=False)
    creation_date = models.DateTimeField(auto_now_add=True)
    debt_collection_date = models.DateTimeField()
    locked = models.BooleanField(default=False)

    association = models.ForeignKey(Association, related_name='debt_collections', on_delete=models.PROTECT)

    # history = HistoricalRecords()

    objects = DebtCollectionManager()

    @computed(models.DecimalField(validators=[MinValueValidator(0.0)], decimal_places=2, max_digits=5, default=0),
              depends=[['debt_entries', ['total_price']], ['association', ['id']]])
    def amount(self):
        return sum([entry.total_price for entry in self.debt_entries.all()])

    class Meta:
        verbose_name = "Debt collection"
        verbose_name_plural = "Debt collections"
        ordering = ["debt_collection_date"]

    def __str__(self):
        return str(self.slug)

    def clean(self):
        if self.debt_collection_date < self.creation_date:
            raise ValidationError("Cannot collect the debt before the debt collection has been created")
