import uuid

from django.core.validators import MinValueValidator
from django.db import models

from apps.MySU.models import AssociationMembership


class DebtEntryQuerySet(models.QuerySet):
    pass


class DebtEntryManager(models.Manager):
    def get_queryset(self):
        return DebtEntryQuerySet(self.model, using=self._db)


class DebtEntry(models.Model):
    """
    Many action by members give rise to a financial obligation. In order to collected this, the financial obligations
    for a member are linked to debt entries.
    """
    slug = models.SlugField(default=uuid.uuid4, editable=False)
    total_price = models.DecimalField(validators=[MinValueValidator(0.0)], decimal_places=2, max_digits=5)
    paid_by = models.CharField(max_length=20, choices=AssociationMembership.PayBy.choices, default=AssociationMembership.PayBy.DEBT_COLLECTION)
    cancelled = models.BooleanField(default=False)
    paid_after_cancellation = models.BooleanField(default=False)

    association = models.ForeignKey("MySU.Association", related_name='debt_entries', on_delete=models.CASCADE)
    sepa_mandate = models.ForeignKey("financial.SepaMandate", related_name='debt_entries', on_delete=models.CASCADE, null=True, blank=True)
    debt_collection = models.ForeignKey("financial.DebtCollection", related_name='debt_entries', on_delete=models.CASCADE, null=True, blank=True)
    bank_account = models.ForeignKey("financial.BankAccount", related_name='bank_accounts', on_delete=models.PROTECT, null=True, blank=True)
    profile = models.ForeignKey("MySU.Profile", related_name='debt_entries', on_delete=models.PROTECT)

    objects = DebtEntryManager()

    class Meta:
        verbose_name = "Debt Entry"
        verbose_name_plural = "Debt Entries"

    def __str__(self):
        return str(self.slug)
