import datetime
import uuid

from django.core.validators import MinValueValidator
from django.db import models

from apps.financial.models import DebtEntry
from apps.MySU.models import Association
from apps.MySU.models import Profile


class InvoiceQuerySet(models.QuerySet):
    pass


class InvoiceManager(models.Manager):
    def get_queryset(self):
        return InvoiceQuerySet(self.model, using=self._db)

    def locked(self):
        return self.filter(debt_entry__isnull=False)


class Invoice(models.Model):
    """
    Allows the association to add invoices to the debt collection.
    """
    slug = models.SlugField(default=uuid.uuid4, editable=False)
    reason = models.CharField(max_length=120)
    price = models.DecimalField(validators=[MinValueValidator(0.0)], decimal_places=2, max_digits=6)
    date_issued = models.DateField(default=datetime.date.today)

    association = models.ForeignKey(Association, related_name='invoices', on_delete=models.CASCADE)
    debt_entry = models.ForeignKey(DebtEntry, related_name='invoices', on_delete=models.SET_NULL, blank=True, null=True)
    profile = models.ForeignKey(Profile, related_name='invoices', on_delete=models.PROTECT)

    @property
    def locked(self):
        return self.debt_entry_id is not None

    objects = InvoiceManager()

    class Meta:
        verbose_name = "Invoice"
        verbose_name_plural = "Invoices"

    def __str__(self):
        return str(self.slug)
