import datetime
import uuid

from django.core.validators import MaxValueValidator
from django.core.validators import MinValueValidator
from django.db import models

from apps.financial.models import DebtEntry
from apps.MySU.models import Association
from apps.MySU.models import AssociationMembership
from apps.MySU.models import MemberType


class MembershipFeeQuerySet(models.QuerySet):
    pass


class MembershipFeeManager(models.Manager):
    def get_queryset(self):
        return MembershipFeeQuerySet(self.model, using=self._db)


class MembershipFee(models.Model):
    """
    Membertypes can change every year for a member. Being a member also brings a financial obligation to the member
    depending on the type the member has. This obligation has to be stored somewhere. That somewhere is this model.
    """
    slug = models.SlugField(default=uuid.uuid4, editable=False)
    payments = models.PositiveIntegerField(
        validators=[
            MinValueValidator(1900),
            MaxValueValidator(datetime.date.today().year)],
        help_text="Use the following format: <YYYY>")

    debt_entry = models.ForeignKey(DebtEntry, related_name='membership_fees', on_delete=models.CASCADE)
    association = models.ForeignKey(Association, related_name='membership_fees', on_delete=models.CASCADE)
    membertype = models.ForeignKey(MemberType, related_name='membership_fees', on_delete=models.PROTECT)
    association_membership = models.ForeignKey(AssociationMembership, related_name='membership_fees', on_delete=models.PROTECT)

    objects = MembershipFeeManager()

    class Meta:
        verbose_name = "Membership fee"
        verbose_name_plural = "Membership fees"

    def __str__(self):
        return str(self.slug)
