from typing import Any

from rest_framework import serializers

from apps.financial.models import DebtEntry
from MySU.helper_functions import get_for_patch


class DebtEntrySerializer(serializers.HyperlinkedModelSerializer):
    name = serializers.ReadOnlyField(source='profile.full_name')
    bank_account_name = serializers.ReadOnlyField(source='bank_account.name')
    iban = serializers.ReadOnlyField(source='bank_account.iban')
    mandate_id = serializers.ReadOnlyField(source='sepa_mandate.mandate_id')
    signature_date = serializers.ReadOnlyField(source='sepa_mandate.signature_date')
    bic = serializers.ReadOnlyField(source='bank_account.bic')
    email = serializers.ReadOnlyField(source='profile.email')
    date = serializers.ReadOnlyField(source='debt_collection.debt_collection_date')

    class Meta:
        model = DebtEntry
        fields = ['slug', 'date', 'total_price', 'name', 'paid_by', 'bank_account_name', 'email', 'iban', 'bic',
                  'cancelled', 'paid_after_cancellation', 'mandate_id', 'signature_date']
        extra_kwargs = {
            'url': {'lookup_field': 'slug'},
        }

    def update(self, instance: DebtEntry, validated_data: Any) -> Any:
        validated_data = { key: get_for_patch(instance, validated_data, key) for key in ['cancelled', 'paid_after_cancellation']}
        return super().update(instance, validated_data)
