from django.core.exceptions import ValidationError
from rest_framework import serializers

from apps.financial.models import Invoice
from apps.MySU.serializers.base import BaseSerializer
from MySU.fields.association import AssociationField


class InvoiceSerializer(BaseSerializer):
    association = AssociationField(view_name='association-detail', lookup_field='slug')
    debt_collection_date = serializers.ReadOnlyField(source='debt_entry.debt_collection.debt_collection_date')

    class Meta:
        model = Invoice
        fields = ('url', 'slug', 'reason', 'price', 'date_issued', 'association', 'debt_entry', 'debt_collection_date', 'profile')
        extra_kwargs = {
            'url': {'lookup_field': 'slug'},
            'association': {'lookup_field': 'slug'},
            'debt_entry': {'lookup_field': 'slug', 'read_only': True},
            'profile': {'lookup_field': 'slug'},
        }

    def validate(self, attrs):
        attrs = super().validate(attrs)
        instance = self.instance
        if instance and instance.locked:
            raise ValidationError({"locked": "Locked invoices cannot be changed."})
        return attrs
