from guardian.shortcuts import get_objects_for_user
from guardian.shortcuts import Q
from rest_framework import status
from rest_framework import viewsets
from rest_framework.permissions import BasePermission
from rest_framework.permissions import IsAuthenticated
from rest_framework.response import Response
from url_filter import filtersets

from apps.financial.models import DebtCollection
from apps.financial.serializers.debt_collection import DebtCollectionSerializer
from apps.financial.serializers.debt_collection import DebtCollectionSerializerList
from apps.MySU.models import Association

"""
----------------------------------------------------------------------------------------------------------------------
 DebtCollection
----------------------------------------------------------------------------------------------------------------------

 GET:       Board and self
 PUT/PATCH: Self
 POST:      Board

"""


class DebtCollectionHttpPermissions(BasePermission):
    def has_object_permission(self, request, view, obj):
        if request.method == "PATCH" or request.method == "PUT":
            return request.user.has_perm('board', obj.association)
        return True

    def has_permission(self, request, view):
        if request.method == "POST":
            return get_objects_for_user(request.user, 'MySU.board').count() > 0
        if request.method == "DELETE":
            return get_objects_for_user(request.user, 'MySU.board').count() > 0
        return True


class DebtCollectionFilterSet(filtersets.ModelFilterSet):
    class Meta:
        model = DebtCollection


class DebtCollectionViewSet(viewsets.ModelViewSet):
    queryset = DebtCollection.objects.none()
    permission_classes = (IsAuthenticated, DebtCollectionHttpPermissions,)
    http_method_names = ['get', 'post', 'put', 'patch', 'delete', 'head', 'options']
    lookup_field = 'slug'
    ordering_fields = ['slug', 'debt_collection_date', 'amount']
    ordering = ['debt_collection_date']
    filter_class = DebtCollectionFilterSet

    def get_serializer_class(self):
        if self.action == "list":
            return DebtCollectionSerializerList
        return DebtCollectionSerializer

    def get_queryset(self):
        associations = get_objects_for_user(self.request.user, 'board', klass=Association)
        queryset = DebtCollection.objects.filter(
            Q(association__in=associations)
        ).distinct()
        return queryset

    def destroy(self, request: 'Request', *args: 'Any', **kwargs: 'Any') -> 'Response':
        debt_collection = self.get_object()
        if debt_collection.locked:
            return Response(
                status=status.HTTP_400_BAD_REQUEST,
                data={"status": "Once Locked debt collection can no longer be removed"}
            )
        self.perform_destroy(debt_collection)
        return Response(status=status.HTTP_204_NO_CONTENT)
