from guardian.shortcuts import get_objects_for_user
from guardian.shortcuts import Q
from rest_framework import viewsets
from rest_framework.permissions import BasePermission
from rest_framework.permissions import IsAuthenticated
from url_filter import filtersets

from apps.financial.models import DebtEntry
from apps.financial.serializers.debt_entry import DebtEntrySerializer
from apps.MySU.models import Association

"""
----------------------------------------------------------------------------------------------------------------------
 Debt Entry
----------------------------------------------------------------------------------------------------------------------

 GET:       Board and self

"""


class DebtEntryHttpPermissions(BasePermission):
    def has_object_permission(self, request, view, obj):
        if request.method == "PUT" or request.method == "PATCH":  # HANDLES THE UPDATE
            return request.user.has_perm('board', obj.association)
        return True

    def has_permission(self, request, view):
        if request.method == "GET":  # HANDLES THE LIST
            return True
        if request.method == "POST":  # HANDLES THE CREATION/INSERTION
            return False
        return True


class DebtEntryFilterSet(filtersets.ModelFilterSet):
    class Meta:
        model = DebtEntry


class DebtEntryViewSet(viewsets.ModelViewSet):
    queryset = DebtEntry.objects.none()
    serializer_class = DebtEntrySerializer
    permission_classes = (IsAuthenticated, DebtEntryHttpPermissions,)
    http_method_names = ['get', 'patch', 'head', 'options']
    lookup_field = 'slug'
    ordering_fields = ['slug']
    ordering = ['slug']
    filter_class = DebtEntryFilterSet

    def get_queryset(self):
        associations = get_objects_for_user(self.request.user, 'board', klass=Association)
        queryset = DebtEntry.objects.filter(
            Q(profile__user=self.request.user) | Q(association__in=associations)
        ).select_related("bank_account", "profile", "debt_collection", "sepa_mandate").distinct()
        return queryset
