from guardian.shortcuts import get_objects_for_user
from guardian.shortcuts import Q
from rest_framework import viewsets
from rest_framework.permissions import BasePermission
from rest_framework.permissions import IsAuthenticated
from url_filter import filtersets

from apps.financial.models import MembershipFee
from apps.financial.serializers.membership_fee import MemberShipFeeSerializer
from apps.MySU.models import Association

"""
----------------------------------------------------------------------------------------------------------------------
 Membership Fee
----------------------------------------------------------------------------------------------------------------------

 GET:       Board and self
 PUT/PATCH: -
 POST:      -
"""


class MembershipFeeHttpPermissions(BasePermission):
    def has_object_permission(self, request, view, obj):
        if request.method == "PATCH" or request.method == "PUT":  # HANDLES THE UPDATE
            return False
        return True

    def has_permission(self, request, view):
        if request.method == "GET":  # HANDLES THE LIST
            return True
        if request.method == "POST":  # HANDLES THE CREATION / INSERTION
            return False
        return True


class MembershipFeeFilterSet(filtersets.ModelFilterSet):
    class Meta:
        model = MembershipFee


class MembershipFeeViewSet(viewsets.ModelViewSet):
    queryset = MembershipFee.objects.none()
    serializer_class = MemberShipFeeSerializer
    permission_classes = (IsAuthenticated, MembershipFeeHttpPermissions,)
    http_method_names = ['get', 'post', 'head', 'options']
    lookup_field = 'slug'
    ordering_fields = ['slug']
    ordering = ['slug']
    filter_class = MembershipFeeFilterSet

    def get_queryset(self):
        associations = get_objects_for_user(self.request.user, 'board', klass=Association)
        queryset = MembershipFee.objects.filter(
            Q(user=self.request.user) | Q(association__in=associations)
        ).distinct()
        return queryset
