from guardian.shortcuts import get_objects_for_user
from guardian.shortcuts import Q
from rest_framework import viewsets
from rest_framework.permissions import BasePermission
from rest_framework.permissions import IsAuthenticated
from url_filter import filtersets

from apps.financial.models import SepaMandate
from apps.financial.serializers.sepa_mandate import SepaMandateSerializer
from apps.MySU.models import Association

"""
----------------------------------------------------------------------------------------------------------------------
 Debt Entry
----------------------------------------------------------------------------------------------------------------------

 GET:       Board and self

"""


class SepaMandateHttpPermissions(BasePermission):
    def has_object_permission(self, request, view, obj):
        return request.method == "GET"

    def has_permission(self, request, view):
        return request.method == "GET"


class SepaMandateFilterSet(filtersets.ModelFilterSet):
    class Meta:
        model = SepaMandate


class SepaMandateViewSet(viewsets.ModelViewSet):
    queryset = SepaMandate.objects.none()
    serializer_class = SepaMandateSerializer
    permission_classes = (IsAuthenticated, SepaMandateHttpPermissions,)
    http_method_names = ['get', 'head', 'options']
    lookup_field = 'slug'
    ordering_fields = ['slug']
    ordering = ['slug']
    filter_class = SepaMandateFilterSet

    def get_queryset(self):
        associations = get_objects_for_user(self.request.user, 'board', klass=Association)
        queryset = SepaMandate.objects.filter(
            Q(profile__user=self.request.user) | Q(association__in=associations)
        ).distinct()
        return queryset
