#!/bin/bash

ARGS=$(getopt -o fqh --long fix,quiet,help -- "$@")
eval set -- "$ARGS"
while true; do
    case "$1" in
        -f | --fix ) FIX=true; shift ;;
        -q | --quiet ) QUIET=true; shift ;;
        -h | --help ) HELP=true; shift ;;
        -- ) shift; break ;;
        * ) break ;;
    esac
done

FIX=${FIX:-false}
HELP=${HELP:-false}
QUIET=${QUIET:-false}


if $HELP; then
    cat << "EOF"
 This script checks the line endings in the project and corrects, if asked.

 The following is done in order:
  - Find the wrong line endings
  - Echo the wrong line endings, if specified
  - Fix line endings, if specified

 The following parameters are able to be used:
  -f | --fix        Fix the found errors
  -q | --quiet      Hide the output of the program
  -h | --help       Raise this help
EOF
    exit 0
fi


find . ! -path './venv/*' ! -path './.idea/*' ! -path './MySU/fixtures/generated/*' -type f -exec file {} \; | grep 'CRLF'  | awk -F ':' '{ print $1 }' > "wrong_line_endings"

FOUND_WRONG_LINE_ENDINGS="$(cat ./wrong_line_endings | wc -c)"

if [[ $FOUND_WRONG_LINE_ENDINGS -ne 0 ]]; then
    if $FIX && $QUIET; then
        cat ./wrong_line_endings | xargs sed -i 's/\r$//'
        echo
        echo -e "\e[1;32mFixed. All line endings are Unix now.\e[0m"
        rm ./wrong_line_endings
    elif $FIX; then
        echo -e "\e[1;31mWrong line endings found in:\e[0m"
        cat ./wrong_line_endings | awk -F ':' '{ print $1 }'
        cat ./wrong_line_endings | xargs sed -i 's/\r$//'
        echo
        echo -e "\e[1;32mFixed. All line endings are Unix now.\e[0m"
        rm ./wrong_line_endings
    elif $QUIET; then
        rm ./wrong_line_endings
        exit 1
    else
        echo -e "\e[1;31mWrong line endings found in:\e[0m"
        cat ./wrong_line_endings | awk -F ':' '{ print $1 }'
        echo
        echo "Please set all line ending to UNIX(\n)"
        echo "You can set the the line endings in the project to UNIX by running "
        echo -e "\e[33mcheck_line_endings.sh --fix --quiet\e[0m"
        echo "or by properly configuring your IDE"
        echo -e -n "e.g. PyCharm: Set in the menu '\e[33mSettings->Editor->Code Style\e[0m' the line separator to UNIX"
        echo "(\n)"
        echo
        rm ./wrong_line_endings
        exit 1
    fi
else
    echo
    echo -e "\e[1;32mAll line endings are UNIX\e[0m"
    echo
fi
