#!/bin/bash

ARGS=$(getopt -o h --long help -- "$@")
eval set -- "$ARGS"
while true; do
    case "$1" in
        -h | --help ) HELP=true; shift ;;
        -- ) shift; break ;;
        * ) break ;;
    esac
done

FIX=${FIX:local}
HELP=${HELP:-false}
QUIET=${QUIET:-false}


if $HELP; then
    cat << "EOF"
 This script checks whether the code satisfies the PEP standards. In these checks the length of
 the code lines is not included.

 The following is done in order:
  - Find and echo code style violations

 The following parameters are able to be used:
  -h | --help       Raise this help
EOF
    exit 0
fi


echo
echo -e "\e[1;32mChecking style...\e[0m"
echo
NUMBER_OF_ERRORS="$(pycodestyle --ignore=E501 --exclude='./venv/*' . | wc -l)" || exit 1
if [[ $NUMBER_OF_ERRORS -ne 0 ]]; then
    echo -e "\e[1;31m$NUMBER_OF_ERRORS errors found, please fix the following:\e[0m"
    pycodestyle --ignore=E501 --exclude='./venv/*' . || exit 1
    exit 1
else
    echo "Code checked. No error found!"
fi
