import datetime
import json
import random
import uuid

from django.utils.text import slugify
from faker import Faker

from MySU.objects.yearlessdate import date_to_yearlessdate
from MySU.objects.yearlessdate import YearlessDate

fake = Faker('nl_NL')
number_assocations_to_generate = 150
content_type_natural_key = ['MySU', 'association']
permission_natural_key = ['board', 'MySU', 'association']
board_functions = ["Chairman", "Secretary", "Treasurer"]
student_number_range = random.sample(
    range(2000000, 2999999, len(board_functions)),
    len(board_functions) * number_assocations_to_generate
)

"""
----------------------------------------------------------------------------------------------------------------------
 PREDETERMINED
----------------------------------------------------------------------------------------------------------------------
"""
assoc_array = []
board_auth_group_array = []
board_membergroup_array = []
board_memberships_array = []
board_association_memberships_array = []
board_profile_array = []
board_user_array = []
board_objectpermissions_array = []
"""
--------------------------------------
 SU
--------------------------------------
"""
i = 1
k = i + number_assocations_to_generate
s = i + 10000
founding_date = str(datetime.date(1999, 7, 2))
company = "Student Union"
assoc_array.append({
    'model': 'core.association',
    'pk': 1,
    'fields': {
        'phone_number': '053-489 8006',
        'address': 'De Hems 10',
        'zip_code': '7522 NL',
        'city': 'Enschede',
        'country': 'Netherlands',
        'slug': slugify(company),
        'name': company,
        'short_name': 'SU',
        'founding_date': founding_date,
        'email': 'board@union.nl',
        'color': '#77ff00',
        'logo': 'association/logo/SU_logo.png',
        'dissolution_date': None,
        'postal_address': '',
        'full_year_membership_start_date': str(YearlessDate(8, 1))
    }
})
top_board_uuid = 'dd0d1b61-e054-4495-b540-3a6c65a5f610'
board_auth_group_array.append({
    "model": "auth.group",
    "pk": i,
    "fields": {
        "name": top_board_uuid,
        "permissions": []
    }
})
board_membergroup_array.append(
    {
        "model": "MySU.group",
        "pk": i,
        "fields": {
            "slug": top_board_uuid,
            "short_name": "Boards of SU",
            "full_name": 'boards of {}'.format(company),
            "alternate_name": "SUb's",
            "description": 'Aliquam veritatis illum nihil eos sapiente voluptatem. Quas facilis doloremque odio.',
            "number": 0,
            "motto": "",
            "association": i,
            "parent_group": None,
            "group": i,
            "founding_date": founding_date,
            "dissolution_date": None,
            "archived": False,
            "board_group": True,
            "current_board": False
        }
    }
)
board_uuid = '705f007c-dba5-4da6-9dd7-ba7c22af88bc'
board_auth_group_array.append(
    {
        "model": "auth.group",
        "pk": k,
        "fields": {
            "name": board_uuid,
            "permissions": []
        }
    }
)
board_membergroup_array.append(
    {
        "model": "MySU.group",
        "pk": k,
        "fields": {
            "slug": board_uuid,
            "short_name": "Board of SU",
            "full_name": 'board of {}'.format(company),
            "alternate_name": "College van Bestuur",
            "description": 'Similique repudiandae minus sit corrupti doloribus nemo. Possimus magni dolorem.\n'
                           'Unde nemo suscipit iure officia. Unde eius atque itaque placeat eius sapiente.',
            "number": 0,
            "motto": "",
            "association": i,
            "parent_group": i,
            "group": k,
            "founding_date": founding_date,
            "dissolution_date": None,
            "archived": False,
            "board_group": True,
            "current_board": True
        }
    }
)
board_objectpermissions_array.append(
    {
        "model": "guardian.groupobjectpermission",
        "pk": i,
        "fields": {
            "object_pk": i,
            "content_type": content_type_natural_key,
            "permission": permission_natural_key,
            "group": i
        }
    }
)
board_objectpermissions_array.append(
    {
        "model": "guardian.groupobjectpermission",
        "pk": k,
        "fields": {
            "object_pk": i,
            "content_type": content_type_natural_key,
            "permission": permission_natural_key,
            "group": k
        }
    }
)
for j, f in enumerate(board_functions):
    u = i * len(board_functions) + j
    s = u + 10000
    # The Treasurer is the default 's1234567' user, so no need to create that user again.
    if f == "Treasurer":
        board_memberships_array.append({
            "model": "MySU.groupmembership",
            "pk": s,
            "fields": {
                "slug": str(uuid.uuid4()),
                "duty": f,
                "profile": 3,
                "user": 3,
                "association": i,
                "group": k,
                "date_joined": fake.date_between(start_date="-9y", end_date="today").strftime('%Y-%m-%d'),
                "date_left": None,
                "email": fake.email()
            }
        })
    else:
        username = student_number_range[u]
        board_user_array.append(
            {
                "model": "MySU.user",
                "pk": s,
                "fields": {
                    "password": "pbkdf2_sha256$30000$FLYQhSXqcuTl$OrWoIIqVB3kw0t7/hSLP44jvtmBVaREoLlBKu+EIf3k=",
                    "last_login": "2017-12-06T13:27:57.849Z",
                    "is_superuser": False,
                    "slug": str(uuid.uuid4()),
                    "username": "s" + str(username),
                    "date_joined": fake.date_between(start_date="-10y", end_date="today").strftime('%Y-%m-%d'),
                    "date_activated": "2017-11-26",
                    "date_left": None,
                    "is_active": True,
                    "is_staff": False,
                    "preferred_profile": s,
                }
            },
        )
        board_profile_array.append(
            {
                "model": "MySU.profile",
                "pk": s,
                "fields": {
                    "profilename": fake.province() + " %s" % u,
                    "address": fake.street_address(),
                    "zip_code": fake.postcode(),
                    "city": fake.city(),
                    "country": "Netherlands",
                    "slug": str(uuid.uuid4()),
                    "user": s,
                    "student_number": "s" + str(username),
                    "given_name": fake.first_name(),
                    "first_names": fake.first_name(),
                    "surname": fake.last_name(),
                    "phone_number": fake.phone_number(),
                    "email": fake.email(),
                    "gender": None,
                    "date_of_birth": fake.date_between(start_date="-30y", end_date="-15y").strftime('%Y-%m-%d')
                }
            }
        )
        board_memberships_array.append(
            {
                "model": "MySU.groupmembership",
                "pk": s,
                "fields": {
                    "slug": str(uuid.uuid4()),
                    "duty": f,
                    "profile": s,
                    "user": s,
                    "association": i,
                    "group": k,
                    "date_joined": fake.date_between(start_date="-9y", end_date="today").strftime('%Y-%m-%d'),
                    "date_left": None
                }
            },
        )
        board_association_memberships_array.append(
            {
                "model": "MySU.associationmembership",
                "pk": s,
                "fields": {
                    "slug": str(uuid.uuid4()),
                    "user": s,
                    "profile": s,
                    "association": i,
                    "type": i,
                    "date_left": None,
                    "date_joined": fake.date_between(start_date="-8y", end_date="today").strftime('%Y-%m-%d')
                }
            },
        )
"""
--------------------------------------
 HELIOS
--------------------------------------
"""
i = 2
k = i + number_assocations_to_generate
s = i + 10000
founding_date = str(datetime.date.min)
company = "Helios"
assoc_array.append({
    'model': 'core.association',
    'pk': 2,
    'fields': {
        'phone_number': '+31687654321',
        'address': 'Ashleypad 6',
        'zip_code': '7654 TF',
        'city': 'Rhodes',
        'country': 'Pluto',
        'slug': slugify("Rising Sun Alienship Helios"),
        'name': "Rising Sun Alienship Helios",
        'short_name': 'Helios',
        'founding_date': founding_date,
        'email': 'despot@helios.plt',
        'color': '#ff4400',
        'logo': 'association/logo/Samba.png',
        'dissolution_date': None,
        'postal_address': '',
        'full_year_membership_start_date': str(YearlessDate(1, 1))
    }
})
top_board_uuid = '2d9551a0-7e40-461e-aa49-e578eb608b65'
board_auth_group_array.append({
    "model": "auth.group",
    "pk": i,
    "fields": {
        "name": top_board_uuid,
        "permissions": []
    }
})
board_membergroup_array.append(
    {
        "model": "MySU.group",
        "pk": i,
        "fields": {
            "slug": top_board_uuid,
            "short_name": "Boards of Helios",
            "full_name": 'Boards of {}'.format(company),
            "alternate_name": "Heli's",
            "description": 'Molestias harum delectus optio. Perferendis alias distinctio quo excepturi numquam molestiae. '
                           'Porro possimus hic deleniti nihil. Similique veniam quae modi totam tempore officia.',
            "number": 0,
            "motto": "",
            "association": i,
            "parent_group": None,
            "group": i,
            "founding_date": founding_date,
            "dissolution_date": None,
            "archived": False,
            "board_group": True,
            "current_board": False
        }
    }
)
board_uuid = 'c2d3528a-46b3-4830-afa0-dc00831f040a'
board_auth_group_array.append(
    {
        "model": "auth.group",
        "pk": k,
        "fields": {
            "name": board_uuid,
            "permissions": []
        }
    }
)
board_membergroup_array.append(
    {
        "model": "MySU.group",
        "pk": k,
        "fields": {
            "slug": board_uuid,
            "short_name": "Board of Helios",
            "full_name": 'board of {}'.format(company),
            "alternate_name": "",
            "description": 'Tempora odio dolor mollitia placeat praesentium atque. Tempora occaecati delectus possimus '
                           'rem totam itaque.\nVoluptate nisi dolor ipsam corrupti harum ea voluptas. Mollitia exercitationem eligendi.',
            "number": 0,
            "motto": "",
            "association": i,
            "parent_group": i,
            "group": k,
            "founding_date": founding_date,
            "dissolution_date": None,
            "archived": False,
            "board_group": True,
            "current_board": True
        }
    }
)
board_objectpermissions_array.append(
    {
        "model": "guardian.groupobjectpermission",
        "pk": i,
        "fields": {
            "object_pk": i,
            "content_type": content_type_natural_key,
            "permission": permission_natural_key,
            "group": i
        }
    }
)
board_objectpermissions_array.append(
    {
        "model": "guardian.groupobjectpermission",
        "pk": k,
        "fields": {
            "object_pk": i,
            "content_type": content_type_natural_key,
            "permission": permission_natural_key,
            "group": k
        }
    }
)
for j, f in enumerate(board_functions):
    u = i * len(board_functions) + j
    s = u + 10000
    username = student_number_range[u]
    board_user_array.append(
        {
            "model": "MySU.user",
            "pk": s,
            "fields": {
                "password": "pbkdf2_sha256$30000$FLYQhSXqcuTl$OrWoIIqVB3kw0t7/hSLP44jvtmBVaREoLlBKu+EIf3k=",
                "last_login": "2017-12-06T13:27:57.849Z",
                "is_superuser": False,
                "slug": str(uuid.uuid4()),
                "username": "s" + str(username),
                "date_joined": fake.date_between(start_date="-10y", end_date="today").strftime('%Y-%m-%d'),
                "date_activated": "2017-11-26",
                "date_left": None,
                "is_active": True,
                "is_staff": False,
                "preferred_profile": s,
            }
        },
    )
    board_profile_array.append(
        {
            "model": "MySU.profile",
            "pk": s,
            "fields": {
                "profilename": fake.province() + " %s" % u,
                "address": fake.street_address(),
                "zip_code": fake.postcode(),
                "city": fake.city(),
                "country": "Netherlands",
                "slug": str(uuid.uuid4()),
                "user": s,
                "student_number": "s" + str(username),
                "given_name": fake.first_name(),
                "first_names": fake.first_name(),
                "surname": fake.last_name(),
                "phone_number": fake.phone_number(),
                "email": fake.email(),
                "gender": None,
                "date_of_birth": fake.date_between(start_date="-30y", end_date="-15y").strftime('%Y-%m-%d')
            }
        }
    )
    board_memberships_array.append(
        {
            "model": "MySU.groupmembership",
            "pk": s,
            "fields": {
                "slug": str(uuid.uuid4()),
                "duty": f,
                "profile": s,
                "user": s,
                "association": i,
                "group": k,
                "date_joined": fake.date_between(start_date="-9y", end_date="today").strftime('%Y-%m-%d'),
                "date_left": None
            }
        },
    )
    board_association_memberships_array.append(
        {
            "model": "MySU.associationmembership",
            "pk": s,
            "fields": {
                "slug": str(uuid.uuid4()),
                "user": s,
                "profile": s,
                "association": i,
                "type": 1,
                "date_left": None,
                "date_joined": fake.date_between(start_date="-8y", end_date="today").strftime('%Y-%m-%d')
            }
        },
    )
"""
--------------------------------------
 RA
--------------------------------------
"""
i = 3
k = i + number_assocations_to_generate
s = i + 10000
founding_date = str(datetime.date.min)
company = "Ra"
assoc_array.append({
    'model': 'core.association',
    'pk': i,
    'fields': {
        'phone_number': '+31687654321',
        'address': 'Dewiring 8',
        'zip_code': '4786 JY',
        'city': 'Memphis',
        'country': 'Egypt',
        'slug': slugify("Solar Barque Companionship Ra"),
        'name': "Solar Barque Companionship Helios",
        'short_name': 'Ra',
        'founding_date': founding_date,
        'email': 'pharaoh@amon.ra',
        'color': '#778800',
        'logo': 'association/logo/ankh.jpg',
        'dissolution_date': None,
        'postal_address': '',
        'full_year_membership_start_date': str(YearlessDate(3, 1))
    }
})
top_board_uuid = 'c21e9719-e173-4a48-ac3c-4dfc47e2b159'
board_auth_group_array.append({
    "model": "auth.group",
    "pk": i,
    "fields": {
        "name": top_board_uuid,
        "permissions": []
    }
})
board_membergroup_array.append(
    {
        "model": "MySU.group",
        "pk": i,
        "fields": {
            "slug": top_board_uuid,
            "short_name": "Boards of Ra",
            "full_name": 'Boards of {}'.format(company),
            "alternate_name": "Horus",
            "description": fake.text(max_nb_chars=200, ext_word_list=None),
            "number": 0,
            "motto": "",
            "association": i,
            "parent_group": None,
            "group": i,
            "founding_date": founding_date,
            "dissolution_date": None,
            "archived": False,
            "board_group": True,
            "current_board": False
        }
    }
)
board_uuid = '675a6d90-92ee-4647-bb77-34f1b4419bb4'
board_auth_group_array.append(
    {
        "model": "auth.group",
        "pk": k,
        "fields": {
            "name": board_uuid,
            "permissions": []
        }
    }
)
board_membergroup_array.append(
    {
        "model": "MySU.group",
        "pk": k,
        "fields": {
            "slug": board_uuid,
            "short_name": "Board of Ra",
            "full_name": 'board of {}'.format(company),
            "alternate_name": "",
            "description": 'Accusamus reiciendis error sed. Voluptatibus aspernatur velit labore architecto reiciendis.\n'
                           'Id numquam beatae consequuntur placeat quaerat tempora. Alias accusantium saepe quidem iste qui.',
            "number": 0,
            "motto": "",
            "association": i,
            "parent_group": i,
            "group": k,
            "founding_date": founding_date,
            "dissolution_date": None,
            "archived": False,
            "board_group": True,
            "current_board": True
        }
    }
)
board_objectpermissions_array.append(
    {
        "model": "guardian.groupobjectpermission",
        "pk": i,
        "fields": {
            "object_pk": i,
            "content_type": content_type_natural_key,
            "permission": permission_natural_key,
            "group": i
        }
    }
)
board_objectpermissions_array.append(
    {
        "model": "guardian.groupobjectpermission",
        "pk": k,
        "fields": {
            "object_pk": i,
            "content_type": content_type_natural_key,
            "permission": permission_natural_key,
            "group": k
        }
    }
)
for j, f in enumerate(board_functions):
    u = i * len(board_functions) + j
    s = u + 10000
    username = student_number_range[u]
    board_user_array.append(
        {
            "model": "MySU.user",
            "pk": s,
            "fields": {
                "password": "pbkdf2_sha256$30000$FLYQhSXqcuTl$OrWoIIqVB3kw0t7/hSLP44jvtmBVaREoLlBKu+EIf3k=",
                "last_login": "2017-12-06T13:27:57.849Z",
                "is_superuser": False,
                "slug": str(uuid.uuid4()),
                "username": "s" + str(username),
                "date_joined": fake.date_between(start_date="-10y", end_date="today").strftime('%Y-%m-%d'),
                "date_activated": "2017-11-26",
                "date_left": None,
                "is_active": True,
                "is_staff": False,
                "preferred_profile": s,
            }
        },
    )
    board_profile_array.append(
        {
            "model": "MySU.profile",
            "pk": s,
            "fields": {
                "profilename": fake.province() + " %s" % u,
                "address": fake.street_address(),
                "zip_code": fake.postcode(),
                "city": fake.city(),
                "country": "Netherlands",
                "slug": str(uuid.uuid4()),
                "user": s,
                "student_number": "s" + str(username),
                "given_name": fake.first_name(),
                "first_names": fake.first_name(),
                "surname": fake.last_name(),
                "phone_number": fake.phone_number(),
                "email": fake.email(),
                "gender": None,
                "date_of_birth": fake.date_between(start_date="-30y", end_date="-15y").strftime('%Y-%m-%d')
            }
        }
    )
    board_memberships_array.append(
        {
            "model": "MySU.groupmembership",
            "pk": s,
            "fields": {
                "slug": str(uuid.uuid4()),
                "duty": f,
                "profile": s,
                "user": s,
                "association": i,
                "group": k,
                "date_joined": fake.date_between(start_date="-9y", end_date="today").strftime('%Y-%m-%d'),
                "date_left": None
            }
        },
    )
    board_association_memberships_array.append(
        {
            "model": "MySU.associationmembership",
            "pk": s,
            "fields": {
                "slug": str(uuid.uuid4()),
                "user": s,
                "profile": s,
                "association": i,
                "type": 1,
                "date_left": None,
                "date_joined": fake.date_between(start_date="-8y", end_date="today").strftime('%Y-%m-%d')
            }
        },
    )

fixture_dir = "preset"
# with open('./MySU/fixtures/{}/association.json'.format(fixture_dir), 'w+') as json_file:
#     json.dump(assoc_array, json_file, indent=4)
# with open('./MySU/fixtures/{}/board_association_memberships.json'.format(fixture_dir), 'w+') as json_file:
#     json.dump(board_association_memberships_array, json_file, indent=4)
# with open('./MySU/fixtures/{}/board_django_groups.json'.format(fixture_dir), 'w+') as json_file:
#     json.dump(board_auth_group_array, json_file, indent=4)
# with open('./MySU/fixtures/{}/board_groups.json'.format(fixture_dir), 'w+') as json_file:
#     json.dump(board_membergroup_array, json_file, indent=4)
# with open('./MySU/fixtures/{}/board_memberships.json'.format(fixture_dir), 'w+') as json_file:
#     json.dump(board_memberships_array, json_file, indent=4)
# with open('./MySU/fixtures/{}/board_objectpermissions.json'.format(fixture_dir), 'w+') as json_file:
#     json.dump(board_objectpermissions_array, json_file, indent=4)
# with open('./MySU/fixtures/{}/board_profiles.json'.format(fixture_dir), 'w+') as json_file:
#     json.dump(board_profile_array, json_file, indent=4)
# with open('./MySU/fixtures/{}/board_users.json'.format(fixture_dir), 'w+') as json_file:
#     json.dump(board_user_array, json_file, indent=4)

"""
----------------------------------------------------------------------------------------------------------------------
 RANDOM GENERATED
----------------------------------------------------------------------------------------------------------------------
"""
assoc_array = []
board_auth_group_array = []
board_membergroup_array = []
board_memberships_array = []
board_association_memberships_array = []
board_profile_array = []
board_user_array = []
board_objectpermissions_array = []

for i in range(4, number_assocations_to_generate):
    k = i + number_assocations_to_generate
    s = i + 10000
    company = fake.company()
    founding_date = fake.date_between(start_date="-30y", end_date="-10y").strftime('%Y-%m-%d')
    assoc_array.append(
        {
            'model': 'core.association',
            'pk': i,
            'fields': {
                'phone_number': fake.phone_number(),
                'address': fake.street_address(),
                'zip_code': fake.postcode(),
                'city': fake.city(),
                'country': 'Netherlands',
                'slug': slugify(company),
                'name': company,
                'short_name': 'SU',
                'founding_date': founding_date,
                'email': fake.email(),
                'color': fake.safe_hex_color(),
                'logo': '',
                'dissolution_date': None,
                'postal_address': '',
                'full_year_membership_start_date': date_to_yearlessdate(fake.date_between(start_date="-30y", end_date="-10y"))
            }
        }
    )
    top_board_uuid = str(uuid.uuid4())
    board_auth_group_array.append(
        {
            "model": "auth.group",
            "pk": i,
            "fields": {
                "name": top_board_uuid,
                "permissions": []
            }
        }
    )
    board_membergroup_array.append(
        {
            "model": "MySU.group",
            "pk": i,
            "fields": {
                "slug": top_board_uuid,
                "short_name": "Boards of SU",
                "full_name": 'boards of {}'.format(company),
                "alternate_name": fake.user_name(),
                "description": fake.text(max_nb_chars=200, ext_word_list=None),
                "number": 0,
                "motto": "",
                "association": i,
                "parent_group": None,
                "group": i,
                "founding_date": founding_date,
                "dissolution_date": None,
                "archived": False,
                "board_group": True,
                "current_board": False
            }
        }
    )
    board_uuid = str(uuid.uuid4())
    board_auth_group_array.append(
        {
            "model": "auth.group",
            "pk": k,
            "fields": {
                "name": board_uuid,
                "permissions": []
            }
        }
    )
    board_membergroup_array.append(
        {
            "model": "MySU.group",
            "pk": k,
            "fields": {
                "slug": board_uuid,
                "short_name": "Board of SU",
                "full_name": 'board of {}'.format(company),
                "alternate_name": fake.user_name(),
                "description": fake.text(max_nb_chars=200, ext_word_list=None),
                "number": 0,
                "motto": "",
                "association": i,
                "parent_group": i,
                "group": k,
                "founding_date": founding_date,
                "dissolution_date": None,
                "archived": False,
                "board_group": True,
                "current_board": True
            }
        }
    )
    board_objectpermissions_array.append(
        {
            "model": "guardian.groupobjectpermission",
            "pk": i,
            "fields": {
                "object_pk": i,
                "content_type": content_type_natural_key,
                "permission": permission_natural_key,
                "group": i
            }
        }
    )
    board_objectpermissions_array.append(
        {
            "model": "guardian.groupobjectpermission",
            "pk": k,
            "fields": {
                "object_pk": i,
                "content_type": content_type_natural_key,
                "permission": permission_natural_key,
                "group": k
            }
        }
    )
    for j, f in enumerate(board_functions):
        u = i * len(board_functions) + j
        s = u + 10000
        username = student_number_range[u]
        board_user_array.append(
            {
                "model": "MySU.user",
                "pk": s,
                "fields": {
                    "password": "pbkdf2_sha256$30000$FLYQhSXqcuTl$OrWoIIqVB3kw0t7/hSLP44jvtmBVaREoLlBKu+EIf3k=",
                    "last_login": "2017-12-06T13:27:57.849Z",
                    "is_superuser": False,
                    "slug": str(uuid.uuid4()),
                    "username": "s" + str(username),
                    "date_joined": fake.date_between(start_date="-10y", end_date="today").strftime('%Y-%m-%d'),
                    "date_activated": "2017-11-26",
                    "date_left": None,
                    "is_active": True,
                    "is_staff": False,
                    "preferred_profile": s,
                }
            },
        )
        board_profile_array.append(
            {
                "model": "MySU.profile",
                "pk": s,
                "fields": {
                    "profilename": fake.province() + " %s" % u,
                    "address": fake.street_address(),
                    "zip_code": fake.postcode(),
                    "city": fake.city(),
                    "country": "Netherlands",
                    "slug": str(uuid.uuid4()),
                    "user": s,
                    "student_number": "s" + str(username),
                    "given_name": fake.first_name(),
                    "first_names": fake.first_name(),
                    "surname": fake.last_name(),
                    "phone_number": fake.phone_number(),
                    "email": fake.email(),
                    "gender": None,
                    "date_of_birth": fake.date_between(start_date="-30y", end_date="-15y").strftime('%Y-%m-%d')
                }
            }
        )
        board_memberships_array.append(
            {
                "model": "MySU.groupmembership",
                "pk": s,
                "fields": {
                    "slug": str(uuid.uuid4()),
                    "duty": f,
                    "profile": s,
                    "user": s,
                    "association": i,
                    "group": k,
                    "date_joined": fake.date_between(start_date="-9y", end_date="today").strftime('%Y-%m-%d'),
                    "date_left": None
                }
            },
        )
        board_association_memberships_array.append(
            {
                "model": "MySU.associationmembership",
                "pk": s,
                "fields": {
                    "slug": str(uuid.uuid4()),
                    "user": s,
                    "profile": s,
                    "association": i,
                    "type": 1,
                    "date_left": None,
                    "date_joined": fake.date_between(start_date="-8y", end_date="today").strftime('%Y-%m-%d')
                }
            },
        )
fixture_dir = "generated"
with open('./MySU/fixtures/{}/association.json'.format(fixture_dir), 'w+') as json_file:
    json.dump(assoc_array, json_file, indent=4)
with open('./MySU/fixtures/{}/board_association_memberships.json'.format(fixture_dir), 'w+') as json_file:
    json.dump(board_association_memberships_array, json_file, indent=4)
with open('./MySU/fixtures/{}/board_django_groups.json'.format(fixture_dir), 'w+') as json_file:
    json.dump(board_auth_group_array, json_file, indent=4)
with open('./MySU/fixtures/{}/board_groups.json'.format(fixture_dir), 'w+') as json_file:
    json.dump(board_membergroup_array, json_file, indent=4)
with open('./MySU/fixtures/{}/board_memberships.json'.format(fixture_dir), 'w+') as json_file:
    json.dump(board_memberships_array, json_file, indent=4)
with open('./MySU/fixtures/{}/board_objectpermissions.json'.format(fixture_dir), 'w+') as json_file:
    json.dump(board_objectpermissions_array, json_file, indent=4)
with open('./MySU/fixtures/{}/board_profiles.json'.format(fixture_dir), 'w+') as json_file:
    json.dump(board_profile_array, json_file, indent=4)
with open('./MySU/fixtures/{}/board_users.json'.format(fixture_dir), 'w+') as json_file:
    json.dump(board_user_array, json_file, indent=4)
