import json
import random
import uuid

from faker import Faker
fake = Faker('nl_NL')

number_assocations_to_generate = 150
number_of_committees_per_association = 5
student_number_range = random.sample(
    range(3000000, 3999999),
    number_of_committees_per_association * number_assocations_to_generate
)

"""
----------------------------------------------------------------------------------------------------------------------
 PREDETERMINED
----------------------------------------------------------------------------------------------------------------------
"""
# committee_array = []
# committee_user_array = []
# committee_profile_array = []
# committee_memberships_array = []
# committee_association_memberships_array = []
# committee_auth_group_array = []
#
# fixture_dir = "preset"
# with open('./MySU/fixtures/{}/committee_memberships.json'.format(fixture_dir), 'w+') as json_file:
#     json.dump(committee_memberships_array, json_file, indent=4)
# with open('./MySU/fixtures/{}/committee_association_memberships.json'.format(fixture_dir), 'w+') as json_file:
#     json.dump(committee_association_memberships_array, json_file, indent=4)
# with open('./MySU/fixtures/{}/committee_profile.json'.format(fixture_dir), 'w+') as json_file:
#     json.dump(committee_profile_array, json_file, indent=4)
# with open('./MySU/fixtures/{}/committee_user.json'.format(fixture_dir), 'w+') as json_file:
#     json.dump(committee_user_array, json_file, indent=4)
# with open('./MySU/fixtures/{}/committee.json'.format(fixture_dir), 'w+') as json_file:
#     json.dump(committee_array, json_file, indent=4)
# with open('./MySU/fixtures/{}/committee_auth_group.json'.format(fixture_dir), 'w+') as json_file:
#     json.dump(committee_auth_group_array, json_file, indent=4)

"""
----------------------------------------------------------------------------------------------------------------------
 RANDOM GENERATED
----------------------------------------------------------------------------------------------------------------------
"""
committee_array = []
committee_user_array = []
committee_profile_array = []
committee_memberships_array = []
committee_association_memberships_array = []
committee_auth_group_array = []

for a in range(1, number_assocations_to_generate):
    for c in range(number_of_committees_per_association):
        u = c + (a - 1) * number_of_committees_per_association
        k = u + 1000
        s = u + 200000
        username = student_number_range[u]
        committee_auth_group_array.append(
            {
                "model": "auth.group",
                "pk": k,
                "fields": {
                    "name": str(uuid.uuid4()),
                    "permissions": []
                }
            }
        )
        committee_array.append(
            {
                "model": "MySU.group",
                "pk": k,
                "fields": {
                    "slug": str(uuid.uuid4()),
                    "short_name": fake.last_name(),
                    "full_name": fake.name(),
                    "alternate_name": fake.user_name(),
                    "description": fake.text(max_nb_chars=200, ext_word_list=None),
                    "number": 0,
                    "motto": "",
                    "association": a,
                    "parent_group": None,
                    "group": k,
                    "founding_date": fake.date_between(start_date="-9y", end_date="-8y").strftime('%Y-%m-%d'),
                    "dissolution_date": None,
                    "archived": False,
                    "board_group": False
                }
            }
        )
        committee_user_array.append(
            {
                "model": "MySU.user",
                "pk": s,
                "fields": {
                    "password": "pbkdf2_sha256$30000$FLYQhSXqcuTl$OrWoIIqVB3kw0t7/hSLP44jvtmBVaREoLlBKu+EIf3k=",
                    "last_login": "2017-12-06T13:27:57.849Z",
                    "is_superuser": False,
                    "slug": str(uuid.uuid4()),
                    "username": "s" + str(username),
                    "date_joined": fake.date_between(start_date="-10y", end_date="today").strftime('%Y-%m-%d'),
                    "date_activated": "2017-11-26",
                    "date_left": None,
                    "is_active": True,
                    "is_staff": False,
                    "preferred_profile": s,
                }
            },
        )
        committee_profile_array.append(
            {
                "model": "MySU.profile",
                "pk": s,
                "fields": {
                    "profilename": fake.province() + " %s" % username,
                    "address": fake.street_address(),
                    "zip_code": fake.postcode(),
                    "city": fake.city(),
                    "country": "Netherlands",
                    "slug": str(uuid.uuid4()),
                    "user": s,
                    "student_number": "s" + str(username),
                    "given_name": fake.first_name(),
                    "first_names": fake.first_name(),
                    "surname": fake.last_name(),
                    "phone_number": fake.phone_number(),
                    "email": fake.email(),
                    "gender": None,
                    "date_of_birth": fake.date_between(start_date="-30y", end_date="-15y").strftime('%Y-%m-%d')
                }
            }
        )
        committee_memberships_array.append(
            {
                "model": "MySU.groupmembership",
                "pk": s,
                "fields": {
                    "slug": str(uuid.uuid4()),
                    "duty": "General Member",
                    "profile": s,
                    "user": s,
                    "association": a,
                    "group": k,
                    "date_joined": fake.date_between(start_date="-9y", end_date="today").strftime('%Y-%m-%d'),
                    "date_left": None,
                    "email": fake.email()
                }
            },
        )
        committee_association_memberships_array.append(
            {
                "model": "MySU.associationmembership",
                "pk": s,
                "fields": {
                    "slug": str(uuid.uuid4()),
                    "user": s,
                    "profile": s,
                    "association": a,
                    "type": a,
                    "date_left": None,
                    "date_joined": fake.date_between(start_date="-8y", end_date="today").strftime('%Y-%m-%d')
                }
            },
        )
fixture_dir = "generated"
with open('./MySU/fixtures/{}/committee_memberships.json'.format(fixture_dir), 'w+') as json_file:
    json.dump(committee_memberships_array, json_file, indent=4)
with open('./MySU/fixtures/{}/committee_association_memberships.json'.format(fixture_dir), 'w+') as json_file:
    json.dump(committee_association_memberships_array, json_file, indent=4)
with open('./MySU/fixtures/{}/committee_profile.json'.format(fixture_dir), 'w+') as json_file:
    json.dump(committee_profile_array, json_file, indent=4)
with open('./MySU/fixtures/{}/committee_user.json'.format(fixture_dir), 'w+') as json_file:
    json.dump(committee_user_array, json_file, indent=4)
with open('./MySU/fixtures/{}/committee.json'.format(fixture_dir), 'w+') as json_file:
    json.dump(committee_array, json_file, indent=4)
with open('./MySU/fixtures/{}/committee_auth_group.json'.format(fixture_dir), 'w+') as json_file:
    json.dump(committee_auth_group_array, json_file, indent=4)
