#!/bin/bash


# ------------------------------------------
# Handling parameters
# ------------------------------------------
ARGS=$(getopt -o th --long test,help -- "$@")
eval set -- "$ARGS"
while true; do
    case "$1" in
        -t | --test ) TEST=true; shift ;;
        -h | --help ) HELP=true; shift ;;
        -- ) shift; break ;;
        * ) break ;;
    esac
done
TEST=${TEST:-false}
HELP=${HELP:-false}

if $HELP; then
    cat << "EOF"
 This script will populate with the database with data saved in the fixtures. This will either be specific
 data based used for tests or data generated from the Faker database.

 The following is done in order:
  - loads fixtures

 The following parameters are able to be used:
  -t | --test       Specify which dataset to populate the database with, -t True means only preset fixtures.
  -h | --help       Raise this help
EOF
    exit 0
else
    # ------------------------------------------
    # Detection of Python version
    # ------------------------------------------
    USE_PYTHON3=true
    if ! which python3 >& /dev/null; then
        USE_PYTHON3=false
    fi
    if ! which python >& /dev/null && ! $USE_PYTHON3; then
        echo
        echo -e "\e[1;31mMake sure you have python3.4 or python3.5 installed and appended to your PATH.\e[0m"
        echo
        exit 1
    fi

    # ------------------------------------------
    # Loading fixtures
    # ------------------------------------------
    echo
    echo -e "\e[1;32mLoading fixtures\e[0m"
    echo
    if $USE_PYTHON3; then
        python3 manage.py loaddata apps/*/fixtures/preset/*.json || exit 1
        echo -e "\e[1;32mPreset fixtures loaded\e[0m"
        if ! $TEST; then
            python3 manage.py loaddata apps/*/fixtures/generated/*.json || exit 1
            echo -e "\e[1;32mGenerated fixtures loaded\e[0m"
        fi
    else
        python manage.py loaddata apps/*/fixtures/preset/*.json || exit 1
        echo -e "\e[1;32mPreset fixtures loaded\e[0m"
        if ! $TEST; then
            python manage.py loaddata apps/*/fixtures/generated/*.json || exit 1
            echo -e "\e[1;32mGenerated fixtures loaded\e[0m"
        fi
    fi
    echo
fi
