from rest_framework.test import APIClient
from rest_framework.test import APITestCase
from apps.MySU.viewsets import association

from apps.MySU.models import User
from tests.tests import fixtures


class AssociationTestCase(APITestCase):
    fixtures = fixtures

    def setUp(self):
        self.user = User.objects.get(username='s1234567')
        self.client = APIClient()
        self.client.force_authenticate(user=self.user)

    def test_put_helios(self):
        response = self.client.put('/associations/rising-sun-alienship-helios', data={})
        self.assertEqual(403, response.status_code)

    def test_patch_helios(self):
        response = self.client.put('/associations/rising-sun-alienship-helios', data={})
        self.assertEqual(403, response.status_code)

    def test_get_ra(self):
        response = self.client.get('/associations/solar-barque-companionship-ra')
        self.assertEqual(200, response.status_code)

    def test_put_ra(self):
        response = self.client.put('/associations/solar-barque-companionship-ra', data={})
        self.assertEqual(403, response.status_code)

    def test_patch_ra(self):
        response = self.client.put('/associations/solar-barque-companionship-ra', data={})
        self.assertEqual(403, response.status_code)

    def test_post(self):
        data = {
            'name': 'foo',
            'short_name': 'bar',
            'email': 'foo@bar'
        }
        response = self.client.post('/associations', data=data)
        self.assertEqual(403, response.status_code)

    def test_get_student_union(self):
        response = self.client.get('/associations/student-union')
        self.assertEqual(200, response.status_code)

    def test_put_student_union(self):
        # TODO Photo & Logo
        data = {
            'name': 'foo',
            'short_name': 'foo',
            'founding_date': '1995-01-13',
            'description': 'Coole description of the vereniging',
            # TODO Logo
            # TODO Photo
            'dissolution_date': '2012-12-21',
            'postal_address': 'Postbus 500',
            'email': 'spam@helios.plt',
            'color': '#E6B800',
            'phone_number': '0800-0700',
            'address': '0800-0700',
            'zip_code': '0800-0700',
            'city': 'Athene',
            'country': 'Greece',
            "full_year_membership_start_date": "08-01"
        }
        response = self.client.put('/associations/student-union', data=data)
        self.assertEqual(200, response.status_code, response.data)

    def test_patch_student_union(self):
        # uneditable fields
        # ------------------
        response = self.client.patch('/associations/student-union', data={'slug': "Ha, dikke doei"})
        self.assertEqual(200, response.status_code, msg="Tried to patch {data}, {response}".format(data='slug',
                                                                                                   response=response.data))
        self.assertEqual(response.data['slug'], 'student-union', msg="slug changed; shouldn't be possible")

        # editable fields
        # ------------------
        response = self.client.patch('/associations/student-union', data={'name': 'foo'})
        self.assertEqual(200, response.status_code, msg="Tried to patch {data}".format(data='name'))
        response = self.client.patch('/associations/student-union', data={'short_name': 'foo'})
        self.assertEqual(200, response.status_code, msg="Tried to patch {data}".format(data='short_name'))
        response = self.client.patch('/associations/student-union', data={'founding_date': '1995-01-13'})
        self.assertEqual(200, response.status_code, msg="Tried to patch {data}".format(data='founding_date'))
        # TODO Photo & Logo
        response = self.client.patch('/associations/student-union', data={'dissolution_date': '2012-12-21'})
        self.assertEqual(200, response.status_code, msg="Tried to patch {data}".format(data='dissolution_date'))
        response = self.client.patch('/associations/student-union', data={'postal_address': 'Postbus 500'})
        self.assertEqual(200, response.status_code, msg="Tried to patch {data}".format(data='postal_address'))
        response = self.client.patch('/associations/student-union', data={'default_join_date': '2013-09-04'})
        self.assertEqual(200, response.status_code, msg="Tried to patch {data}".format(data='default_join_date'))
        response = self.client.patch('/associations/student-union', data={'email': 'spam@helios.plt'})
        self.assertEqual(200, response.status_code, msg="Tried to patch {data}".format(data='email'))
        response = self.client.patch('/associations/student-union', data={'color': '#E6B800'})
        self.assertEqual(200, response.status_code, msg="Tried to patch {data}".format(data='color'))
        response = self.client.patch('/associations/student-union', data={'phone_number': '0800-0700'})
        self.assertEqual(200, response.status_code, msg="Tried to patch {data}".format(data='phone_number'))
        response = self.client.patch('/associations/student-union', data={'address': '0800-0700'})
        self.assertEqual(200, response.status_code, msg="Tried to patch {data}".format(data='address'))
        response = self.client.patch('/associations/student-union', data={'zip_code': '0800-0700'})
        self.assertEqual(200, response.status_code, msg="Tried to patch {data}".format(data='zip_code'))
        response = self.client.patch('/associations/student-union', data={'city': 'Athene'})
        self.assertEqual(200, response.status_code, msg="Tried to patch {data}".format(data='city'))
        response = self.client.patch('/associations/student-union', data={'country': 'Greece'})
        self.assertEqual(200, response.status_code, msg="Tried to patch {data}".format(data='country'))

    def test_list(self):
        response = self.client.get('/associations')
        self.assertEqual(200, response.status_code)
        self.assertEqual(3, response.data['count'], msg="number of associations doesn't match expected number of associations")
        for result in response.data['results']:
            if result['slug'] == 'student-union':
                self.assertTrue(result['is_board'], result)
            if result['slug'] == 'helios':
                self.assertFalse(result['is_board'])
            if result['slug'] == 'ra':
                self.assertFalse(result['is_board'])
