from rest_framework.test import APIClient
from rest_framework.test import APITestCase

from apps.MySU.models import User
from tests.tests import fixtures
from rest_framework.parsers import JSONParser
from apps.MySU.serializers import AssociationSpecificDataSerializer


class AssociationSpecificDataTestCase(APITestCase):
    fixtures = fixtures

    def setUp(self):
        self.user = User.objects.get(username='s1234567')
        self.user.is_staff = True
        self.client = APIClient()
        self.client.force_authenticate(user=self.user)


    def test_put_helios(self):
        data = {
            "value": "No",
            "data_field": "/association_data_fields/ca3cb4f6-015a-4751-a74c-b7063d3961bb",
            "name": "ERO",
            "membership": "/memberships/26ad699a-cb8a-4fcf-a709-437cc671a4e6",
            "association": "/associations/rising-sun-alienship-helios"
        }

        response = self.client.put('/association_data/04672188-e7e9-43d4-9b2c-6acb6241c19b', data=data)
        self.assertEqual(200, response.status_code)

    def test_put_student_union(self):
        data = {
            "value": "36",
            "data_field": "/association_data_fields/8e4dcd33-d176-47fd-ad36-ba4e7c737c57",
            "name": "Favorite Number",
            "membership": "/memberships/ec47e37b-f30e-478f-9b7f-f9c7f3ca5aa6",
            "association": "/associations/student-union"
        }

        response = self.client.put('/association_data/1db97210-f323-4fe2-bfc6-10d1c048cb67', data=data)
        self.assertEqual(200, response.status_code)

    def test_patch_helios(self):
        data = {
            "value": "No",
            "data_field": "/association_data_fields/ca3cb4f6-015a-4751-a74c-b7063d3961bb",
            "name": "ERO"
        }

        response = self.client.patch('/association_data/04672188-e7e9-43d4-9b2c-6acb6241c19b', data=data)
        self.assertEqual(200, response.status_code)

    def test_patch_student_union(self):
        data = {
            "value": "36",
            "data_field": "/association_data_fields/8e4dcd33-d176-47fd-ad36-ba4e7c737c57",
            "name": "Favorite Number"
        }

        response = self.client.patch('/association_data/1db97210-f323-4fe2-bfc6-10d1c048cb67', data=data)
        self.assertEqual(200, response.status_code)

    def test_post_helios(self): #Only one answer can be given by each user
        pass

    def test_post_student_union(self): #Only one answer can be given by each user
        pass

    def test_delete_helios(self):
        data = {
            "name": "ERO",
            "membership": "/memberships/26ad699a-cb8a-4fcf-a709-437cc671a4e6",
            "slug": "04672188-e7e9-43d4-9b2c-6acb6241c19b",
            "data_field": "/association_data_fields/ca3cb4f6-015a-4751-a74c-b7063d3961bb",
            "url": "/association_data/04672188-e7e9-43d4-9b2c-6acb6241c19b",
            "value": "Yes",
            "association": "/associations/rising-sun-alienship-helios"
        }
        response = self.client.delete('/association_data/04672188-e7e9-43d4-9b2c-6acb6241c19b', data=data)
        self.assertEqual(204, response.status_code)

    def test_delete_student_union(self):
        data = {
            "name": "Favorite Number",
            "membership": "/memberships/ec47e37b-f30e-478f-9b7f-f9c7f3ca5aa6",
            "slug": "1db97210-f323-4fe2-bfc6-10d1c048cb67",
            "data_field": "/association_data_fields/8e4dcd33-d176-47fd-ad36-ba4e7c737c57",
            "url": "/association_data/1db97210-f323-4fe2-bfc6-10d1c048cb67",
            "value": "48",
            "association": "/associations/student-union"
        }
        response = self.client.delete('/association_data/1db97210-f323-4fe2-bfc6-10d1c048cb67', data=data)
        self.assertEqual(204, response.status_code)

    def test_get_helios(self):
        response = self.client.get('/association_data/04672188-e7e9-43d4-9b2c-6acb6241c19b')
        self.assertEqual(200, response.status_code)

    def test_get_student_union(self):
        response = self.client.get('/association_data/1db97210-f323-4fe2-bfc6-10d1c048cb67')
        self.assertEqual(200, response.status_code)

    def test_list(self):
        response = self.client.get('/association_data/?ordering=association')
        self.assertEqual(200, response.status_code, "Error: {}, Input: List of all specific data".format(response.data))
        self.assertEqual(response.data['count'], 5, msg="number of specific data doesn't match expected number of specific data")
        results = response.data['results']
        self.assertEqual('http://testserver/associations/rising-sun-alienship-helios', results[0]['association'], "Error: {}".format(response.data))
        self.assertEqual('http://testserver/associations/rising-sun-alienship-helios', results[1]['association'], "Error: {}".format(response.data))
        self.assertEqual('http://testserver/associations/student-union', results[2]['association'], "Error: {}".format(response.data))
        self.assertEqual('http://testserver/associations/student-union', results[3]['association'], "Error: {}".format(response.data))
        self.assertEqual('http://testserver/associations/student-union', results[4]['association'], "Error: {}".format(response.data))
