from rest_framework.test import APIClient
from rest_framework.test import APITestCase

from apps.MySU.models import AssociationSpecificData
from apps.MySU.models import User
from tests.tests import fixtures


class AssociationSpecificDataFieldsTestCase(APITestCase):
    fixtures = fixtures

    def setUp(self):
        self.user = User.objects.get(username='s1234567')
        self.client = APIClient()
        self.client.force_authenticate(user=self.user)

    def test_put_Number(self):
        data = {
            "choices": "",
            "default": "34",
            "board_only": False,
            "type": "Number",
            "name": "Favorite Number",
            "helper_text": "",
            "mandatory": False,
            "slug": "8e4dcd33-d176-47fd-ad36-ba4e7c737c57",
            "association": "/associations/student-union",
            "url": "/association_data_fields/8e4dcd33-d176-47fd-ad36-ba4e7c737c57"
        }
        response = self.client.put('/association_data_fields/8e4dcd33-d176-47fd-ad36-ba4e7c737c57', data=data)
        self.assertEqual(200, response.status_code)

    def test_put_String(self):
        data = {
            "type": "String",
            "slug": "a826094e-a8ea-49d6-aa8c-7de943e25f46",
            "mandatory": False,
            "helper_text": "",
            "board_only": False,
            "choices": "",
            "default": "False",
            "association": "/associations/student-union",
            "url": "/association_data_fields/a826094e-a8ea-49d6-aa8c-7de943e25f46",
            "name": "Drivers Test Licence"
        }
        response = self.client.put('/association_data_fields/a826094e-a8ea-49d6-aa8c-7de943e25f46', data=data)
        self.assertEqual(200, response.status_code)

    def test_put_Choice(self):
        data = {
            "type": "Choice",
            "slug": "0d6ff0a9-f2cb-40da-b81b-d4952cc16442",
            "mandatory": False,
            "helper_text": "",
            "board_only": True,
            "choices": "apple pie,quark pie",
            "default": "quark pie",
            "association": "/associations/student-union",
            "url": "/association_data_fields/0d6ff0a9-f2cb-40da-b81b-d4952cc16442",
            "name": "Quark Pie"
        }
        response = self.client.put('/association_data_fields/0d6ff0a9-f2cb-40da-b81b-d4952cc16442', data=data)
        self.assertEqual(200, response.status_code)

    def test_put_Boolean(self):
        data = {
            "choices": "",
            "mandatory": True,
            "name": "Test",
            "type": "Boolean",
            "slug": "ca3cb4f6-015a-4751-a74c-b7063d3961bb",
            "default": "False",
            "board_only": False,
            "association": "/associations/rising-sun-alienship-helios",
            "url": "/association_data_fields/ca3cb4f6-015a-4751-a74c-b7063d3961bb",
            "helper_text": ""
        }
        response = self.client.put("/association_data_fields/ca3cb4f6-015a-4751-a74c-b7063d3961bb", data=data)
        self.assertEqual(403, response.status_code) # 403 is because only the board of helios can perform changes

    def test_patch_Choice(self):
        data = {
            "type": "Choice",
            "slug": "0d6ff0a9-f2cb-40da-b81b-d4952cc16442",
            "mandatory": False,
            "board_only": True,
            "choices": "apple pie,quark pie",
            "default": "quark pie",
            "name": "Quark Pie"
        }
        response = self.client.patch('/association_data_fields/0d6ff0a9-f2cb-40da-b81b-d4952cc16442', data=data)
        self.assertEqual(200, response.status_code)

    def test_patch_Boolean(self):
        data = {
            "choices": "",
            "mandatory": True,
            "name": "Test",
            "type": "Boolean",
            "slug": "ca3cb4f6-015a-4751-a74c-b7063d3961bb",
            "default": "False",
            "board_only": False,
        }
        response = self.client.patch("/association_data_fields/ca3cb4f6-015a-4751-a74c-b7063d3961bb", data=data)
        self.assertEqual(403, response.status_code)  # 403 is because only the board of helios can perform changes

    def test_post_student_union_other(self):
        # Invalid type
        data = {
            "association": "/associations/student-union",
            "name": "Brick",
            "helper_text": "Brick",
            "default": "",
            "type": "Brick",
        }
        response = self.client.post('/association_data_fields/', data=data)
        self.assertEqual(400, response.status_code, response.data)

    def test_post_student_union_number(self):
        # Invalid default; must be string with integer
        data = {
            "association": "/associations/student-union",
            "name": "Number",
            "helper_text": "Number",
            "default": "124.123",
            "type": "Number",
        }
        response = self.client.post('/association_data_fields/', data=data)
        self.assertEqual(400, response.status_code, response.data)
        # Proper Input
        data["default"] = "48"
        response = self.client.post('/association_data_fields/', data=data)
        self.assertEqual(201, response.status_code, response.data)

    def test_post_student_union_string(self):
        # Proper Input
        data = {
            "association": "/associations/student-union",
            "name": "String",
            "helper_text": "String",
            "default": "",
            "type": "String",
        }
        response = self.client.post('/association_data_fields/', data=data)
        self.assertEqual(201, response.status_code, response.data)

    def test_post_student_union_choice(self):
        # Choice field needs choices
        data = {
            "association": "/associations/student-union",
            "name": "Choice",
            "helper_text": "Choice",
            "choices": "",
            "default": "True",
            "type": "Choice",
            "board_only": False,
            "mandatory": False,
        }
        response = self.client.post('/association_data_fields/', data=data)
        self.assertEqual(400, response.status_code, response.data)
        # default must be a valid choice
        data["choices"] = "123,456,789"
        response = self.client.post('/association_data_fields/', data=data)
        self.assertEqual(400, response.status_code, response.data)
        # Proper Input
        data["default"] = "456"
        response = self.client.post('/association_data_fields/', data=data)
        self.assertEqual(201, response.status_code, response.data)

    def test_post_student_union_boolean(self):
        # board_only means mandatory must be False
        data = {
            "association": "/associations/student-union",
            "name": "Boolean",
            "helper_text": "Boolean",
            "choices": "123",
            "default": "True",
            "type": "Boolean",
            "board_only": True,
            "mandatory": True,
        }
        response = self.client.post('/association_data_fields/', data=data)
        self.assertEqual(400, response.status_code, response.data)
        # Proper Input
        data = {
            "association": "/associations/student-union",
            "name": "Boolean",
            "helper_text": "Boolean",
            "choices": "123",
            "default": "True",
            "type": "Boolean",
            "board_only": False,
            "mandatory": False,
        }
        response = self.client.post('/association_data_fields/', data=data)
        self.assertEqual(201, response.status_code, response.data)

    def test_delete_helios(self):
        self.assertTrue(AssociationSpecificData.objects.filter(data_field__slug="de38e577-94ab-4057-a798-59049229dbe8").exists())
        response = self.client.delete('/association_data_fields/de38e577-94ab-4057-a798-59049229dbe8')
        self.assertEqual(403, response.status_code)
        self.assertTrue(AssociationSpecificData.objects.filter(data_field__slug="de38e577-94ab-4057-a798-59049229dbe8").exists())

    def test_delete_student_union(self):
        self.assertTrue(AssociationSpecificData.objects.filter(data_field__slug="0d6ff0a9-f2cb-40da-b81b-d4952cc16442").exists())
        response = self.client.delete('/association_data_fields/0d6ff0a9-f2cb-40da-b81b-d4952cc16442')
        self.assertEqual(204, response.status_code)
        self.assertFalse(AssociationSpecificData.objects.filter(data_field__slug="0d6ff0a9-f2cb-40da-b81b-d4952cc16442").exists())

    def test_get(self):
        response = self.client.get('/association_data_fields/de38e577-94ab-4057-a798-59049229dbe8')
        self.assertEqual(403, response.status_code)
        response = self.client.get('/association_data_fields/0d6ff0a9-f2cb-40da-b81b-d4952cc16442')
        self.assertEqual(200, response.status_code)

    def test_list(self):
        response = self.client.get('/association_data_fields')
        self.assertEqual(200, response.status_code, "Error: {}, Input: List of all specific data fields".format(response.data))
        self.assertEqual(response.data['count'], 11, msg="number of fields doesn't match expected number of fields")
        results = response.data['results']
        self.assertEqual('http://testserver/associations/student-union', results[0]['association'], "Error: {}".format(response.data))
        self.assertEqual('http://testserver/associations/student-union', results[1]['association'], "Error: {}".format(response.data))
        self.assertEqual('http://testserver/associations/student-union', results[2]['association'], "Error: {}".format(response.data))
        self.assertEqual('http://testserver/associations/rising-sun-alienship-helios', results[3]['association'], "Error: {}".format(response.data))
        self.assertEqual('http://testserver/associations/rising-sun-alienship-helios', results[4]['association'], "Error: {}".format(response.data))
