from rest_framework.test import APIClient
from rest_framework.test import APITestCase

from apps.MySU.models import Group
from apps.MySU.models import User
from tests.tests import fixtures


class GroupTestCase(APITestCase):
    fixtures = fixtures

    def setUp(self):
        self.user = User.objects.get(username='s1234567')
        self.client = APIClient()
        self.client.force_authenticate(user=self.user)

    def test_grant_board_permissions_helios(self):
        response = self.client.get("/groups/2d9551a0-7e40-461e-aa49-e578eb608b65")
        self.assertEqual(200, response.status_code, response.data)
        group = response.data
        self.assertEqual([], group["permissions"], "Permissions do not match")
        data = {
            "permissions": ["board"]
        }
        response = self.client.patch("/groups/2d9551a0-7e40-461e-aa49-e578eb608b65", data=data)
        self.assertEqual(403, response.status_code, response.data)

    def test_grant_board_permissions_student_union(self):
        response = self.client.get("/groups/dd0d1b61-e054-4495-b540-3a6c65a5f610")
        self.assertEqual(200, response.status_code, response.data)
        group = response.data
        self.assertEqual([], group["permissions"], "Initial permissions do not match")
        data = {
            "permissions": ["board"]
        }
        response = self.client.patch("/groups/dd0d1b61-e054-4495-b540-3a6c65a5f610", data=data)
        self.assertEqual(200, response.status_code, response.data)
        group = response.data
        self.assertEqual(["board"], group["permissions"], "Final permissions do not match")

    def test_revoke_board_permissions_helios(self):
        response = self.client.get("/groups/c2d3528a-46b3-4830-afa0-dc00831f040a")
        self.assertEqual(200, response.status_code, response.data)
        group = response.data
        self.assertEqual(["board"], group["permissions"], "Permissions do not match")
        data = {
            "permissions": []
        }
        response = self.client.patch("/groups/c2d3528a-46b3-4830-afa0-dc00831f040a", data=data)
        self.assertEqual(403, response.status_code, response.data)

    def test_revoke_board_permissions_student_union(self):
        response = self.client.get("/groups/705f007c-dba5-4da6-9dd7-ba7c22af88bc")
        self.assertEqual(200, response.status_code, response.data)
        group = response.data
        self.assertEqual(["board"], group["permissions"], "Initial permissions do not match")
        data = {
            "permissions": [],
        }
        response = self.client.patch("/groups/705f007c-dba5-4da6-9dd7-ba7c22af88bc", data=data, format="json")
        self.assertEqual(200, response.status_code, response.data)
        group = response.data
        self.assertEqual([], group["permissions"], "Final permissions do not match")

    def test_put_helios(self):
        data = {
            "theme": None,
            "board_group": True,
            "edition": None,
            "days_in_existence": 738737,
            "archived": False,
            "founding_date": "0001-01-01",
            "description": "Tempora odio dolor mollitia placeat praesentium atque. Tempora occaecati delectus possimus rem totam itaque.\nVoluptate nisi dolor ipsam corrupti harum ea voluptas. Mollitia exercitationem eligendi.",
            "permissions": [
                "board"
            ],
            "current_board": True,
            "groupmemberships": [
                {
                    "url": "http://127.0.0.1:8000/group_memberships/74c1ff41-3c0f-4037-a5cb-5230324adbae",
                    "full_name": "Myrthe Adriaensdr",
                    "slug": "74c1ff41-3c0f-4037-a5cb-5230324adbae",
                    "duty": "Chairman",
                    "description": "",
                    "date_joined": "2016-12-14",
                    "date_left": None,
                    "archived": False,
                    "email": None,
                    "order": 1,
                    "profile": "http://127.0.0.1:8000/profiles/60d23c87-9c65-4b73-8d0a-97cd168b3ac6"
                },
                {
                    "url": "http://127.0.0.1:8000/group_memberships/b95224c7-74d2-40b3-9b51-09cd7f7330a6",
                    "full_name": "Rik van den Velde",
                    "slug": "b95224c7-74d2-40b3-9b51-09cd7f7330a6",
                    "duty": "Secretary",
                    "description": "",
                    "date_joined": "2018-11-07",
                    "date_left": None,
                    "archived": False,
                    "email": None,
                    "order": 1,
                    "profile": "http://127.0.0.1:8000/profiles/c4d30249-1a58-47e2-bafc-7b0605972b9e"
                },
                {
                    "url": "http://127.0.0.1:8000/group_memberships/523cf33e-dc2e-41fa-9976-47270237ccd6",
                    "full_name": "Myrthe van der Sloot",
                    "slug": "523cf33e-dc2e-41fa-9976-47270237ccd6",
                    "duty": "Treasurer",
                    "description": "",
                    "date_joined": "2017-03-14",
                    "date_left": None,
                    "archived": False,
                    "email": None,
                    "order": 1,
                    "profile": "http://127.0.0.1:8000/profiles/18cd7edc-b2c5-4378-839a-ee8e3c618774"
                }
            ],
            "slug": "c2d3528a-46b3-4830-afa0-dc00831f040a",
            "email": None,
            "end_year": None,
            "logo": None,
            "url": "http://127.0.0.1:8000/groups/c2d3528a-46b3-4830-afa0-dc00831f040a",
            "photo": None,
            "number": 0,
            "association": "http://127.0.0.1:8000/associations/rising-sun-alienship-helios",
            "short_name": "Board of Helios",
            "dissolution_date": None,
            "parent_group": "http://127.0.0.1:8000/groups/2d9551a0-7e40-461e-aa49-e578eb608b65",
            "start_year": None,
            "alternate_name": "",
            "creed": "",
            "full_name": "board of Helios"
        }

        response = self.client.put('/groups/c2d3528a-46b3-4830-afa0-dc00831f040a', data=data)
        self.assertEqual(403, response.status_code)

    def test_put_student_union(self):
        data = {
            "theme": None,
            "board_group": True,
            "edition": None,
            "days_in_existence": 8801,
            "archived": False,
            "founding_date": "1999-07-02",
            "description": "Similique repudiandae minus sit corrupti doloribus nemo. Possimus magni dolorem.\nUnde nemo suscipit iure officia. Unde eius atque itaque placeat eius sapiente.",
            "permissions": [
                "board"
            ],
            "current_board": True,
            "groupmemberships": [
                {
                    "url": "http://127.0.0.1:8000/group_memberships/1c9ff3f7-b098-4cd5-aa00-48b762ad398b",
                    "full_name": "Femke Dries",
                    "slug": "1c9ff3f7-b098-4cd5-aa00-48b762ad398b",
                    "duty": "Chairman",
                    "description": "",
                    "date_joined": "2013-05-09",
                    "date_left": None,
                    "archived": False,
                    "email": None,
                    "order": 1,
                    "profile": "http://127.0.0.1:8000/profiles/67dc393a-73fa-4cbe-aa7c-fe59d37ce630"
                },
                {
                    "url": "http://127.0.0.1:8000/group_memberships/60127381-26e0-490f-ab89-816c573c17a2",
                    "full_name": "Max van Boven",
                    "slug": "60127381-26e0-490f-ab89-816c573c17a2",
                    "duty": "Secretary",
                    "description": "",
                    "date_joined": "2017-11-10",
                    "date_left": None,
                    "archived": False,
                    "email": None,
                    "order": 1,
                    "profile": "http://127.0.0.1:8000/profiles/a3709a0b-80dc-475a-9e9e-39b0239ffded"
                },
                {
                    "url": "http://127.0.0.1:8000/group_memberships/2252aaf8-aea1-4c7b-af89-f02b75636f1e",
                    "full_name": "Jonas Lorreijn",
                    "slug": "2252aaf8-aea1-4c7b-af89-f02b75636f1e",
                    "duty": "Treasurer",
                    "description": "",
                    "date_joined": "2019-02-16",
                    "date_left": None,
                    "archived": False,
                    "email": "odaaya@hotmail.com",
                    "order": 1,
                    "profile": "http://127.0.0.1:8000/profiles/75b5beed-97c2-4b43-bf3f-6c7ba85b3a5c"
                }
            ],
            "slug": "705f007c-dba5-4da6-9dd7-ba7c22af88bc",
            "email": None,
            "end_year": None,
            "logo": None,
            "url": "http://127.0.0.1:8000/groups/705f007c-dba5-4da6-9dd7-ba7c22af88bc",
            "photo": None,
            "number": 0,
            "association": "http://127.0.0.1:8000/associations/student-union",
            "short_name": "Board of SU",
            "dissolution_date": None,
            "parent_group": "http://127.0.0.1:8000/groups/dd0d1b61-e054-4495-b540-3a6c65a5f610",
            "start_year": None,
            "alternate_name": "College van Bestuur",
            "creed": "",
            "full_name": "board of Student Union"
        }

        response = self.client.put('/groups/705f007c-dba5-4da6-9dd7-ba7c22af88bc', data=data)
        self.assertEqual(200, response.status_code)

    def test_patch_helios(self):
        response = self.client.get("/groups/c2d3528a-46b3-4830-afa0-dc00831f040a")
        self.assertEqual(200, response.status_code, response.data)
        group = response.data
        self.assertEqual("board of Helios", group["full_name"])
        data = {
            "full_name": 'TestCie',
        }
        response = self.client.patch("/groups/c2d3528a-46b3-4830-afa0-dc00831f040a", data=data)
        self.assertEqual(403, response.status_code, response.data)

    def test_patch_student_union(self):
        response = self.client.get("/groups/705f007c-dba5-4da6-9dd7-ba7c22af88bc")
        self.assertEqual(200, response.status_code, response.data)
        group = response.data
        self.assertEqual("board of Student Union", group["full_name"])
        data = {
            "full_name": 'TestCie',
        }
        response = self.client.patch("/groups/705f007c-dba5-4da6-9dd7-ba7c22af88bc", data=data)
        self.assertEqual(200, response.status_code, response.data)
        group = response.data
        self.assertEqual("TestCie", group["full_name"])

    def test_patch_association(self):
        # confirm that we cannot change the association
        data = {
            "association": '/associations/rising-sun-alienship-helios',
        }
        response = self.client.patch("/groups/705f007c-dba5-4da6-9dd7-ba7c22af88bc", data=data)
        self.assertEqual(400, response.status_code, response.data)

    def test_post_helios_without_permissions(self):
        data = {
            "association": '/associations/rising-sun-alienship-helios',
            "full_name": "Testcie",
        }
        response = self.client.post("/groups", data=data)
        self.assertEqual(400, response.status_code, response.data)

    def test_post_helios_with_permissions(self):
        data = {
            "association": '/associations/rising-sun-alienship-helios',
            "full_name": "Testcie",
            "permissions": [
                "board"
            ],
        }
        response = self.client.post("/groups", data=data)
        self.assertEqual(400, response.status_code, response.data)

    def test_post_student_union_without_permissions(self):
        data = {
            "association": "/associations/student-union",
            "full_name": "Testcie",
        }
        response = self.client.post("/groups", data=data)
        self.assertEqual(201, response.status_code, response.data)
        group = response.data
        self.assertEqual([], group['permissions'], "Permissions do not match")

    def test_post_student_union_with_valid_permissions_2(self):
        data = {
            "association": "/associations/student-union",
            "full_name": "Testcie",
            "permissions": [],
        }
        response = self.client.post("/groups", data=data)
        self.assertEqual(201, response.status_code, response.data)
        group = response.data
        self.assertEqual([], group['permissions'], "Permissions do not match")

    def test_post_student_union_with_valid_permissions(self):
        data = {
            "association": "/associations/student-union",
            "full_name": "Testcie",
            "permissions": [
                "board"
            ]
        }
        response = self.client.post("/groups", data=data)
        self.assertEqual(201, response.status_code, response.data)
        group = response.data
        self.assertEqual(["board"], group['permissions'], "Permissions do not match")

    def test_post_student_union_with_invalid_permissions(self):
        data = {
            "association": "/associations/student-union",
            "full_name": "Testcie",
            "permissions": [
                "change_association"
            ]
        }
        response = self.client.post("/groups", data=data)
        self.assertEqual(400, response.status_code, response.data)

    def test_get_helios(self):
        response = self.client.get("/groups/218eccab-d65f-4749-8b94-c69872b952ed")
        self.assertEqual(200, response.status_code, response.data)

    def test_get_student_union(self):
        response = self.client.get("/groups/3ce35105-5042-4852-839f-3cc8af77f5ce")
        self.assertEqual(200, response.status_code, response.data)

    def test_list(self):
        response = self.client.get('/groups')
        self.assertEqual(200, response.status_code, "Error: {}, Input: List of all groups".format(response.data))
        self.assertEqual(8, response.data['count'], msg="number of groups doesn't match expected number of groups")

        response = self.client.get('/groups?board_group=True')
        self.assertEqual(200, response.status_code, "Error: {}, Input: List of all groups".format(response.data))
        self.assertEqual(5, response.data['count'], msg="number of boards doesn't match expected number of boards")

        response = self.client.get('/groups?board_group=False')
        self.assertEqual(200, response.status_code, "Error: {}, Input: List of all committees".format(response.data))
        self.assertEqual(3, response.data['count'], msg="number of committees doesn't match expected number of committees")

        self.assertEqual(Group.objects.filter(current_board=True).count(), 3, "There are too many current_boards found")

    def test_delete(self):
        response = self.client.delete('/groups/3ce35105-5042-4852-839f-3cc8af77f5ce')
        self.assertEqual(204, response.status_code, response.data)
        response = self.client.delete('/groups/c2d3528a-46b3-4830-afa0-dc00831f040a')
        self.assertEqual(403, response.status_code, response.data)
