from rest_framework.test import APIClient
from rest_framework.test import APITestCase

from apps.MySU.models import User
from tests.tests import fixtures


class GroupMembershipTestCase(APITestCase):
    fixtures = fixtures

    def setUp(self):
        self.user = User.objects.get(username='s1234567')
        self.client = APIClient()
        self.client.force_authenticate(user=self.user)

    def test_put_helios(self):
        # data = {
        #     "group": "http://127.0.0.1:8000/groups/705f007c-dba5-4da6-9dd7-ba7c22af88bc",
        #     "order": 1,
        #     "archived": False,
        #     "profile": "http://127.0.0.1:8000/profiles/a3709a0b-80dc-475a-9e9e-39b0239ffded",
        #     "description": "",
        #     "date_joined": "2017-11-10",
        #     "duty": "Secretary",
        #     "slug": "60127381-26e0-490f-ab89-816c573c17a2",
        #     "email": None,
        #     "full_name": "Max van Boden",
        #     "url": "http://127.0.0.1:8000/group_memberships/60127381-26e0-490f-ab89-816c573c17a2",
        #     "association": "http://127.0.0.1:8000/associations/student-union",
        #     "date_left": None
        # }
        # response = self.client.put('/group_memberships/60127381-26e0-490f-ab89-816c573c17a2', data=data)
        # self.assertEqual(200, response.status_code)
        pass

    def test_put_student_union(self):
        data = {
            "group": "http://127.0.0.1:8000/groups/705f007c-dba5-4da6-9dd7-ba7c22af88bc",
            "order": 1,
            "archived": False,
            "profile": "http://127.0.0.1:8000/profiles/67dc393a-73fa-4cbe-aa7c-fe59d37ce630",
            "description": "",
            "date_joined": "2013-05-09",
            "duty": "Chairman",
            "slug": "1c9ff3f7-b098-4cd5-aa00-48b762ad398b",
            "email": None,
            "full_name": "Femke Dries",
            "url": "http://127.0.0.1:8000/group_memberships/1c9ff3f7-b098-4cd5-aa00-48b762ad398b",
            "association": "http://127.0.0.1:8000/associations/student-union",
            "date_left": None
        }

        response = self.client.put('/group_memberships/1c9ff3f7-b098-4cd5-aa00-48b762ad398b', data=data)
        self.assertEqual(200, response.status_code)
        pass
    
    def test_patch_helios(self):
        response = self.client.get("/group_memberships/74c1ff41-3c0f-4037-a5cb-5230324adbae")
        group_membership = response.data
        self.assertEqual(group_membership["order"], 1)
        self.assertEqual(group_membership["duty"], "Chairman")
        data = {
            "order": 2,
            "duty": "President"
        }
        response = self.client.patch("/group_memberships/74c1ff41-3c0f-4037-a5cb-5230324adbae", data=data)
        self.assertEqual(403, response.status_code, response.data)

    def test_patch_student_union(self):
        response = self.client.get("/group_memberships/1c9ff3f7-b098-4cd5-aa00-48b762ad398b")
        self.assertEqual(200, response.status_code, response.data)
        group_membership = response.data
        self.assertEqual(group_membership["order"], 1)
        self.assertEqual(group_membership["duty"], "Chairman")
        data = {
            "order": 2,
            "duty": "President"
        }
        response = self.client.patch("/group_memberships/1c9ff3f7-b098-4cd5-aa00-48b762ad398b", data=data)
        self.assertEqual(200, response.status_code, response.data)
        group_membership = response.data
        self.assertEqual(group_membership["order"], 2)
        self.assertEqual(group_membership["duty"], "President")

    def test_post_helios(self):
        data = {
            "association": "/associations/rising-sun-alienship-helios",
            "profile": "/profiles/75b5beed-97c2-4b43-bf3f-6c7ba85b3a5c",
            "group": "/groups/218eccab-d65f-4749-8b94-c69872b952ed"
        }
        response = self.client.post('/group_memberships', data=data)
        self.assertEqual(400, response.status_code, response.data)

    def test_post_student_union(self):
        data = {
            "association": "/associations/student-union",
            "profile": "/profiles/75b5beed-97c2-4b43-bf3f-6c7ba85b3a5c",
            "group": "/groups/3ce35105-5042-4852-839f-3cc8af77f5ce"
        }
        response = self.client.post('/group_memberships', data=data)
        self.assertEqual(201, response.status_code, response.data)

    def test_post_student_union_bulk(self):
        data = [{
            "association": "/associations/student-union",
            "profile": "/profiles/75b5beed-97c2-4b43-bf3f-6c7ba85b3a5c",
            "group": "/groups/3ce35105-5042-4852-839f-3cc8af77f5ce"
        }]
        response = self.client.post('/group_memberships', data=data, format="json")
        self.assertEqual(201, response.status_code, response.data)

    def test_get_helios(self):
        pass

    def test_get_student_union(self):
        pass

    def test_list(self):
        response = self.client.get('/group_memberships')
        self.assertEqual(200, response.status_code, "Error: {}, Input: List of all groupmemberships".format(response.data))
        self.assertEqual(response.data['count'], 4, msg="number of memberships doesn't match expected number of memberships")
        for result in response.data['results']:
            self.assertEqual('http://testserver/associations/student-union', result['association'], "Error: {}".format(response.data))
        response = self.client.get('/group_memberships?django_group__group__slug=52a8437f-5499-4667-878e-50d0e36994bd')
        self.assertEqual(200, response.status_code, "Error: {}, Input: List of all groupmemberships for group 52a8437f-5499-4667-878e-50d0e36994bd".format(response.data))
        self.assertEqual(response.data['count'], 1, msg="number of memberships doesn't match expected number of memberships")
        self.assertEqual('http://testserver/profiles/75b5beed-97c2-4b43-bf3f-6c7ba85b3a5c', response.data['results'][0]['profile'], "Error: {}".format(response.data))
