from rest_framework.test import APIClient
from rest_framework.test import APITestCase

from apps.MySU.models import Association
from apps.MySU.models import AssociationMembership
from apps.MySU.models import User
from tests.tests import fixtures


class ImportAssociationMembersTestCase(APITestCase):
    fixtures = fixtures

    def setUp(self):
        self.user = User.objects.get(username="s1000000")
        self.client = APIClient()
        self.client.force_authenticate(user=self.user)
        self.client.force_login(user=self.user)

    def test_import_template(self):
        response = self.client.get("/admin/MySU/profile/import/")
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, 'django_admin/import.html')
        self.assertContains(response, 'form action="."')

    def test_import_process(self):
        self.assertEqual(6, AssociationMembership.objects.filter(association__slug="student-union").count(),
                         "Mismatch in expected initial memberships")
        data = {
            'association': Association.objects.get(slug='student-union').id,
            'csv_file': (open('tests/student-union.csv', 'rb'), 'text/csv'),
            'delimiter': ","
        }
        response = self.client.post("/admin/MySU/profile/import/", data=data, format='multipart')
        self.assertEqual(200, response.status_code, response)
        self.assertNotContains(response, 'class="error">I')
        self.assertTemplateUsed('django_admin/csv_form.html')
        self.assertEqual(6, AssociationMembership.objects.filter(association__slug="student-union").count(),
                         "Mismatch in expected memberships")

        # data is irrelevant, since data has been submitted in previous request and stored.
        response = self.client.post("/admin/MySU/profile/import-process/", data={}, follow=True, format='multipart')
        self.assertEqual(1, len(response.redirect_chain))
        self.assertEqual(200, response.status_code, response)
        self.assertNotContains(response, 'class="error">I')
        self.assertContains(response, "Import Succesful")
        self.assertTemplateUsed('django_admin/csv_form.html')
        self.assertEqual(8, AssociationMembership.objects.filter(association__slug="student-union").count(),
                         "Mismatch in expected final memberships")
