from rest_framework.test import APIClient
from rest_framework.test import APITestCase

from apps.MySU.models import User
from tests.tests import fixtures


class MembertypeTestCase(APITestCase):
    fixtures = fixtures

    def setUp(self):
        self.user = User.objects.get(username='s1234567')
        self.client = APIClient()
        self.client.force_authenticate(user=self.user)

    def test_put_helios(self):
        data = {
            "type": "General",
            "url": "membertypes/a9e124fa-51e8-410c-83d6-f6919907a839",
            "only_chooseable_by_board": 'True'
        }

        response = self.client.put('/membertypes/a9e124fa-51e8-410c-83d6-f6919907a839', data)
        self.assertEqual(400, response.status_code, response.data)

        data = {
            "type": "General",
            "url": "http://localhost:8000/membertypes/83109063-f45e-41ab-adb9-5517ae33a966",
            "only_chooseable_by_board": 'True'
        }
        response = self.client.put('/membertypes/83109063-f45e-41ab-adb9-5517ae33a966', data)
        self.assertEqual(403, response.status_code, response.data)

    def test_patch(self):
        # Student-Union
        response = self.client.patch('/membertypes/a9e124fa-51e8-410c-83d6-f6919907a839', data={'type': "test"})
        self.assertEqual(200, response.status_code, response.data)
        response = self.client.patch('/membertypes/a9e124fa-51e8-410c-83d6-f6919907a839', data={'only_chooseable_by_board': 'True'})
        self.assertEqual(200, response.status_code, response.data)

        # Helios
        response = self.client.patch('/membertypes/83109063-f45e-41ab-adb9-5517ae33a966', data={'type': "test"})
        self.assertEqual(403, response.status_code, response.data)
        response = self.client.patch('/membertypes/83109063-f45e-41ab-adb9-5517ae33a966',
                                     data={'only_chooseable_by_board': 'True'})
        self.assertEqual(403, response.status_code, response.data)

    def test_post_invalid_association(self):
        data = {
            'association': ''
        }
        response = self.client.post('/membertypes', data=data)
        self.assertEqual(400, response.status_code, response.data)

    def test_post_valid_helios(self):
        data = {
            'type': 'Cheese',
            'membership_fee': 8.5,
            'association': "/associations/student-union"
        }
        response = self.client.post('/membertypes', data=data)
        self.assertEqual(201, response.status_code, response.data)

    def test_delete(self):
        data = {
            "change_membertype_to": "a9e124fa-51e8-410c-83d6-f6919907a839"
        }
        response = self.client.delete('/membertypes/9e3f9f5a-15f2-489f-bd94-a92fed0f4154', data=data)
        self.assertEqual(204, response.status_code)
        response = self.client.get('/membertypes/9e3f9f5a-15f2-489f-bd94-a92fed0f4154')
        self.assertEqual(404, response.status_code)

    def test_get(self):
        response = self.client.get('/membertypes/a9e124fa-51e8-410c-83d6-f6919907a839')
        self.assertEqual(200, response.status_code)

    def test_list(self):
        response = self.client.get('/membertypes')
        self.assertEqual(200, response.status_code)
        self.assertEqual(response.data['count'], 6,
                         msg="number of associations doesn't match expected number of associations")

        for result in response.data['results']:
            if result['only_chooseable_by_board'] == 'True':
                self.assertEqual(result['assocation'].slug, 'student-union')
