from rest_framework.test import APIClient
from rest_framework.test import APITestCase

from apps.MySU.models import AssociationMembership
from apps.MySU.models import User
from tests.tests import fixtures


class AssociationTestCase(APITestCase):
    fixtures = fixtures

    def setUp(self):
        self.user = User.objects.get(username='s1234567')
        self.client = APIClient()
        self.client.force_authenticate(user=self.user)

    def test_patch_pay_by(self):
        response = self.client.get("/memberships/ec47e37b-f30e-478f-9b7f-f9c7f3ca5aa6")
        self.assertEqual(200, response.status_code, response.data)
        self.assertEqual(AssociationMembership.PayBy.DEBT_COLLECTION.value, response.data["pay_by"], response.data)
        self.assertTrue(response.data.get("debt_collection_mandate_id"), response.data)

        response = self.client.patch("/memberships/ec47e37b-f30e-478f-9b7f-f9c7f3ca5aa6", data={"pay_by": AssociationMembership.PayBy.INVOICE.value})
        self.assertEqual(200, response.status_code, response.data)
        self.assertEqual(AssociationMembership.PayBy.INVOICE.value, response.data["pay_by"], response.data)
        self.assertFalse(response.data.get("debt_collection_mandate_id"), response.data)

        response = self.client.patch("/memberships/ec47e37b-f30e-478f-9b7f-f9c7f3ca5aa6", data={"pay_by": AssociationMembership.PayBy.DEBT_COLLECTION.value})
        self.assertEqual(400, response.status_code, response.data)

        response = self.client.patch("/memberships/ec47e37b-f30e-478f-9b7f-f9c7f3ca5aa6", data={"pay_by": AssociationMembership.PayBy.DEBT_COLLECTION.value, 'debt_collection_mandate_given': True})
        self.assertEqual(200, response.status_code, response.data)
        self.assertEqual(AssociationMembership.PayBy.DEBT_COLLECTION.value, response.data["pay_by"], response.data)
        self.assertTrue(response.data.get("debt_collection_mandate_id"), response.data)

class SepaMandateTestCase(APITestCase):
    fixtures = fixtures

    def setUp(self):
        self.user = User.objects.get(username='s1234567')
        self.client = APIClient()
        self.client.force_authenticate(user=self.user)

    def test_patch(self):
        response = self.client.patch('/sepa_mandates/033bf6d5-302a-4bdd-a97e-6b8e36160108', data={})
        self.assertEqual(403, response.status_code)

    def test_put(self):
        response = self.client.put('/sepa_mandates/033bf6d5-302a-4bdd-a97e-6b8e36160108', data={})
        self.assertEqual(403, response.status_code)

    def test_post(self):
        response = self.client.post('/sepa_mandates', data={})
        self.assertEqual(403, response.status_code)

    def test_get(self):
        response = self.client.get('/sepa_mandates/db997951-a3d4-428c-b349-4b198b35a853')
        self.assertEqual(200, response.status_code)
        response = self.client.get('/sepa_mandates/0181ba87-ce53-413f-8690-64b458f73111')
        self.assertEqual(404, response.status_code)

    def test_list(self):
        response = self.client.get('/sepa_mandates')
        self.assertEqual(200, response.status_code)
        self.assertEqual(response.data['count'], 7,
                         msg="number of sepa_mandates doesn't match expected number of sepa mandates")

    def test_delete_self(self):
        pass
