from rest_framework.test import APIClient
from rest_framework.test import APITestCase

from apps.MySU.models import User
from tests.tests import fixtures


class UserTestCase(APITestCase):
    fixtures = fixtures

    def setUp(self):
        self.user = User.objects.get(username='s1234567')
        self.client = APIClient()
        self.client.force_authenticate(user=self.user)

    def test_put_self(self):
        data = {'student_number': 's7654321'}
        response = self.client.put('/users/efc00c85-87a8-446d-ad71-2d2d07d4b588', data=data)
        self.assertEqual(200, response.status_code, "Error: {}, Input: Other student number".format(response.data))

    def test_put_other(self):
        data = {'student_number': 's7654321'}
        response = self.client.put('/users/5b5742e6-b52b-4133-8151-a49d7b186026', data=data)
        self.assertEqual(404, response.status_code, "Error: {}, Input: Other student number".format(response.data))

    def test_patch_self(self):
        data = {
            'email': "cheese@goo.com"
        }
        response = self.client.patch('/users/efc00c85-87a8-446d-ad71-2d2d07d4b588', data=data)
        self.assertEqual(200, response.status_code, "Error: {}, Input: Other student number".format(response.data))
        self.assertEqual(response.data['email'], "cheese@goo.com")

    def test_patch_other(self):
        data = {
            'student_number': 's7654321',
            'email': "Cheese@goo.com"
        }
        response = self.client.patch('/users/5b5742e6-b52b-4133-8151-a49d7b186026', data=data)
        self.assertEqual(404, response.status_code, "Error: {}, Input: Other student number".format(response.data))

    def test_get_self(self):
        response = self.client.get('/users/me')
        self.assertEqual(200, response.status_code, "Error: {}, Input: user via user/me".format(response.data))
        user_from_users_me = response.data
        response = self.client.get('/users/efc00c85-87a8-446d-ad71-2d2d07d4b588')
        self.assertEqual(200, response.status_code, "Error: {}, Input: user via user/slug".format(response.data))
        user_from_users_slug = response.data
        self.assertEqual(user_from_users_slug, user_from_users_me, "User from /me doesn't match user from /slug")
        response = self.client.get('/users/me?expand=preferred_profile.study,profiles.study')
        self.assertEqual(200, response.status_code, "Error: {}, Input: user via user/me".format(response.data))
        response = self.client.get('/users/me?expand=profiles,association_memberships.association')
        self.assertEqual(200, response.status_code, "Error: {}, Input: user via user/me".format(response.data))
        memberships = response.data["association_memberships"]
        su_membership_board_status = False
        for membership in memberships:
            if membership["association"]["slug"] == "student-union":
                su_membership_board_status = membership["association"]["is_board"]
        self.assertTrue(su_membership_board_status, "User should be a board member be `is_board` is False.")

    def test_get_other(self):
        response = self.client.get('/users/5b5742e6-b52b-4133-8151-a49d7b186026')
        self.assertEqual(404, response.status_code, "Error: {}, Input: user via user/slug".format(response.data))

    def test_list(self):
        response = self.client.get('/users')
        self.assertEqual(200, response.status_code, "Error: {}, Input: List of all users".format(response.data))
        self.assertEqual(response.data['count'], 1, msg="number of users doesn't match expected number of users")
