from rest_framework.test import APIClient
from rest_framework.test import APITestCase
from rest_framework_extensions import utils
from rest_framework_extensions.settings import extensions_api_settings

from apps.calendar.models import Enrollment
from apps.MySU.models import User
from tests.tests import fixtures


class EnrollmentTestCase(APITestCase):
    fixtures = fixtures

    def setUp(self):
        self.user = User.objects.get(username='s1234567')
        self.client = APIClient()
        self.client.force_authenticate(user=self.user)
        self.protection_headers = {
            utils.prepare_header_name(extensions_api_settings.DEFAULT_BULK_OPERATION_HEADER_NAME): 'true'
        }

    def test_put_helios(self):
        pass

    def test_put_student_union(self):
        pass

    def test_patch_helios(self):
        pass

    def test_patch_student_union(self):
        pass

    def test_post_helios__self(self):
        data = {
            "association": "/associations/rising-sun-alienship-helios",
            "event": "/events/cb4b4ca7-4c8d-4312-bd96-50d7c4e2f1eb",
            "enrollment_option": "/enrollment_options/cb4b4ca7-4c8d-4312-bd96-50d7c4e2f1eb",
            "profile": "/profiles/75b5beed-97c2-4b43-bf3f-6c7ba85b3a5c"
        }
        response = self.client.post('/enrollments', data=data)
        self.assertEqual(201, response.status_code, "Error: {}, Input: Enrollment for helios, Data: {}".format(response.data, response.data))

    def test_post_helios__other(self):
        data = {
            "association": "/associations/rising-sun-alienship-helios",
            "event": "/events/8331c272-825d-4d6a-abd7-4f39fd964322",
            "enrollment_option": "/enrollment_options/e3775f3f-7c0b-417b-821c-a16d474ddda5",
            "profile": "/profiles/83d82e6d-9e9c-4705-a803-49865f32eab7"
        }
        response = self.client.post('/enrollments', data=data)
        self.assertEqual(400, response.status_code, "Error: {}, Input: Enrollment for helios, Data: {}".format(response.data, data))

    def test_post_student_union(self):
        data = {
            "association": "/associations/student-union",
            "event": "/events/00a9c8bd-b51b-4904-ace6-8aff2ee27dba",
            "enrollment_option": "/enrollment_options/00a9c8bd-b51b-4904-ace6-8aff2ee27dba",
            "profile": "/profiles/83d82e6d-9e9c-4705-a803-49865f32eab7"
        }
        response = self.client.post('/enrollments', data=data)
        self.assertEqual(201, response.status_code, "Error: {}, Input: Enrollment for helios, Data: {}".format(response.data, data))

    def test_get_helios(self):
        response = self.client.get("/enrollments/e3775f3f-7c0b-417b-821c-a16d474ddda5")
        self.assertEqual(200, response.status_code, "Error: {}, Input: Enrollment for helios".format(response.data))

    def test_get_student_union(self):
        response = self.client.get("/enrollments/89ac22dc-5a71-4fac-8af4-5844b7eb71a1")
        self.assertEqual(200, response.status_code, "Error: {}, Input: Enrollment for helios".format(response.data))

    def test_list(self):
        response = self.client.get('/enrollments')
        self.assertEqual(200, response.status_code, "Error: {}, Input: List of all enrollments".format(response.data))
        self.assertEqual(3, response.data['count'], msg="number of enrollments doesn't match expected number of enrollments")

    def delete_enrollment(self):
        response = self.client.delete('/enrollments/89ac22dc-5a71-4fac-8af4-5844b7eb71a1')
        self.assertEqual(400, response.status_code, "Error: {}".format(response.data))
        response = self.client.delete('/enrollments/e3775f3f-7c0b-417b-821c-a16d474ddda5')
        self.assertEqual(204, response.status_code, "Error: {}".format(response.data))

    def test_bulk_create(self):
        enrollment_1 = {
            "association": "/associations/student-union",
            "event": "/events/00a9c8bd-b51b-4904-ace6-8aff2ee27dba",
            "enrollment_option": "/enrollment_options/00a9c8bd-b51b-4904-ace6-8aff2ee27dba",
            "profile": "/profiles/75b5beed-97c2-4b43-bf3f-6c7ba85b3a5c"
        }
        enrollment_2 = {
            "association": "/associations/student-union",
            "event": "/events/ebbf1381-5bf3-428d-a6cf-96039911a170",
            "enrollment_option": "/enrollment_options/89ac22dc-5a71-4fac-8af4-5844b7eb71a1",
            "profile": "/profiles/be3f26fb-97d1-4270-8367-3e8539b3484d"
        }
        data = [enrollment_1, enrollment_2]
        response = self.client.post('/enrollments', data=data, format="json")
        self.assertEqual(201, response.status_code, response.data)

    def test_bulk_delete__with_header(self):
        number_of_enrollments_SU_before = Enrollment.objects.filter(association__slug="student-union").count()
        number_of_enrollments_H_before = Enrollment.objects.filter(association__slug="rising-sun-alienship-helios").count()

        enrollments = [enrollment.slug for enrollment in Enrollment.objects.filter(slug="d87064ed-3869-486b-8a3b-3763f1502321").all()]
        self.assertEqual(1, len(enrollments))
        response = self.client.delete('/enrollments?slug__in={}'.format(",".join(enrollments)), **self.protection_headers)
        self.assertEqual(204, response.status_code, response.data)

        number_of_enrollments_SU_after = Enrollment.objects.filter(association__slug="student-union").count()
        number_of_enrollments_H_after = Enrollment.objects.filter(association__slug="rising-sun-alienship-helios").count()
        self.assertEqual(number_of_enrollments_SU_before - 1, number_of_enrollments_SU_after, "Number of enrollments of student-union does not match")
        self.assertEqual(number_of_enrollments_H_before, number_of_enrollments_H_after, "Number of enrollments of Helios does not match")
