import datetime

from rest_framework.test import APIClient
from rest_framework.test import APITestCase

from apps.calendar.models import Event
from apps.MySU.models import User
from tests.tests import fixtures


class EventTestCase(APITestCase):
    fixtures = fixtures

    def setUp(self):
        self.user = User.objects.get(username='s1234567')
        self.client = APIClient()
        self.client.force_authenticate(user=self.user)

    def test_put_helios(self):
        pass

    def test_put_student_union(self):
        pass

    def test_patch_helios(self):
        data = {'public': True}
        response = self.client.patch('/events/e3775f3f-7c0b-417b-821c-a16d474ddda5', data=data)
        self.assertEqual(403, response.status_code, "Error: {}, Input: Change private to public, Data: {}".format(response.data, data))
        data = {'start_date': str(datetime.datetime(2019, 1, 1)), 'end_date': str(datetime.datetime(2019, 1, 2))}
        response = self.client.patch('/events/e3775f3f-7c0b-417b-821c-a16d474ddda5', data=data)
        self.assertEqual(403, response.status_code, "Error: {}, Input: Change start and end dates, Data: {}".format(response.data, data))

    def test_patch_student_union(self):
        data = {'start_date': datetime.datetime(2020, 9, 12), 'end_date': datetime.datetime(2020, 9, 13)}
        response = self.client.patch('/events/89ac22dc-5a71-4fac-8af4-5844b7eb71a1', data=data)
        self.assertEqual(200, response.status_code, "Error: {}, Input: Change start and end dates, Data: {}".format(response.data, data))
        data = {'public': True}
        response = self.client.patch('/events/89ac22dc-5a71-4fac-8af4-5844b7eb71a1', data=data)
        self.assertEqual(200, response.status_code, "Error: {}, Input: Change private to public, Data: {}".format(response.data, data))
        data = {'enrollable': True}
        response = self.client.patch('/events/89ac22dc-5a71-4fac-8af4-5844b7eb71a1', data=data)
        self.assertEqual(400, response.status_code, "Error: {}, Input: Enrollable without dates, Data: {}".format(response.data, data))
        data = {'enrollable': True, 'enrollable_from': datetime.datetime(2020, 9, 10), 'enrollable_until': datetime.datetime(2020, 9, 9)}
        response = self.client.patch('/events/89ac22dc-5a71-4fac-8af4-5844b7eb71a1', data=data)
        self.assertEqual(400, response.status_code, "Error: {}, Input: enrollable_until before enrollable_from, Data: {}".format(response.data, data))
        data = {'enrollable': True, 'enrollable_from': datetime.datetime(2020, 9, 10), 'enrollable_until': datetime.datetime(2020, 9, 14)}
        response = self.client.patch('/events/89ac22dc-5a71-4fac-8af4-5844b7eb71a1', data=data)
        self.assertEqual(400, response.status_code, "Error: {}, Input: Enrollable with invalid dates, Data: {}".format(response.data, data))
        data = {'enrollable': True, 'enrollable_from': datetime.datetime(2020, 9, 10), 'enrollable_until': datetime.datetime(2020, 9, 12)}
        response = self.client.patch('/events/89ac22dc-5a71-4fac-8af4-5844b7eb71a1', data=data)
        self.assertEqual(400, response.status_code, "Error: {}, Input: Valid enrollable without max number of enrollments, Data: {}".format(response.data, data))
        data = {'enrollable': True, 'enrollable_from': datetime.datetime(2020, 9, 10), 'enrollable_until': datetime.datetime(2020, 9, 12), 'max_number_of_enrollments': 48}
        response = self.client.patch('/events/89ac22dc-5a71-4fac-8af4-5844b7eb71a1', data=data)
        self.assertEqual(400, response.status_code, "Error: {}, Input: Enrollable but public, Data: {}".format(response.data, data))
        data['public'] = False
        response = self.client.patch('/events/89ac22dc-5a71-4fac-8af4-5844b7eb71a1', data=data)
        self.assertEqual(200, response.status_code, "Error: {}, Input: Valid enrollable, Data: {}".format(response.data, data))
        # response = self.client.patch('/events/89ac22dc-5a71-4fac-8af4-5844b7eb71a1', data=data)
        # self.assertEqual(400, response.status_code, "Error: {}, Input: Valid enrollable without options, Data: {}".format(response.data, data))
        # data['enrollment_options_write'] = [{"name": "BBQ", "participation_fee": "Chicken"}]
        # response = self.client.patch('/events/89ac22dc-5a71-4fac-8af4-5844b7eb71a1', data=data)
        # self.assertEqual(400, response.status_code, "Error: {}, Input: Valid enrollable with invalid options, Data: {}".format(response.data, data))
        # data['enrollment_options_write'] = [{"name": "BBQ"}]
        # response = self.client.patch('/events/89ac22dc-5a71-4fac-8af4-5844b7eb71a1', data=data)
        # self.assertEqual(400, response.status_code, "Error: {}, Input: Valid enrollable with missing options, Data: {}".format(response.data, data))
        # data['enrollment_options_write'] = [{"name": "BBQ", "participation_fee": 48}, {"name": "BBQ", "participation_fee": 0.1}]
        # response = self.client.patch('/events/89ac22dc-5a71-4fac-8af4-5844b7eb71a1', data=data)
        # self.assertEqual(400, response.status_code, "Error: {}, Input: Valid enrollable with duplicate options, Data: {}".format(response.data, data))
        # data['enrollment_options_write'] = [{"name": "BBQ", "participation_fee": 48}]
        # response = self.client.patch('/events/89ac22dc-5a71-4fac-8af4-5844b7eb71a1', data=data)
        # self.assertEqual(200, response.status_code, "Error: {}, Input: Valid enrollable with valid options, Data: {}".format(response.data, data))
        data = {'unenrollable': True}
        response = self.client.patch('/events/89ac22dc-5a71-4fac-8af4-5844b7eb71a1', data=data)
        self.assertEqual(400, response.status_code, "Error: {}, Input: Unenrollable without dates, Data: {}".format(response.data, data))
        data = {'unenrollable': True, 'unenrollable_until': datetime.datetime(2020, 9, 9)}
        response = self.client.patch('/events/89ac22dc-5a71-4fac-8af4-5844b7eb71a1', data=data)
        self.assertEqual(400, response.status_code, "Error: {}, Input: unenrollable_until before enrollable_from, Data: {}".format(response.data, data))
        data = {'unenrollable': True, 'unenrollable_until': datetime.datetime(2020, 9, 13)}
        response = self.client.patch('/events/89ac22dc-5a71-4fac-8af4-5844b7eb71a1', data=data)
        self.assertEqual(400, response.status_code, "Error: {}, Input: unenrollable_until after end of event, Data: {}".format(response.data, data))
        data = {'unenrollable': True, 'unenrollable_until': datetime.datetime(2020, 9, 12)}
        response = self.client.patch('/events/89ac22dc-5a71-4fac-8af4-5844b7eb71a1', data=data)
        self.assertEqual(200, response.status_code, "Error: {}, Input: Valid event with enroll and unenroll, Data: {}".format(response.data, data))

    def test_post_helios(self):
        data = {
            'association': '/associations/rising-sun-alienship-helios',
            'type': '/event_types/e3775f3f-7c0b-417b-821c-a16d474ddda5',
            'name': 'Dies Natalis',
            'description': 'Ride the chariot into the next day!',
            'start_date': str(datetime.datetime(2019, 6, 21)),
            'end_date': str(datetime.datetime(2019, 6, 22)),
            'enrollable': True,
            'enrollable_from': str(datetime.datetime(2019, 6, 10)),
            'enrollable_until': str(datetime.datetime(2019, 6, 12)),
            'max_number_of_enrollments': 48
        }
        response = self.client.post('/events', data=data)
        self.assertEqual(400, response.status_code, "Error: {}, Input: Valid enrollable event, Data: {}".format(response.data, data))

    def test_post_student_union(self):
        data = {
            'association': '/associations/student-union',
            'type': '/event_types/e3775f3f-7c0b-417b-821c-a16d474ddda5',
            'name': 'Dies Natalis',
            'description': 'Student Union is the better!',
            'start_date': datetime.datetime(2019, 9, 12),
            'end_date': datetime.datetime(2019, 9, 13)
        }
        response = self.client.post('/events', data=data)
        self.assertEqual(400, response.status_code, "Error: {}, Input: Invalid type supplied, Data: {}".format(response.data, data))
        data['type'] = '/event_types/89ac22dc-5a71-4fac-8af4-5844b7eb71a1'
        response = self.client.post('/events', data=data)
        self.assertEqual(201, response.status_code, "Error: {}, Input: Valid event without enroll, Data: {}".format(response.data, data))
        data['enrollable'] = True
        response = self.client.post('/events', data=data)
        self.assertEqual(400, response.status_code, "Error: {}, Input: Enrollable event without dates, Data: {}".format(response.data, data))
        data['enrollable_from'] = datetime.datetime(2019, 9, 10)
        response = self.client.post('/events', data=data)
        self.assertEqual(400, response.status_code, "Error: {}, Input: Missing enrollable_until, Data: {}".format(response.data, data))
        data['enrollable_until'] = datetime.datetime(2019, 9, 14)
        response = self.client.post('/events', data=data)
        self.assertEqual(400, response.status_code, "Error: {}, Input: enrollable_until after event end, Data: {}".format(response.data, data))
        data['enrollable_until'] = datetime.datetime(2019, 9, 12)
        response = self.client.post('/events', data=data)
        self.assertEqual(400, response.status_code, "Error: {}, Input: Missing max_number_of_enrollments, Data: {}".format(response.data, data))
        data['max_number_of_enrollments'] = 48
        response = self.client.post('/events', data=data)
        self.assertEqual(201, response.status_code, "Error: {}, Input: Valid enrollable event, Data: {}".format(response.data, data))
        # response = self.client.post('/events', data=data)
        # self.assertEqual(400, response.status_code, "Error: {}, Input: Valid enrollable event without options, Data: {}".format(response.data, data))
        # data['enrollment_options_write'] = [{"name": "BBQ", "participation_fee": 48}]
        # response = self.client.post('/events', data=data)
        # self.assertEqual(201, response.status_code, "Error: {}, Input: Valid enrollable event with options, Data: {}".format(response.data, data))
        data['unenrollable'] = True
        response = self.client.post('/events', data=data)
        self.assertEqual(400, response.status_code, "Error: {}, Input: Missing unenrollable_until, Data: {}".format(response.data, data))
        data['unenrollable_until'] = datetime.datetime(2019, 9, 9)
        response = self.client.post('/events', data=data)
        self.assertEqual(400, response.status_code, "Error: {}, Input: unenrollable_until before enrollable_from, Data: {}".format(response.data, data))
        data['unenrollable_until'] = datetime.datetime(2019, 9, 13)
        response = self.client.post('/events', data=data)
        self.assertEqual(400, response.status_code, "Error: {}, Input: unenrollable_until after end of event, Data: {}".format(response.data, data))
        data['unenrollable_until'] = datetime.datetime(2019, 9, 12)
        response = self.client.post('/events', data=data)
        self.assertEqual(201, response.status_code, "Error: {}, Input: Valid event with enroll and unenroll, Data: {}".format(response.data, data))
        data['public'] = True
        response = self.client.post('/events', data=data)
        self.assertEqual(400, response.status_code, "Error: {}, Input: Public event with (un)enrolling, Data: {}".format(response.data, data))

    def test_get_helios(self):
        response = self.client.get('/events/e3775f3f-7c0b-417b-821c-a16d474ddda5')
        self.assertEqual(200, response.status_code, "Error: {}, Input: Public Event".format(response.data))
        response = self.client.get('/events/8331c272-825d-4d6a-abd7-4f39fd964322')
        self.assertEqual(200, response.status_code, "Error: {}, Input: Private Event".format(response.data))
        response = self.client.get('/events/ce057518-5a6d-444f-b1dc-d569b9c21a37')
        self.assertEqual(404, response.status_code, "Error: {}, Input: Hidden Event".format(response.data))

    def test_get_student_union(self):
        # without expanded fields
        response = self.client.get('/events/ebbf1381-5bf3-428d-a6cf-96039911a170')
        self.assertEqual(200, response.status_code, response.data)
        response = self.client.get('/events/89ac22dc-5a71-4fac-8af4-5844b7eb71a1')
        self.assertEqual(200, response.status_code, response.data)
        response = self.client.get('/events/468b40cf-71a3-4c49-96c3-65dd99aaac00')
        self.assertEqual(200, response.status_code, "Error: {}, Input: Hidden Event".format(response.data))
        # with expanded fields
        response = self.client.get('/events/ebbf1381-5bf3-428d-a6cf-96039911a170?expand=enrollment_options,enrollments')
        self.assertEqual(200, response.status_code, response.data)
        response = self.client.get('/events/89ac22dc-5a71-4fac-8af4-5844b7eb71a1?expand=enrollment_options,enrollments')
        self.assertEqual(200, response.status_code, response.data)

    def test_get_ra(self):
        response = self.client.get('/events/fc753977-8a99-4905-a453-c9eeb0df3830')
        self.assertEqual(200, response.status_code, "Error: {}, Input: Public Event".format(response.data))
        response = self.client.get('/events/eabf5cec-045a-49df-a69e-b0032e144462')
        self.assertEqual(404, response.status_code, "Error: {}, Input: Private Event".format(response.data))

    def test_list(self):
        response = self.client.get('/events')
        self.assertEqual(200, response.status_code, "Error: {}, Input: List of all events".format(response.data))
        self.assertEqual(8, response.data['count'], msg="number of events doesn't match expected number of events")
        results = response.data['results']
        events_student_union = list(filter(lambda x: x['association'] == 'http://testserver/associations/student-union', results))
        events_helios = list(filter(lambda x: x['association'] == 'http://testserver/associations/rising-sun-alienship-helios', results))
        events_ra = list(filter(lambda x: x['association'] == 'http://testserver/associations/solar-barque-companionship-ra', results))
        self.assertEqual(4, len(events_student_union), "Wrong number of SU events found")
        self.assertEqual(3, len(events_helios), "Wrong number of Helios events found")
        self.assertEqual(1, len(events_ra), "Wrong number of Ra events found")
        self.assertEqual("Tau day", events_student_union[0]['name'], 'Wrong SU event name found')
        self.assertFalse(events_student_union[0]['public'], 'SU event should be private')
        self.assertEqual('Hidden event', events_student_union[1]['name'], 'Wrong SU event name found')
        self.assertFalse(events_student_union[1]['public'], 'SU event should be private')
        self.assertTrue(events_student_union[1]['hidden_for_members'], 'SU event should be hidden')
        self.assertEqual("New Year's day", events_student_union[2]['name'], 'Wrong SU event name found')
        self.assertTrue(events_student_union[2]['public'], 'SU event should be public')
        self.assertEqual('Pi day', events_student_union[3]['name'], 'Wrong SU event name found')
        self.assertFalse(events_student_union[3]['public'], 'SU event should be private')
        self.assertEqual('Knotsbal tournament', events_helios[0]['name'], 'Wrong Helios event name found')
        self.assertTrue(events_helios[0]['public'], 'Helios event "Knotsbal tournament" should be private')
        self.assertEqual('Padel tournament', events_helios[1]['name'], 'Wrong Helios event name found')
        self.assertFalse(events_helios[1]['public'], 'Helios event "Padel tournament" should be private')
        self.assertEqual('Hearts tournament', events_helios[2]['name'], 'Wrong Helios event name found')
        self.assertFalse(events_helios[2]['public'], 'Helios event "Hearts tournament" should be private')
        self.assertEqual('Karaoke', events_ra[0]['name'], 'Wrong Ra event name found')
        self.assertTrue(events_ra[0]['public'], 'Ra event should be public')

    def test_make_not_enrollable(self):
        response = self.client.get('/events/00a9c8bd-b51b-4904-ace6-8aff2ee27dba')
        event = response.data
        self.assertEqual(True, event["enrollable"], "Event should be enrollable")
        # enrollment_options = event['enrollment_options']
        # self.assertEqual(1, len(enrollment_options), "Number of enrollment option is wrong")
        data = {
            "enrollable": False,
            "unenrollable": False,
        }
        response = self.client.patch('/events/00a9c8bd-b51b-4904-ace6-8aff2ee27dba', data=data)
        self.assertEqual(400, response.status_code, "Event should not be able to be made unenrollable")
        # self.assertEqual(200, response.status_code, "Error: {}, Input: Event made not enrollable".format(response.data))
        # event = response.data
        # enrollment_options = event['enrollment_options']
        # self.assertEqual(0, len(enrollment_options), "After making event not enrollable the number of options should be 0")

    def test_make_enrollable(self):
        response = self.client.get('/events/89ac22dc-5a71-4fac-8af4-5844b7eb71a1')
        event = response.data
        self.assertEqual(False, event["enrollable"], {"message": "Event should not be enrollable", "Event": event})
        # enrollment_options = event['enrollment_options']
        # self.assertEqual(0, len(enrollment_options), "Number of enrollment option is wrong")
        data = {
            "public": False,
            "enrollable": True,
            "enrollable_from": "2019-01-01 00:00:00",
            "enrollable_until": "2020-01-01 00:00:00",
            "max_number_of_enrollments": 0,
            # 'enrollment_options_write': [
            #     {"name": "BBQ", "participation_fee": 48},
            #     {"name": "Henk", "participation_fee": 0.1}
            # ]
        }
        response = self.client.patch('/events/89ac22dc-5a71-4fac-8af4-5844b7eb71a1', data=data)
        self.assertEqual(200, response.status_code, "Error: {}, Input: Event made enrollable".format(response.data))
        # event = response.data
        # enrollment_options = event['enrollment_options']
        # self.assertEqual(2, len(enrollment_options), "After making event enrollable the number of options should be 2")

    def test_delete_student_union(self):
        pass
        # response = self.client.delete('/events/89ac22dc-5a71-4fac-8af4-5844b7eb71a1')
        # self.assertEqual(204, response.status_code, "Error: {}, Input: Deleted event".format(response.data))
        # response = self.client.get('/events/89ac22dc-5a71-4fac-8af4-5844b7eb71a1')
        # self.assertEqual(404, response.status_code, "Error: {}, Input: Deleted event".format(response.data))

    def test_delete_helios(self):
        response = self.client.delete('/events/e3775f3f-7c0b-417b-821c-a16d474ddda5')
        self.assertEqual(403, response.status_code, "Error: {}, Input: Deleted event".format(response.data))
        response = self.client.get('/events/e3775f3f-7c0b-417b-821c-a16d474ddda5')
        self.assertEqual(200, response.status_code, "Error: {}, Input: Deleted event".format(response.data))

    def test_delete_event_has_started(self):
        response = self.client.delete('/events/89ac22dc-5a71-4fac-8af4-5844b7eb71a1')
        self.assertEqual(400, response.status_code, "Error: {}, Input: Deleted event".format(response.data))
        response = self.client.get('/events/89ac22dc-5a71-4fac-8af4-5844b7eb71a1')
        self.assertEqual(200, response.status_code, "Error: {}, Input: Deleted event".format(response.data))

    def test_delete_event_success(self):
        response = self.client.delete('/events/00a9c8bd-b51b-4904-ace6-8aff2ee27dba')
        self.assertEqual(204, response.status_code, "Error: {}, Input: Deleted event".format(response.data))

    def test_delete_with_enrollments(self):
        response = self.client.delete('/events/ebbf1381-5bf3-428d-a6cf-96039911a170')
        self.assertEqual(400, response.status_code, "Error: {}, Input: Deleted event".format(response.data))

    def test_delete_associated_enrollments_options(self):
        response = self.client.delete('/events/00a9c8bd-b51b-4904-ace6-8aff2ee27dba')
        self.assertEqual(204, response.status_code, "Error: {}, Input: Deleted event".format(response.data))
        response = self.client.get('/enrollment_options/00a9c8bd-b51b-4904-ace6-8aff2ee27dba')
        self.assertEqual(404, response.status_code, "Error: {}, Input: Deleted event".format(response.data))
