from rest_framework.test import APIClient
from rest_framework.test import APITestCase
from rest_framework_extensions import utils
from rest_framework_extensions.settings import extensions_api_settings

from apps.financial.models import Invoice
from apps.MySU.models import User
from tests.tests import fixtures


class InvoiceTestCase(APITestCase):
    fixtures = fixtures

    def setUp(self):
        self.user = User.objects.get(username='s1234567')
        self.client = APIClient()
        self.client.force_authenticate(user=self.user)
        self.protection_headers = {
            utils.prepare_header_name(extensions_api_settings.DEFAULT_BULK_OPERATION_HEADER_NAME): 'true'
        }

    def test_put_helios(self):
        pass
        # data = {
        #     "url": "invoices/8a8f1509-071f-416b-a222-33208fb25059",
        # }
        # response = self.client.put('/invoices/8a8f1509-071f-416b-a222-33208fb25059', data)
        # self.assertEqual(405, response.status_code, response.data)

    def test_put_student_union(self):
        pass
        # data = {
        #     "url": "invoices/93f47e50-444e-450e-94b9-cabf535df342",
        # }
        # response = self.client.put('/invoices/93f47e50-444e-450e-94b9-cabf535df342', data)
        # self.assertEqual(405, response.status_code, response.data)

    def test_patch_helios(self):
        pass
        # data = {
        #     "url": "invoices/8a8f1509-071f-416b-a222-33208fb25059",
        # }
        # response = self.client.put('/invoices/8a8f1509-071f-416b-a222-33208fb25059', data)
        # self.assertEqual(405, response.status_code, response.data)

    def test_patch_student_union(self):
        pass
        # data = {
        #     "url": "invoices/93f47e50-444e-450e-94b9-cabf535df342",
        # }
        #
        # response = self.client.put('/invoices/93f47e50-444e-450e-94b9-cabf535df342', data)
        # self.assertEqual(405, response.status_code, response.data)

    def test_post_helios(self):
        data = {
            "association": "/associations/rising-sun-alienship-helios",
            "profile": '/profiles/75b5beed-97c2-4b43-bf3f-6c7ba85b3a5c',
            "reason": 'For eating a banana',
            "price": 5.48
        }
        response = self.client.post('/invoices', data=data)
        self.assertEqual(response.status_code, 400, response.data)

    def test_post_student_union(self):
        data = {
            "association": "/associations/student-union",
            "profile": '/profiles/be3f26fb-97d1-4270-8367-3e8539b3484d',
            "reason": 'For eating a banana',
            "price": 5.48
        }
        response = self.client.post('/invoices', data=data)
        self.assertEqual(response.status_code, 201, response.data)

    def test_get_helios(self):
        # get a invoice user is in
        response = self.client.get("/invoices/17fc212d-c86d-484e-9ece-b67a0bb92a9b")
        self.assertEqual(response.status_code, 200, "Error: {}, Input: Invoice with the user for helios".format(response.data))
        # get a invoice user is not in
        response = self.client.get("/invoices/d41d58ca-89b8-43bd-b9bd-30de9a6c7248")
        self.assertEqual(response.status_code, 404, "Error: {}, Input: Invoice without the user for helios".format(response.data))

    def test_get_student_union(self):
        # get a invoice user is in
        response = self.client.get("/invoices/e3775f3f-7c0b-417b-821c-a16d474ddda5")
        self.assertEqual(response.status_code, 200, "Error: {}, Input: Invoice with the user for student union".format(response.data))
        # get a invoice user is not in
        response = self.client.get("/invoices/9f543d84-cc75-4d4f-b0b4-37d13a3869f5")
        self.assertEqual(response.status_code, 200, "Error: {}, Input: Invoice without the user for student union".format(response.data))

    def test_list(self):
        response = self.client.get('/invoices')
        self.assertEqual(response.status_code, 200, "Error: {}, Input: List of all invoices".format(response.data))
        self.assertEqual(response.data['count'], 4, msg="number of invoices doesn't match expected number of invoices")

    def test_delete_helios(self):
        # delete an invoice user is in
        response = self.client.delete("/invoices/17fc212d-c86d-484e-9ece-b67a0bb92a9b")
        self.assertEqual(response.status_code, 403, "Error: {}, Input: Invoice with the user for helios".format(response.data))
        # delete an invoice user is not in
        response = self.client.delete("/invoices/d41d58ca-89b8-43bd-b9bd-30de9a6c7248")
        self.assertEqual(response.status_code, 404, "Error: {}, Input: Invoice without the user for helios".format(response.data))

    def test_delete_student_union(self):
        # delete an invoice user is in
        response = self.client.delete("/invoices/e3775f3f-7c0b-417b-821c-a16d474ddda5")
        self.assertEqual(response.status_code, 204, "Error: {}, Input: Invoice with the user for student union".format(response.data))
        # delete an invoice user is not in
        response = self.client.delete("/invoices/9f543d84-cc75-4d4f-b0b4-37d13a3869f5")
        self.assertEqual(response.status_code, 204, "Error: {}, Input: Invoice without the user for student union".format(response.data))
        # delete a locked invoice
        response = self.client.delete("/invoices/89ac22dc-5a71-4fac-8af4-5844b7eb71a1")
        self.assertEqual(response.status_code, 400, "Error: {}, Input: Locked Invoice with the user for student union".format(response.data))

    def test_bulk_create(self):
        invoice_1 = {
            "association": "/associations/student-union",
            "profile": '/profiles/be3f26fb-97d1-4270-8367-3e8539b3484d',
            "reason": 'For eating a banana',
            "price": 5.48
        }
        invoice_2 = {
            "association": "/associations/student-union",
            "profile": '/profiles/be3f26fb-97d1-4270-8367-3e8539b3484d',
            "reason": 'For eating a banana',
            "price": 5.48
        }
        data = [invoice_1, invoice_2]
        response = self.client.post('/invoices', data=data, format="json")
        self.assertEqual(response.status_code, 201, response.data)

    def test_bulk_delete__without_header(self):
        number_of_invoices_SU_before = Invoice.objects.filter(association__slug="student-union").count()
        number_of_invoices_H_before = Invoice.objects.filter(association__slug="rising-sun-alienship-helios").count()
        self.assertEqual(number_of_invoices_SU_before, 3, "Number of invoices of student-union does not match")
        self.assertEqual(number_of_invoices_H_before, 2, "Number of invoices of Helios does not match")

        invoices = [invoice.slug for invoice in Invoice.objects.filter(association__slug="student-union").all()[0:1]]
        response = self.client.delete('/invoices?slug__in={}'.format(",".join(invoices)))
        self.assertEqual(response.status_code, 400, response.data)

        number_of_invoices_SU_after = Invoice.objects.filter(association__slug="student-union").count()
        number_of_invoices_H_after = Invoice.objects.filter(association__slug="rising-sun-alienship-helios").count()
        self.assertEqual(number_of_invoices_SU_after, number_of_invoices_SU_before, "Number of invoices of student-union does not match")
        self.assertEqual(number_of_invoices_H_after, number_of_invoices_H_before, "Number of invoices of Helios does not match")

    def test_bulk_delete__with_header(self):
        # check the number of invoices before hand
        number_of_invoices_SU_before = Invoice.objects.filter(association__slug="student-union").count()
        number_of_invoices_H_before = Invoice.objects.filter(association__slug="rising-sun-alienship-helios").count()
        self.assertEqual(number_of_invoices_SU_before, 3, "Number of invoices of student-union does not match")
        self.assertEqual(number_of_invoices_H_before, 2, "Number of invoices of Helios does not match")

        # Try remove all the invoices for an association
        invoices = [invoice.slug for invoice in Invoice.objects.filter(association__slug="student-union").all()]
        response = self.client.delete('/invoices?slug__in={}'.format(",".join(invoices)), **self.protection_headers)
        self.assertEqual(response.status_code, 400, response.data)
        self.assertEqual(response.data, {'locked': 'Query contains locked invoices; these cannot be removed.'})

        # check the number of invoices is still correct
        number_of_invoices_SU_before = Invoice.objects.filter(association__slug="student-union").count()
        number_of_invoices_H_before = Invoice.objects.filter(association__slug="rising-sun-alienship-helios").count()
        self.assertEqual(number_of_invoices_SU_before, 3, "Number of invoices of student-union does not match")
        self.assertEqual(number_of_invoices_H_before, 2, "Number of invoices of Helios does not match")

        # Try remove unlocked invoices
        invoices = ["e3775f3f-7c0b-417b-821c-a16d474ddda5", "9f543d84-cc75-4d4f-b0b4-37d13a3869f5"]
        response = self.client.delete('/invoices?slug__in={}'.format(",".join(invoices)), **self.protection_headers)
        self.assertEqual(response.status_code, 204, response.data)

        # check the number of invoices is correctly updated
        number_of_invoices_SU_after = Invoice.objects.filter(association__slug="student-union").count()
        number_of_invoices_H_after = Invoice.objects.filter(association__slug="rising-sun-alienship-helios").count()
        self.assertEqual(number_of_invoices_SU_after, number_of_invoices_SU_before - 2, "Number of invoices of student-union does not match")
        self.assertEqual(number_of_invoices_H_after, number_of_invoices_H_before, "Number of invoices of Helios does not match")
